package gregapi.data;

import static gregapi.data.CS.TICKS_PER_SMELT;
import static gregapi.data.CS.U;
import static gregapi.data.TD.Atomic.*;
import static gregapi.data.TD.Compounds.*;
import static gregapi.data.TD.ItemGenerator.*;
import static gregapi.data.TD.Processing.*;
import static gregapi.data.TD.Properties.*;
import static gregapi.render.TextureSet.*;
import gregapi.code.HashSetNoNulls;
import gregapi.oredict.OreDictConfigurationComponent;
import gregapi.oredict.OreDictItemData;
import gregapi.oredict.OreDictMaterial;
import gregapi.render.TextureSet;
import gregapi.util.OM;
import net.minecraft.enchantment.Enchantment;
import net.minecraft.stats.AchievementList;

/**
 * @author Gregorius Techneticies
 * 
 * List of all Materials. The Short Name is for ease of overview and stands for "MaTerial".
 * 
 * Note: I wrote those shortcuts not only because of overview Reasons. I have hit the 65536 Limit of the static initialiser multiple times by now.
 */
public class MT {
	/** This Set is for GregTech usage only, do not add your Materials to this! */
	public static final HashSetNoNulls<OreDictMaterial> ALL_MATERIALS_REGISTERED_HERE = new HashSetNoNulls<OreDictMaterial>();
	
	/** Making the Table a little bit more overviewable. DO NOT USE THESE FUNCTIONS YOURSELF!!! Use "OreDictMaterial.createMaterial(YOUR-ID-AS-SPECIFIED-IN-THE-ID-RANGES, OREDICT-NAME, LOCALISED-NAME)" */
	static OreDictMaterial unused		(String aNameOreDict) {return create(-1, aNameOreDict).add(UNUSED_MATERIAL);}
	static OreDictMaterial depricated	(String aNameOreDict) {return create(-1, aNameOreDict).add(UNUSED_MATERIAL);}
	static OreDictMaterial invalid		(String aNameOreDict) {return unused(aNameOreDict).add(INVALID_MATERIAL);}
	static OreDictMaterial create		(int aID, String aNameOreDict) {if (aID >= 10000) return null; OreDictMaterial rMaterial = OreDictMaterial.createMaterial(aID, aNameOreDict, aNameOreDict); ALL_MATERIALS_REGISTERED_HERE.add(rMaterial); return rMaterial;}
	static OreDictMaterial create		(int aID, String aNameOreDict, TextureSet[] aSets) {return create(aID, aNameOreDict).setTextures(aSets);}
	static OreDictMaterial create		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA) {return create(aID, aNameOreDict, aSets).setRGBa(aR, aG, aB, aA).add(aR==256?UNUSED_MATERIAL:null).hide(aR==256);}
	
	/** Making the Table a little bit more overviewable. DO NOT USE THESE FUNCTIONS YOURSELF!!! Use "OreDictMaterial.createMaterial(YOUR-ID-AS-SPECIFIED-IN-THE-ID-RANGES, OREDICT-NAME, LOCALISED-NAME)" */
	static OreDictMaterial element		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return create		(aID, aNameOreDict, aSets, aR, aG, aB, aA).setStats(aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter).add(ELEMENT);}
	static OreDictMaterial metal		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return element		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(METAL, SMITHABLE, MELTING, EXTRUDER);}
	static OreDictMaterial metalloid	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return element		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(METALLOID, SMITHABLE, MELTING, EXTRUDER);}
	static OreDictMaterial nonmetal		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return element		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(NONMETAL);}
	static OreDictMaterial diatomic		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return nonmetal	(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(DIATOMIC_NONMETAL);}
	static OreDictMaterial polyatomic	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return nonmetal	(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(POLYATOMIC_NONMETAL);}
	static OreDictMaterial noblegas		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return nonmetal	(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(NOBLE_GAS, GASSES);}
	static OreDictMaterial alkali		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metal		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(ALKALI_METAL);}
	static OreDictMaterial alkaline		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metal		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(ALKALINE_EARTH_METAL);}
	static OreDictMaterial lanthanide	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metal		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(LANTHANIDE);}
	static OreDictMaterial actinide		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metal		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(ACTINIDE);}
	static OreDictMaterial transmetal	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metal		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(TRANSITION_METAL);}
	static OreDictMaterial precmetal	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return transmetal	(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(PRECIOUS_METAL);}
	static OreDictMaterial noblemetal	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return precmetal	(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(NOBLE_METAL);}
	static OreDictMaterial refractmetal	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return transmetal	(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(REFRACTORY_METAL);}
	static OreDictMaterial platingroup	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return precmetal	(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(PLATINUM_GROUP);}
	static OreDictMaterial posttrans	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metal		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(POST_TRANSITION_METAL);}
	
	static OreDictMaterial element		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets)	{return create		(aID, aNameOreDict, aSets, 256, 256, 256, 255).setStats(aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter).add(ELEMENT);}
	static OreDictMaterial unknown		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons																							)	{return element		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, 1000, 3000, 0, SET_SHINY).hide().aspects(TC.RADIO, 1);}
	static OreDictMaterial metalloid	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets)	{return element		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, 256, 256, 256, 255).add(METALLOID, SMITHABLE, MELTING, EXTRUDER);}
	static OreDictMaterial polyatomic	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets)	{return nonmetal	(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, 256, 256, 256, 255).add(POLYATOMIC_NONMETAL);}
	static OreDictMaterial alkali		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets)	{return metal		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, 256, 256, 256, 255).add(ALKALI_METAL);}
	static OreDictMaterial alkaline		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets)	{return metal		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, 256, 256, 256, 255).add(ALKALINE_EARTH_METAL);}
	static OreDictMaterial lanthanide	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets)	{return metal		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, 256, 256, 256, 255).add(LANTHANIDE);}
	static OreDictMaterial actinide		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets)	{return metal		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, 256, 256, 256, 255).add(ACTINIDE);}
	static OreDictMaterial transmetal	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets)	{return metal		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, 256, 256, 256, 255).add(TRANSITION_METAL);}
	static OreDictMaterial posttrans	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets)	{return metal		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, 256, 256, 256, 255).add(POST_TRANSITION_METAL);}
	
	/** Making the Table a little bit more overviewable. DO NOT USE THESE FUNCTIONS YOURSELF!!! Use "OreDictMaterial.createMaterial(YOUR-ID-AS-SPECIFIED-IN-THE-ID-RANGES, OREDICT-NAME, LOCALISED-NAME)" */
	static OreDictMaterial gas			(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return create			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(G_CONTAINERS, CONTAINERS_GAS);}
	static OreDictMaterial gasdcmp		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return gas				(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(DECOMPOSABLE);}
	static OreDictMaterial gascent		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return gasdcmp			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(CENTRIFUGE);}
	static OreDictMaterial gaselec		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return gasdcmp			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(ELECTROLYSER);}
	static OreDictMaterial lqud			(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return create			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(G_CONTAINERS, CONTAINERS_FLUID);}
	static OreDictMaterial lquddcmp		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return lqud			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(DECOMPOSABLE);}
	static OreDictMaterial lqudcent		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return lquddcmp		(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(CENTRIFUGE);}
	static OreDictMaterial lqudelec		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return lquddcmp		(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(ELECTROLYSER);}
	static OreDictMaterial gaschem		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return create			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(G_CONTAINERS, CONTAINERS_GAS);}
	static OreDictMaterial gaschemdcmp	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return gaschem			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(DECOMPOSABLE);}
	static OreDictMaterial gaschemcent	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return gaschemdcmp		(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(CENTRIFUGE);}
	static OreDictMaterial gaschemelec	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return gaschemdcmp		(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(ELECTROLYSER);}
	static OreDictMaterial lqudchem		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return create			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(G_CONTAINERS, CONTAINERS_FLUID);}
	static OreDictMaterial lqudchemdcmp	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return lqudchem		(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(DECOMPOSABLE);}
	static OreDictMaterial lqudchemcent	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return lqudchemdcmp	(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(CENTRIFUGE);}
	static OreDictMaterial lqudchemelec	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return lqudchemdcmp	(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(ELECTROLYSER);}
	static OreDictMaterial gasacid		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return create			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(G_CONTAINERS, CONTAINERS_GAS, ACID);}
	static OreDictMaterial gasaciddcmp	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return gasacid			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(DECOMPOSABLE);}
	static OreDictMaterial gasacidcent	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return gasaciddcmp		(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(CENTRIFUGE);}
	static OreDictMaterial gasacidelec	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return gasaciddcmp		(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(ELECTROLYSER);}
	static OreDictMaterial lqudacid		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return create			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(G_CONTAINERS, CONTAINERS_FLUID, ACID);}
	static OreDictMaterial lqudaciddcmp	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return lqudacid		(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(DECOMPOSABLE);}
	static OreDictMaterial lqudacidcent	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return lqudaciddcmp	(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(CENTRIFUGE);}
	static OreDictMaterial lqudacidelec	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return lqudaciddcmp	(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(ELECTROLYSER);}
	static OreDictMaterial dust			(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return create			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(G_DUST);}
	static OreDictMaterial dustdcmp		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return dust			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(DECOMPOSABLE);}
	static OreDictMaterial dustcent		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return dustdcmp		(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(CENTRIFUGE);}
	static OreDictMaterial dustelec		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return dustdcmp		(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(ELECTROLYSER);}
	static OreDictMaterial mix			(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return create			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(DECOMPOSABLE, CENTRIFUGE);}
	static OreDictMaterial mixdust		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return dustcent		(aID, aNameOreDict, aSets, aR, aG, aB, aA);}
	static OreDictMaterial oredust		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return create			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(G_DUST_ORES);}
	static OreDictMaterial oredustdcmp	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return oredust			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(DECOMPOSABLE);}
	static OreDictMaterial oredustcent	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return oredustdcmp		(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(CENTRIFUGE);}
	static OreDictMaterial oredustelec	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return oredustdcmp		(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(ELECTROLYSER);}
	static OreDictMaterial metal		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return create			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(G_INGOT, SMITHABLE, MELTING, EXTRUDER);}
	static OreDictMaterial metalore		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metal			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(G_INGOT_ORES);}
	static OreDictMaterial metalmachine	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metal			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(G_INGOT_MACHINE);}
	static OreDictMaterial metalmachore	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metalmachine	(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(G_INGOT_MACHINE_ORES);}
	static OreDictMaterial alloy		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metal			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(ALLOY, DECOMPOSABLE);}
	static OreDictMaterial alloyore		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metalore		(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(ALLOY, DECOMPOSABLE);}
	static OreDictMaterial alloymachine	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metalmachine	(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(ALLOY, DECOMPOSABLE);}
	static OreDictMaterial alloymachore	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metalmachore	(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(ALLOY, DECOMPOSABLE);}
	static OreDictMaterial metalnd		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return create			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(G_INGOT_ND, SMITHABLE, MELTING, EXTRUDER);}
	static OreDictMaterial metalmachnd	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metalnd			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(G_INGOT_ND_MACHINE);}
	static OreDictMaterial alloynd		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metalnd			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(ALLOY, DECOMPOSABLE);}
	static OreDictMaterial alloymachnd	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metalmachnd		(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(ALLOY, DECOMPOSABLE);}
	
	/** Technical Material describing a Nullpointer */
	public static final OreDictMaterial NULL = create(-1, "NULL").setStatsElement(0,0,0,0,0).setTextures(SET_NONE).add(INVALID_MATERIAL, DONT_SHOW_THIS_COMPONENT);
	
	/** Material for empty Containers. */
	public static final OreDictMaterial Empty = create(0, "Empty").setStatsElement(0,0,0,0,0).setTextures(SET_NONE).add(EMPTY, AUTO_BLACKLIST, DONT_SHOW_THIS_COMPONENT).setOriginalMod(MD.MC);
	
	/** Some subatomic Particles and, this might be useful if Projectile Materials have to be determined and you shoot a Laser (Photons in that case) or something Radioactive. */
	public static final OreDictMaterial
	y		, Photon					=	y		= create(	1, "Photon"		).setStatsElement(0,0,0,0,0).setStatsEnergetic(0,0,0).setRGBa(255, 255, 255, 255).setTextures(SET_NONE).add(PARTICLE).hide(),
	v		, Neutrino					=	v		= create(	2, "Neutrino"	).setStatsElement(0,0,0,0,0).setStatsEnergetic(0,0,0).setRGBa(180, 180, 180,   0).setTextures(SET_NONE).add(PARTICLE).hide(),
	n		, Neutron					=	n		= create(	3, "Neutron"	).setStatsElement(0,0,1,0,0).setStatsEnergetic(0,0,0).setRGBa(128, 128, 128,   0).setTextures(SET_NONE).add(PARTICLE).hide(),
	p		, Proton					=	p		= create(	4, "Proton"		).setStatsElement(1,0,0,0,0).setStatsEnergetic(0,0,0).setRGBa(255,   0,   0,   0).setTextures(SET_NONE).add(PARTICLE).hide(),
	e		, Electron					=	e		= create(	5, "Electron"	).setStatsElement(0,1,0,0,0).setStatsEnergetic(0,0,0).setRGBa(  0,   0, 255,   0).setTextures(SET_NONE).add(PARTICLE).hide();
	
	/** Yes, I consider Magic as some kind of Element. I placed it at the Spot of "Anti-Empty", since Slot 0 is for the "Empty" Material, and "Anti-Empty" = "Magically-Full", or some crazy shit like that. Also Magic has negative Mass. */
	public static final OreDictMaterial
	Ma		, Magic						=	Ma		= create( 4000, "Magic"		).setStatsElement(0,0,0,-1,0).setStatsEnergetic(0,0,0).setTextures(SET_SHINY).add(ELEMENT, MAGICAL, UNBURNABLE).hide().setQuality(10.0F, 5120, 5);
	
	/**
	 * All the Elements and a few of their Isotopes.
	 * 
	 * in setStatsEnergetic I guessed some of the Values, because they were not listed in Wikipedia at that time.
	 * Those guessed Values are postfixed with an empty Space.
	 */
	public static final OreDictMaterial
	H		, Hydrogen					=	H		= diatomic		(   10,	"Hydrogen"				,   1,   0,    14,    20,  0.00008988, SET_DULL				,   0,   0, 255,  15).add(	CONTAINERS_GAS				, UUM												).aspects(	TC.AQUA, 1															).add(GASSES),
	D		, Deuterium					=	D		= diatomic		(   11,	"Deuterium"				,   1,   1,    14,    20,  0.00008988, SET_SHINY			, 255, 255,   0,  15).add(	CONTAINERS_GAS																	).aspects(	TC.AQUA, 2															).add(GASSES), H_2=D,
	T		, Tritium					=	T		= diatomic		(   12,	"Tritium"				,   1,   2,    14,    20,  0.00008988, SET_SHINY			, 255,   0,   0,  15).add(	CONTAINERS_GAS																	).aspects(	TC.AQUA, 3															).add(GASSES), H_3=T,
	He		, Helium					=	He		= noblegas		(   20,	"Helium"				,   2,   2,     1,     4,  0.0001785 , SET_SHINY			, 255, 255, 120,  15).add(	CONTAINERS_GAS				, FUSION, UUM										).aspects(	TC.AER, 2															),
	He_3	, Helium3					=	He_3	= noblegas		(   21,	"Helium-3"				,   2,   1,	    1,     4,  0.0001785 , SET_SHINY			, 255, 255, 140,  15).add(	CONTAINERS_GAS				, FUSION											).aspects(	TC.AER, 3															),
	Li		, Lithium					=	Li		= alkali		(   30,	"Lithium"				,   3,   4,   453,  1560,  0.534	 , SET_ROUGH			, 225, 220, 255, 255).add(	G_DUST_ORES					, FUSION, UUM										).aspects(	TC.VITREUS, 1, TC.POTENTIA, 2										).setFurnaceBurnTime(TICKS_PER_SMELT*40),
	Li_6	, Lithium6					=	Li_6	= alkali		(   31,	"Lithium-6"				,   3,   3,   453,  1560,  0.534	 , SET_ROUGH			, 230, 225, 255, 255).add(	G_DUST_ORES					, FUSION											).aspects(	TC.VITREUS, 1, TC.POTENTIA, 1, TC.RADIO, 1							).setFurnaceBurnTime(TICKS_PER_SMELT*40),
	Be		, Beryllium					=	Be		= alkaline		(   40,	"Beryllium"				,   4,   5,  1560,  2742,  1.85      , SET_METALLIC			, 100, 180, 100, 255).add(	G_INGOT_ORES				, FUSION, UUM										).aspects(	TC.METALLUM, 2, TC.LUCRUM, 1										).setQuality(14.0F, 64, 2).add(MORTAR),
	B		, Boron						=	B		= metalloid		(   50,	"Boron"					,   5,   5,  2349,  4200,  2.34      , SET_DULL				, 250, 250, 250, 255).add(	G_INGOT_ORES				, FUSION, UUM, ICOSAGEN								).aspects(	TC.METALLUM, 1, TC.VITREUS, 1, TC.ELECTRUM, 1						),
	C		, Carbon					=	C		= polyatomic	(   60,	"Carbon"				,   6,   6,  3800,  4300,  2.267     , SET_FINE				,  20,  20,  20, 255).add(	G_DUST_ORES					, FUSION, UUM, CRYSTALLOGEN							).aspects(	TC.VITREUS, 1, TC.IGNIS, 1											).add(FLAMMABLE, MELTING),
	C_13	, Carbon13					=	C_13	= polyatomic	(   61,	"Carbon-13"				,   6,   7,  3800,  4300,  2.267     , SET_FINE				,  25,  25,  25, 255).add(	G_DUST_ORES					, FUSION, CRYSTALLOGEN								).aspects(	TC.VITREUS, 1, TC.IGNIS, 1, TC.RADIO, 1								).add(FLAMMABLE, MELTING),
	C_14	, Carbon14					=	C_14	= polyatomic	(   62,	"Carbon-14"				,   6,   8,  3800,  4300,  2.267     , SET_FINE				,  30,  30,  30, 255).add(	G_DUST_ORES					, FUSION, CRYSTALLOGEN								).aspects(	TC.VITREUS, 1, TC.IGNIS, 1, TC.RADIO, 1								).add(FLAMMABLE, MELTING),
	N		, Nitrogen					=	N		= diatomic		(   70,	"Nitrogen"				,   7,   7,    63,    77,  0.0012506 , SET_DULL				,   0, 150, 200,  15).add(	CONTAINERS_GAS				, FUSION, UUM, PNICTOGEN							).aspects(	TC.AER, 1															).add(GASSES),
	O		, Oxygen					=	O		= diatomic		(   80,	"Oxygen"				,   8,   8,    54,    90,  0.001429  , SET_DULL				,   0, 100, 200,  15).add(	CONTAINERS_GAS				, FUSION, UUM, CHALCOGEN							).aspects(	TC.AER, 1															).add(GASSES),
	F		, Fluorine					=	F		= diatomic		(   90,	"Fluorine"				,   9,   9,    53,    85,  0.001696  , SET_DULL				,  64, 192,   0, 255).add(	G_CRYSTAL_ORES				, FUSION, UUM, HALOGEN								).aspects(	TC.PERDITIO, 2														).add(GASSES),
	Ne		, Neon						=	Ne		= noblegas		(  100,	"Neon"					,  10,  10,    24,    27,  0.0008999 , SET_SHINY			, 250, 180, 180,  15).add(	CONTAINERS_GAS				, FUSION, UUM										).aspects(	TC.AER, 3															),
	Na		, Sodium					=	Na		= alkali		(  110,	"Sodium"				,  11,  11,   370,  1156,  0.971	 , SET_ROUGH			,   0,   0, 150, 255).add(	G_CRYSTAL_ORES				, FUSION, UUM										).aspects(	TC.VITREUS, 2, TC.LUX, 1											).setFurnaceBurnTime(TICKS_PER_SMELT*20), Natrium=Na.addIdenticalNames("Natrium"),
	Mg		, Magnesium					=	Mg		= alkaline		(  120,	"Magnesium"				,  12,  12,   923,  1363,  1.738	 , SET_METALLIC			, 255, 200, 200, 255).add(	G_INGOT_ORES				, FUSION, UUM										).aspects(	TC.METALLUM, 2, TC.SANO, 1											).add(MOLTEN),
	Al		, Aluminium					=	Al		= posttrans		(  130,	"Aluminium"				,  13,  13,   933,  2792,  2.698	 , SET_METALLIC			, 128, 200, 240, 255).add(	G_INGOT_MACHINE_ORES		, FUSION, UUM, ICOSAGEN								).aspects(	TC.METALLUM, 2, TC.VOLATUS, 1										).setQuality(10.0F, 128, 2).add(WIRES, RAILS, MORTAR, MOLTEN), Aluminum=Al.addIdenticalNames("Aluminum", "NaturalAluminum"),
	Si		, Silicon					=	Si		= metalloid		(  140,	"Silicon"				,  14,  14,  1687,  3538,  2.3296    , SET_METALLIC			,  60,  60,  80, 255).add(	G_INGOT_ORES				, FUSION, UUM, CRYSTALLOGEN							).aspects(	TC.METALLUM, 2, TC.TENEBRAE, 1										).add(MOLTEN),
	P		, Phosphor					=	P		= polyatomic	(  150,	"Phosphor"				,  15,  15,   317,   550,  1.82      , SET_FINE				, 255, 255,   0, 255).add(	G_CRYSTAL_ORES				, FUSION, UUM, PNICTOGEN							).aspects(	TC.IGNIS, 2, TC.POTENTIA, 1											).add(FLAMMABLE, EXPLOSIVE),
	S		, Sulfur					=	S		= polyatomic	(  160,	"Sulfur"				,  16,  16,   388,   717,  2.067	 , SET_FINE				, 200, 200,   0, 255).add(	G_CRYSTAL_ORES				, FUSION, UUM, CHALCOGEN							).aspects(	TC.IGNIS, 1															).add(FLAMMABLE).setOriginalMod(MD.RC).addIdenticalNames("Sulphur"),
	Cl		, Chlorine					=	Cl		= diatomic		(  170,	"Chlorine"				,  17,  18,   171,   239,  0.003214  , SET_DULL				,   0, 240, 255, 255).add(	CONTAINERS_FLUID			, FUSION, UUM, HALOGEN								).aspects(	TC.AQUA, 2, TC.PANNUS, 1											).add(GASSES),
	Ar		, Argon						=	Ar		= noblegas		(  180,	"Argon"					,  18,  22,    83,    87,  0.0017837 , SET_SHINY			,   0, 255,   0,  15).add(	CONTAINERS_GAS				, FUSION, UUM										).aspects(	TC.AER, 3															),
	K		, Potassium					=	K		= alkali		(  190,	"Potassium"				,  19,  20,   336,  1032,  0.862	 , SET_ROUGH			, 250, 250, 250, 255).add(	G_CRYSTAL_ORES				, FUSION, UUM										).aspects(	TC.VITREUS, 1, TC.POTENTIA, 1										), Kalium=K.addIdenticalNames("Kalium"),
	Ca		, Calcium					=	Ca		= alkaline		(  200,	"Calcium"				,  20,  20,  1115,  1757,  1.54      , SET_METALLIC			, 255, 245, 245, 255).add(	G_CRYSTAL_ORES				, FUSION, UUM										).aspects(	TC.SANO, 1, TC.TUTAMEN, 1											),
	Sc		, Scandium					=	Sc		= transmetal	(  210,	"Scandium"				,  21,  24,  1814,  3109,  2.989	 , SET_METALLIC			                    ).add(	G_INGOT_ORES				, FUSION, UUM, SCANDIUM_GROUP						).aspects(	TC.METALLUM, 2, TC.GELUM, 1											),
	Ti		, Titanium					=	Ti		= refractmetal	(  220,	"Titanium"				,  22,  26,  1941,  3560,  4.54	     , SET_METALLIC			, 220, 160, 240, 255).add(	G_INGOT_MACHINE_ORES		, FUSION, UUM, TITANIUM_GROUP						).aspects(	TC.METALLUM, 2, TC.TUTAMEN, 1										).setQuality( 8.0F,2560, 3).add(RAILS, MOLTEN), Titan=Ti.addIdenticalNames("Titan"),
	V		, Vanadium					=	V		= refractmetal	(  230,	"Vanadium"				,  23,  28,  2183,  3680,  6.11	     , SET_METALLIC			,  50,  50,  50, 255).add(	G_INGOT_ORES				, FUSION, UUM, VANADIUM_GROUP						).aspects(	TC.METALLUM, 2, TC.ELECTRUM, 1										).add(MOLTEN),
	Cr		, Chromium					=	Cr		= refractmetal	(  240,	"Chromium"				,  24,  28,  2180,  2944,  7.15	     , SET_SHINY			, 255, 230, 230, 255).add(	G_INGOT_MACHINE_ORES		, FUSION, UUM, CHROMIUM_GROUP						).aspects(	TC.METALLUM, 2, TC.MACHINA, 1										).setQuality(11.0F, 256, 3).add(MOLTEN), Chrome=Cr.addIdenticalNames("Chrome"),
	Mn		, Manganese					=	Mn		= transmetal	(  250,	"Manganese"				,  25,  30,  1519,  2334,  7.44	     , SET_DULL				, 250, 250, 250, 255).add(	G_INGOT_ORES				, FUSION, UUM, MANGANESE_GROUP						).aspects(	TC.METALLUM, 3														).setQuality( 7.0F, 512, 2).add(MAGNETIC_PASSIVE, MOLTEN),
	Fe		, Iron						=	Fe		= transmetal	(  260,	"Iron"					,  26,  30,  1811,  3134,  7.874	 , SET_METALLIC			, 200, 200, 200, 255).add(	G_INGOT_MACHINE_ORES		, FUSION, UUM, IRON_GROUP							).aspects(	TC.METALLUM, 3														).setQuality( 6.0F, 256, 2).add(MOLTEN, WIRES, RAILS, MORTAR, MAGNETIC_PASSIVE).setRGBaLiquid(255, 64, 32, 255).setOriginalMod(MD.MC),
	Co		, Cobalt					=	Co		= transmetal	(  270,	"Cobalt"				,  27,  32,  1768,  3200,  8.86	     , SET_METALLIC			,  80,  80, 250, 255).add(	G_INGOT_ORES				, UUM, COBALT_GROUP									).aspects(	TC.METALLUM, 2, TC.INSTRUMENTUM, 1									).setQuality( 5.0F, 256, 3).add(WASHING_SODIUMPERSULFATE, MORTAR, MAGNETIC_PASSIVE, MOLTEN).setOriginalMod(MD.TiC),
	Co_60	, Cobalt60					=	Co_60	= transmetal	(  278,	"Cobalt-60"				,  27,  43,  1768,  3200,  8.86	     , SET_SHINY			,  90,  90, 250, 255).add(	G_INGOT_ORES				, COBALT_GROUP										).aspects(	TC.METALLUM, 1, TC.INSTRUMENTUM, 1, TC.RADIO, 1						).setQuality( 5.0F, 256, 3).add(WASHING_SODIUMPERSULFATE, MORTAR, MAGNETIC_PASSIVE),
	Ni		, Nickel					=	Ni		= transmetal	(  280,	"Nickel"				,  28,  30,  1728,  3186,  8.912	 , SET_METALLIC			, 250, 250, 200, 255).add(	G_INGOT_ORES				, UUM, NICKEL_GROUP									).aspects(	TC.METALLUM, 2, TC.IGNIS, 1											).setQuality( 6.0F,  64, 2).add(WASHING_SODIUMPERSULFATE, MORTAR, MAGNETIC_PASSIVE, MOLTEN).setOriginalMod(MD.TE),
	Cu		, Copper					=	Cu		= noblemetal	(  290,	"Copper"				,  29,  34,  1357,  2835,  8.96	     , SET_SHINY			, 255, 100,   0, 255).add(	G_INGOT_MACHINE_ORES		, UUM, COPPER_GROUP									).aspects(	TC.METALLUM, 2, TC.PERMUTATIO, 1									).setQuality( 4.0F,  64, 0).add(WASHING_SODIUMPERSULFATE, FURNACE, WIRES, MORTAR, MOLTEN).setOriginalMod(MD.IC2),
	Zn		, Zinc						=	Zn		= transmetal	(  300,	"Zinc"					,  30,  35,   692,  1180,  7.134	 , SET_METALLIC			, 250, 240, 240, 255).add(	G_INGOT_ORES				, UUM, ZINC_GROUP									).aspects(	TC.METALLUM, 2, TC.SANO, 1											).add(WASHING_SODIUMPERSULFATE, WASHING_MERCURY, FURNACE, MORTAR, MOLTEN).setOriginalMod(MD.GT),
	Ga		, Gallium					=	Ga		= posttrans		(  310,	"Gallium"				,  31,  39,   302,  2477,  5.907	 , SET_DULL				, 220, 220, 255, 255).add(	G_INGOT_ORES				, UUM, ICOSAGEN										).aspects(	TC.METALLUM, 2, TC.ELECTRUM, 1										),
	Ge		, Germanium					=	Ge		= metalloid		(  320,	"Germanium"				,  32,  40,  1211,  3106,  5.323	 , SET_SHINY			, 212, 212, 212, 255).add(	G_INGOT_ORES				, UUM, CRYSTALLOGEN									).aspects(	TC.METALLUM, 2, TC.ELECTRUM, 1										),
	As		, Arsenic					=	As		= metalloid		(  330,	"Arsenic"				,  33,  42,   887,  1090,  5.776	 , SET_SHINY			                    ).add(	G_INGOT_ORES				, UUM, PNICTOGEN									).aspects(	TC.METALLUM, 1, TC.VENENUM, 2										),
	Se		, Selenium					=	Se		= polyatomic	(  340,	"Selenium"				,  34,  45,   453,   958,  4.809	 , SET_DULL				                    ).add(	G_CRYSTAL_ORES				, UUM, CHALCOGEN									).aspects(	TC.VITREUS, 1, TC.SPIRITUS, 2										),
	Br		, Bromine					=	Br		= diatomic		(  350,	"Bromine"				,  35,  45,   265,   332,  3.122	 , SET_FLUID			,  80,  10,  10, 255).add(	CONTAINERS_FLUID			, UUM, HALOGEN										).aspects(	TC.METALLUM, 1, TC.AQUA, 1, TC.TEMPESTAS, 1							).add(LIQUID),
	Kr		, Krypton					=	Kr		= noblegas		(  360,	"Krypton"				,  36,  48,   115,   119,  0.003733  , SET_DIAMOND			, 128, 255, 128,  15).add(	CONTAINERS_GAS				, UUM												).aspects(	TC.AER, 3															),
	Rb		, Rubidium					=	Rb		= alkali		(  370,	"Rubidium"				,  37,  48,   312,   961,  1.532	 , SET_SHINY			, 240,  30,  30, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.VITREUS, 1										),
	Sr		, Strontium					=	Sr		= alkaline		(  380,	"Strontium"				,  38,  49,  1050,  1655,  2.64	     , SET_METALLIC			, 200, 200, 200, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.STRONTIO, 1										),
	Y		, Yttrium					=	Y		= transmetal	(  390,	"Yttrium"				,  39,  50,  1799,  3609,  4.469	 , SET_METALLIC			, 220, 250, 220, 255).add(	G_INGOT_ORES				, UUM, SCANDIUM_GROUP								).aspects(	TC.METALLUM, 3 														),
	Zr		, Zirconium					=	Zr		= refractmetal	(  400,	"Zirconium"				,  40,  51,  2128,  4682,  6.506	 , SET_DIAMOND			, 200, 255, 255, 127).add(	G_GEM_ORES_TRANSPARENT		, UUM, TITANIUM_GROUP								).aspects(	TC.METALLUM, 2, TC.VITREUS, 1										).add(CRYSTAL).setQuality(6.0F, 512, 3),
	Nb		, Niobium					=	Nb		= refractmetal	(  410,	"Niobium"				,  41,  53,  2750,  5017,  8.57	     , SET_METALLIC			, 190, 180, 200, 255).add(	G_INGOT_ORES				, UUM, VANADIUM_GROUP								).aspects(	TC.METALLUM, 2, TC.IGNIS, 1											),
	Mo		, Molybdenum				=	Mo		= refractmetal	(  420,	"Molybdenum"			,  42,  53,  2896,  4912, 10.22	     , SET_SHINY			, 180, 180, 220, 255).add(	G_INGOT_ORES				, UUM, CHROMIUM_GROUP								).aspects(	TC.METALLUM, 2, TC.INSTRUMENTUM, 1									).setQuality(7.0F, 512, 2),
	Tc		, Technetium				=	Tc		= transmetal	(  430,	"Technetium"			,  43,  55,  2430,  4538, 11.5	     , SET_SHINY			                    ).add(	G_INGOT_ORES				, MANGANESE_GROUP									).aspects(	TC.METALLUM, 2, TC.RADIO, 1											), Gregorium=Tc.addIdenticalNames("Gregorium"),
	Ru		, Ruthenium					=	Ru		= platingroup	(  440,	"Ruthenium"				,  44,  57,  2607,  4423, 12.37	     , SET_SHINY			, 155, 155, 155, 255).add(	G_INGOT_ORES				, UUM, IRON_GROUP									).aspects(	TC.METALLUM, 3 														),
	Rh		, Rhodium					=	Rh		= platingroup	(  450,	"Rhodium"				,  45,  58,  2237,  3968, 12.41	     , SET_SHINY			, 144, 144, 144, 255).add(	G_INGOT_ORES				, UUM, COBALT_GROUP									).aspects(	TC.METALLUM, 3 														),
	Pd		, Palladium					=	Pd		= platingroup	(  460,	"Palladium"				,  46,  60,  1828,  3236, 12.02	     , SET_SHINY			, 128, 128, 128, 255).add(	G_INGOT_MACHINE_ORES		, UUM, NICKEL_GROUP									).aspects(	TC.METALLUM, 3 														).setQuality(8.0F, 512, 2),
	Ag		, Silver					=	Ag		= noblemetal	(  470,	"Silver"				,  47,  60,  1234,  2435, 10.501	 , SET_SHINY			, 220, 220, 255, 255).add(	G_INGOT_MACHINE_ORES		, UUM, COPPER_GROUP									).aspects(	TC.METALLUM, 2, TC.LUCRUM, 1										).setQuality(10.0F, 64, 2).add(WIRES, RAILS, WASHING_MERCURY, MORTAR, MOLTEN).setOriginalMod(MD.RP),
	Cd		, Cadmium					=	Cd		= transmetal	(  480,	"Cadmium"				,  48,  64,   594,  1040,  8.69	     , SET_SHINY			,  50,  50,  60, 255).add(	G_INGOT_ORES				, UUM, ZINC_GROUP									).aspects(	TC.METALLUM, 1, TC.POTENTIA, 1, TC.VENENUM, 1						),
	In		, Indium					=	In		= posttrans		(  490,	"Indium"				,  49,  65,   429,  2345,  7.31	     , SET_SHINY			,  64,   0, 128, 255).add(	G_INGOT_ORES				, UUM, ICOSAGEN										).aspects(	TC.METALLUM, 3														),
	Sn		, Tin						=	Sn		= posttrans		(  500,	"Tin"					,  50,  68,   505,  2875,  7.287	 , SET_DULL				, 220, 220, 220, 255).add(	G_INGOT_MACHINE_ORES		, UUM, CRYSTALLOGEN									).aspects(	TC.METALLUM, 2, TC.VITREUS, 1										).add(SOLDERING_MATERIAL, FURNACE, WIRES, MORTAR, MOLTEN).setOriginalMod(MD.IC2),
	Sb		, Antimony					=	Sb		= metalloid		(  510,	"Antimony"				,  51,  70,   903,  1860,  6.685	 , SET_SHINY			, 220, 220, 240, 255).add(	G_INGOT_ORES				, UUM, PNICTOGEN									).aspects(	TC.METALLUM, 2, TC.AQUA, 1											).add(MORTAR, MOLTEN),
	Te		, Tellurium					=	Te		= metalloid		(  520,	"Tellurium"				,  52,  75,   722,  1261,  6.232	 , SET_SHINY			                    ).add(	G_INGOT_ORES				, UUM, CHALCOGEN									).aspects(	TC.METALLUM, 2, TC.ELECTRUM, 1										),
	I		, Iodine					=	I		= diatomic		(  530,	"Iodine"				,  53,  74,   386,   457,  4.93	     , SET_DULL				, 255, 240, 240, 255).add(	G_CRYSTAL_ORES				, UUM, HALOGEN										).aspects(	TC.VITREUS, 2, TC.TEMPESTAS, 1										),
	Xe		, Xenon						=	Xe		= noblegas		(  540,	"Xenon"					,  54,  77,   161,   165,  0.005887  , SET_DULL				,   0, 255, 255,  15).add(	CONTAINERS_GAS				, UUM												).aspects(	TC.AER, 3															),
	Cs		, Caesium					=	Cs		= alkali		(  550,	"Caesium"				,  55,  77,   301,   944,  1.873	 , SET_SHINY			                    ).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Ba		, Barium					=	Ba		= alkaline		(  560,	"Barium"				,  56,  81,  1000,  2170,  3.594	 , SET_METALLIC			                    ).add(	G_INGOT_ORES				, UUM												).aspects(	TC.VINCULUM, 3														),
	La		, Lantanium					=	La		= lanthanide	(  570,	"Lantanium"				,  57,  81,  1193,  3737,  6.145	 , SET_METALLIC			                    ).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Ce		, Cerium					=	Ce		= lanthanide	(  580,	"Cerium"				,  58,  82,  1068,  3716,  6.77      , SET_SHINY			, 255, 255, 190, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Pr		, Praseodymium				=	Pr		= lanthanide	(  590,	"Praseodymium"			,  59,  81,  1208,  3793,  6.773	 , SET_METALLIC			                    ).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Nd		, Neodymium					=	Nd		= lanthanide	(  600,	"Neodymium"				,  60,  84,  1297,  3347,  7.007	 , SET_SHINY			, 100, 100, 100, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.MAGNETO, 2										).add(MAGNETIC_PASSIVE, MOLTEN).setQuality(7.0F, 512, 2),
	Pm		, Promethium				=	Pm		= lanthanide	(  610,	"Promethium"			,  61,  83,  1315,  3273,  7.26	     , SET_SHINY			                    ).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Sm		, Samarium					=	Sm		= lanthanide	(  620,	"Samarium"				,  62,  88,  1345,  2067,  7.52	     , SET_METALLIC			                    ).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Eu		, Europium					=	Eu		= lanthanide	(  630,	"Europium"				,  63,  88,  1099,  1802,  5.243	 , SET_METALLIC			                    ).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Gd		, Gadolinium				=	Gd		= lanthanide	(  640,	"Gadolinium"			,  64,  93,  1585,  3546,  7.895	 , SET_METALLIC			                    ).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Tb		, Terbium					=	Tb		= lanthanide	(  650,	"Terbium"				,  65,  93,  1629,  3503,  8.229	 , SET_METALLIC			                    ).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Dy		, Dysprosium				=	Dy		= lanthanide	(  660,	"Dysprosium"			,  66,  96,  1680,  2840,  8.55	     , SET_METALLIC			                    ).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Ho		, Holmium					=	Ho		= lanthanide	(  670,	"Holmium"				,  67,  97,  1734,  2993,  8.795	 , SET_METALLIC			                    ).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Er		, Erbium					=	Er		= lanthanide	(  680,	"Erbium"				,  68,  99,  1802,  3141,  9.066	 , SET_METALLIC			                    ).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Tm		, Thulium					=	Tm		= lanthanide	(  690,	"Thulium"				,  69,  99,  1818,  2223,  9.321	 , SET_METALLIC			                    ).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Yb		, Ytterbium					=	Yb		= lanthanide	(  700,	"Ytterbium"				,  70, 103,  1097,  1469,  6.965	 , SET_METALLIC			                    ).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Lu		, Lutetium					=	Lu		= lanthanide	(  710,	"Lutetium"				,  71, 103,  1925,  3675,  9.84      , SET_METALLIC			                    ).add(	G_INGOT_ORES				, UUM, SCANDIUM_GROUP								).aspects(	TC.METALLUM, 3														),
	Hf		, Hafnium					=	Hf		= refractmetal	(  720,	"Hafnium"				,  72, 106,  2506,  4876, 13.31      , SET_METALLIC			, 140, 140, 150, 255).add(	G_INGOT_ORES				, UUM, TITANIUM_GROUP								).aspects(	TC.METALLUM, 3														).add(FLAMMABLE, EXPLOSIVE),
	Ta		, Tantalum					=	Ta		= refractmetal	(  730,	"Tantalum"				,  73, 107,  3290,  5731, 16.654	 , SET_METALLIC			, 120, 120, 140, 255).add(	G_INGOT_ORES				, UUM, VANADIUM_GROUP								).aspects(	TC.METALLUM, 2, TC.VINCULUM, 1										),
	W		, Tungsten					=	W		= refractmetal	(  740,	"Tungsten"				,  74, 109,  3695,  5828, 19.25	     , SET_METALLIC			,  50,  50,  50, 255).add(	G_INGOT_MACHINE_ORES		, UUM, CHROMIUM_GROUP								).aspects(	TC.METALLUM, 3, TC.TUTAMEN, 1										).setQuality(8.0F, 5120, 3).add(MOLTEN, WIRES, RAILS).setOriginalMod(MD.RP), Wolframium=W.addIdenticalNames("Wolframium"), Wolfram=W.addIdenticalNames("Wolfram"),
	Re		, Rhenium					=	Re		= precmetal		(  750,	"Rhenium"				,  75, 111,  3459,  5869, 21.02	     , SET_SHINY			, 255, 255, 200, 255).add(	G_INGOT_ORES				, UUM, MANGANESE_GROUP								).aspects(	TC.METALLUM, 3														),
	Os		, Osmium					=	Os		= platingroup	(  760,	"OsmiumElemental"		,  76, 114,  3306,  5285, 22.61	     , SET_METALLIC			,  50,  50, 255, 255).add(	G_INGOT_MACHINE_ORES		, UUM, IRON_GROUP									).aspects(	TC.METALLUM, 2, TC.MACHINA, 1, TC.NEBRISUM, 1						).setQuality(16.0F, 1280, 4).add(WIRES, MOLTEN).setLocal("Osmium"),
	Ir		, Iridium					=	Ir		= platingroup	(  770,	"Iridium"				,  77, 115,  2719,  4701, 22.56	     , SET_DULL				, 240, 240, 245, 255).add(	G_INGOT_MACHINE_ORES		, UUM, COBALT_GROUP									).aspects(	TC.METALLUM, 2, TC.MACHINA, 1										).setQuality(6.0F, 5120, 4).add(MOLTEN).setRGBaLiquid(255, 128, 200, 255).setOriginalMod(MD.IC2),
	Pt		, Platinum					=	Pt		= platingroup	(  780,	"Platinum"				,  78, 117,  2041,  4098, 21.46	     , SET_SHINY			, 100, 180, 250, 255).add(	G_INGOT_MACHINE_ORES		, UUM, NICKEL_GROUP									).aspects(	TC.METALLUM, 2, TC.NEBRISUM, 1										).setQuality(15.0F, 64, 2).add(WASHING_MERCURY, WIRES, RAILS, MORTAR, MOLTEN).setOriginalMod(MD.TE),
	Au		, Gold						=	Au		= noblemetal	(  790,	"Gold"					,  79, 117,  1337,  3129, 19.282	 , SET_SHINY			, 255, 230,  80, 255).add(	G_INGOT_MACHINE_ORES		, UUM, COPPER_GROUP									).aspects(	TC.METALLUM, 2, TC.LUCRUM, 2										).setQuality(12.5F, 64, 2).add(WASHING_MERCURY, WIRES, RAILS, MORTAR, MOLTEN).setOriginalMod(MD.MC),
	Hg		, Mercury					=	Hg		= precmetal		(  800,	"Mercury"				,  80, 120,   234,   629, 13.5336    , SET_SHINY			, 230, 220, 220, 255).add(	G_INGOT_ORES				, UUM, ZINC_GROUP									).aspects(	TC.METALLUM, 1, TC.AQUA, 1, TC.VENENUM, 1							).add(FURNACE, MORTAR), Quicksilver=Hg.addIdenticalNames("Quicksilver").addIdenticalNames("QuickSilver").setOriginalMod(MD.TC),
	Tl		, Thallium					=	Tl		= posttrans		(  810,	"Thallium"				,  81, 123,   577,  1746, 11.85	     , SET_METALLIC			                    ).add(	G_INGOT_ORES				, UUM, ICOSAGEN										).aspects(	TC.METALLUM, 3														),
	Pb		, Lead						=	Pb		= posttrans		(  820,	"Lead"					,  82, 125,   600,  2022, 11.342     , SET_DULL				, 140, 100, 140, 255).add(	G_INGOT_MACHINE_ORES		, UUM, CRYSTALLOGEN									).aspects(	TC.METALLUM, 2, TC.ORDO, 1											).setQuality(8.0F, 64, 1).add(SOLDERING_MATERIAL, SOLDERING_MATERIAL_BAD, FURNACE, WIRES, MORTAR, MOLTEN).setOriginalMod(MD.TE),
	Bi		, Bismuth					=	Bi		= posttrans		(  830,	"Bismuth"				,  83, 125,   544,  1837,  9.807     , SET_METALLIC			, 100, 160, 160, 255).add(	G_INGOT_MACHINE_ORES		, PNICTOGEN											).aspects(	TC.METALLUM, 2, TC.MAGNETO, 1										).setQuality(6.0F, 64, 1).add(FURNACE, MORTAR, MAGNETIC_PASSIVE, MOLTEN).setOriginalMod(MD.TFC),
	Po		, Polonium					=	Po		= posttrans		(  840,	"Polonium"				,  84, 124,   527,  1235,  9.32	     , SET_METALLIC			                    ).add(	G_INGOT_ORES				, CHALCOGEN											).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	At		, Astatine					=	At		= metalloid		(  850,	"Astatine"				,  85, 124,   575,   610,  7.0	     , SET_METALLIC			                    ).add(	G_INGOT_ORES				, HALOGEN											).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	Rn		, Radon						=	Rn		= noblegas		(  860,	"Radon"					,  86, 134,   202,   211,  0.00973   , SET_DULL				, 255,   0, 255,  15).add(	CONTAINERS_GAS																	).aspects(	TC.AER, 2, TC.RADIO, 1												),
	Fr		, Francium					=	Fr		= alkali		(  870,	"Francium"				,  87, 134,   300,   950,  1.87      , SET_METALLIC			                    ).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	Ra		, Radium					=	Ra		= alkaline		(  880,	"Radium"				,  88, 136,   973,  2010,  5.5	     , SET_METALLIC			, 255, 255, 205, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	Ac		, Actinium					=	Ac		= actinide		(  890,	"Actinium"				,  89, 136,  1323,  3471, 10.07      , SET_METALLIC			                    ).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	Th		, Thorium					=	Th		= actinide		(  900,	"Thorium"				,  90, 140,  2115,  5061, 11.72      , SET_SHINY			,   0,  30,   0, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											).setQuality(6.0F, 512, 2),
	Pa		, Protactinium				=	Pa		= actinide		(  910,	"Protactinium"			,  91, 138,  1841,  4300, 15.37      , SET_METALLIC			                    ).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	U_238	, Uranium					=	U_238	= actinide		(  920,	"Uranium"				,  92, 146,  1405,  4404, 18.95      , SET_METALLIC			,  50, 240,  50, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											).setQuality(6.0F, 512, 3).setOriginalMod(MD.IC2), Uranium238=U_238.addIdenticalNames("Uranium238"), Uran=U_238.addIdenticalNames("Uran"), // No, I could not make a Variable named "U" here. That would collide with the Material Unit.
	U_235	, Uranium235				=	U_235	= actinide		(  921,	"Uranium-235"			,  92, 143,  1405,  4404, 18.95      , SET_SHINY			,  70, 250,  70, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											).setQuality(6.0F, 512, 3).setOriginalMod(MD.IC2),
	Np		, Neptunium					=	Np		= actinide		(  930,	"Neptunium"				,  93, 144,   917,  4273, 20.45      , SET_DULL				                    ).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	Pu		, Plutonium					=	Pu		= actinide		(  940,	"Plutonium"				,  94, 150,   912,  3501, 19.84      , SET_METALLIC			, 240,  50,  50, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											).setQuality(6.0F, 512, 3).setOriginalMod(MD.IC2), Plutonium244=Pu.addIdenticalNames("Plutonium244"),
	Pu_241	, Plutonium241				=	Pu_241	= actinide		(  943,	"Plutonium-241"			,  94, 147,   912,  3501, 19.84      , SET_SHINY			, 245,  70,  70, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 3											).setQuality(6.0F, 512, 3),
	Pu_243	, Plutonium243				=	Pu_243	= actinide		(  945,	"Plutonium-243"			,  94, 149,   912,  3501, 19.84      , SET_SHINY			, 250,  70,  70, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											).setQuality(6.0F, 512, 3),
	Am		, Americium					=	Am		= actinide		(  950,	"Americium"				,  95, 150,  1449,  2880, 13.69      , SET_METALLIC			, 200, 200, 200, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											).setQuality(4.0F, 256, 2),
	Am_241	, Americium241				=	Am_241	= actinide		(  951,	"Americium-241"			,  95, 146,  1449,  2880, 13.69      , SET_SHINY			, 210, 210, 210, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 3											).setQuality(4.0F, 256, 2),
	Cm		, Curium					=	Cm		= actinide		(  960,	"Curium"				,  96, 153,  1613,  3383, 13.51      , SET_METALLIC			                    ).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	Bk		, Berkelium					=	Bk		= actinide		(  970,	"Berkelium"				,  97, 152,  1259,  2900, 14.79      , SET_METALLIC			                    ).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	Cf		, Californium				=	Cf		= actinide		(  980,	"Californium"			,  98, 153,  1173,  1743, 15.1	     , SET_METALLIC			                    ).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	Es		, Einsteinium				=	Es		= actinide		(  990,	"Einsteinium"			,  99, 153,  1133,  1269,  8.84      , SET_METALLIC			                    ).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	Fm		, Fermium					=	Fm		= actinide		( 1000,	"Fermium"				, 100, 157,  1125, 3000 ,		    0, SET_METALLIC			                    ).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Md		, Mendelevium				=	Md		= actinide		( 1010,	"Mendelevium"			, 101, 157,  1100, 3000 ,		    0, SET_METALLIC			                    ).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	No		, Nobelium					=	No		= actinide		( 1020,	"Nobelium"				, 102, 157,  1100, 3000 ,		    0, SET_METALLIC			                    ).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Lr		, Lawrencium				=	Lr		= actinide		( 1030,	"Lawrencium"			, 103, 159,  1900, 3000 ,		    0, SET_METALLIC			                    ).add(	G_INGOT_ORES				, SCANDIUM_GROUP									).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Rf		, Rutherfordium				=	Rf		= transmetal	( 1040,	"Rutherfordium"			, 104, 161,  2400,  5800, 23.2	     , SET_METALLIC			                    ).add(	G_INGOT_ORES				, TITANIUM_GROUP									).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Db		, Dubnium					=	Db		= transmetal	( 1050,	"Dubnium"				, 105, 163, 1000 , 3000 , 29.3	     , SET_METALLIC			                    ).add(	G_INGOT_ORES				, VANADIUM_GROUP									).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Sg		, Seaborgium				=	Sg		= transmetal	( 1060,	"Seaborgium"			, 106, 165, 1000 , 3000 , 35.0	     , SET_METALLIC			                    ).add(	G_INGOT_ORES				, CHROMIUM_GROUP									).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Bh		, Bohrium					=	Bh		= transmetal	( 1070,	"Bohrium"				, 107, 163, 1000 , 3000 , 37.1	     , SET_METALLIC			                    ).add(	G_INGOT_ORES				, MANGANESE_GROUP									).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Hs		, Hassium					=	Hs		= transmetal	( 1080,	"Hassium"				, 108, 169, 1000 , 3000 , 40.7	     , SET_METALLIC			                    ).add(	G_INGOT_ORES				, IRON_GROUP										).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Mt		, Meitnerium				=	Mt		= element		( 1090,	"Meitnerium"			, 109, 167, 1000 , 3000 , 37.4	     , SET_METALLIC			                    ).add(	G_INGOT_ORES				, COBALT_GROUP										).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Ds		, Darmstadtium				=	Ds		= element		( 1100,	"Darmstadtium"			, 110, 171, 1000 , 3000 , 34.8	     , SET_METALLIC			                    ).add(	G_INGOT_ORES				, NICKEL_GROUP										).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Rg		, Roentgenium				=	Rg		= element		( 1110,	"Roentgenium"			, 111, 169, 1000 , 3000 , 28.7	     , SET_METALLIC			                    ).add(	G_INGOT_ORES				, COPPER_GROUP										).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Cn		, Copernicium				=	Cn		= transmetal	( 1120,	"Copernicium"			, 112, 173,  150 ,   357, 23.7	     , SET_METALLIC			                    ).add(	G_INGOT_ORES				, ZINC_GROUP										).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Nh		, Nihonium					=	Nh		= element		( 1130,	"Nihonium"				, 113, 171,   700,  1400, 16.0	     , SET_METALLIC			                    ).add(	G_INGOT_ORES				, ICOSAGEN											).aspects(	TC.METALLUM, 1, TC.RADIO, 2											), Uut = Nh, Ununtrium = Nh.addIdenticalNames("Ununtrium"),
	Fl		, Flerovium					=	Fl		= posttrans		( 1140,	"Flerovium"				, 114, 175,   340,   420, 14.0	     , SET_METALLIC			                    ).add(	G_INGOT_ORES				, CRYSTALLOGEN										).aspects(	TC.METALLUM, 1, TC.RADIO, 3											),
	Fl_298	, Flerovium298				=	Fl_298	= posttrans		( 1148,	"Flerovium-298"			, 114, 184,   340,   420, 14.0	     , SET_SHINY			                    ).add(	G_INGOT_ORES				, CRYSTALLOGEN										).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Mc		, Moscovium					=	Mc		= element		( 1150,	"Moscovium"				, 115, 174,   700,  1400, 13.5	     , SET_METALLIC			                    ).add(	G_INGOT_ORES				, PNICTOGEN											).aspects(	TC.METALLUM, 1, TC.RADIO, 2											), Uup = Mc, Ununpentium = Mc.addIdenticalNames("Ununpentium"),
	Lv		, Livermorium				=	Lv		= element		( 1160,	"Livermorium"			, 116, 177,   708,  1085, 12.9	     , SET_METALLIC			                    ).add(	G_INGOT_ORES				, CHALCOGEN											).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Fa		, Farnsium					=	Fa		= element		( 1170,	"Farnsium"				, 117, 177,   673,   823,  7.2	     , SET_SHINY			                    ).add(	G_INGOT_ORES				, HALOGEN											).aspects(	TC.METALLUM, 1, TC.RADIO, 2											), Uus = Fa, Ununseptium = Fa.addIdenticalNames("Ununseptium"), Ts = Fa, Tennessine = Fa.addIdenticalNames("Tennessine"),
	Og		, Oganesson					=	Og		= element		( 1180,	"Oganesson"				, 118, 176,   258,   263,  5.0	     , SET_METALLIC			                    ).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											), Uuo = Og, Ununoctium = Og.addIdenticalNames("Ununoctium"),
	Uue		, Ununennium				=	Uue		= element		( 1190,	"Ununennium"			, 119, 178,   290,   903,		    0, SET_METALLIC			                    ).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Ubn		, Unbinilium				=	Ubn		= element		( 1200,	"Unbinilium"			, 120, 180,   953,  1973,		    0, SET_METALLIC			                    ).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	
	// The following Data is in no way reliable, but it is there in case someone wants to use it.
	Ubu		, Unbiunium					=	Ubu		= unknown		( 1210,	"Unbiunium"				, 121, 182),
	Ubb		, Unbibium					=	Ubb		= unknown		( 1220,	"Unbibium"				, 122, 184),
	Ubt		, Unbitrium					=	Ubt		= unknown		( 1230,	"Unbitrium"				, 123, 186),
	Ubq		, Unbiquadium				=	Ubq		= unknown		( 1240,	"Unbiquadium"			, 124, 188),
	Tn		, Tritanium					=	Tn		= element		( 1250,	"TritaniumElemental"	, 125, 198,  1500,  2500, 25.0		 , SET_DULL				,  55, 155, 155, 255).add(	G_DUST_ORES, UUM, MELTING														).aspects(TC.METALLUM, 2, TC.TUTAMEN, 2, TC.VITREUS, 1).setLocal("Elemental Tritanium"), Ubp = Tn, Unbipentium = Tn.addIdenticalNames("Unbipentium"),
	Ke		, Trinium					=	Ke		= element		( 1260,	"Trinium"				, 126, 192,  2645,  4523,  1.06874	 , SET_METALLIC			, 234, 234, 234, 255).add(	G_INGOT_MACHINE_ORES, UUM, MELTING, BRITTLE										).aspects(TC.METALLUM, 3, TC.TUTAMEN, 1, TC.VITREUS, 1).setQuality(12.0F, 2560, 4), Ubh = Ke, Unbihexium = Ke.addIdenticalNames("Unbihexium"),
	Ubs		, Unbiseptium				=	Ubs		= unknown		( 1270,	"Unbiseptium"			, 127, 194),
	Ubo		, Unbioctium				=	Ubo		= unknown		( 1280,	"Unbioctium"			, 128, 196),
	Ube		, Unbiennium				=	Ube		= unknown		( 1290,	"Unbiennium"			, 129, 198),
	Utn		, Untrinilium				=	Utn		= unknown		( 1300,	"Untrinilium"			, 130, 200),
	Utu		, Untriunium				=	Utu		= unknown		( 1310,	"Untriunium"			, 131, 203),
	Utb		, Untribium					=	Utb		= unknown		( 1320,	"Untribium"				, 132, 206),
	Utt		, Untritrium				=	Utt		= unknown		( 1330,	"Untritrium"			, 133, 209),
	Utq		, Untriquadium				=	Utq		= unknown		( 1340,	"Untriquadium"			, 134, 212),
	Utp		, Untripentium				=	Utp		= unknown		( 1350,	"Untripentium"			, 135, 215),
	Uth		, Untrihexium				=	Uth		= unknown		( 1360,	"Untrihexium"			, 136, 218),
	Uts		, Untriseptium				=	Uts		= unknown		( 1370,	"Untriseptium"			, 137, 221),
	Uto		, Untrioctium				=	Uto		= unknown		( 1380,	"Untrioctium"			, 138, 224),
	Ute		, Untriennium				=	Ute		= unknown		( 1390,	"Untriennium"			, 139, 227),
	Uqn		, Unquadnilium				=	Uqn		= unknown		( 1400,	"Unquadnilium"			, 140, 230),
	Uqu		, Unquadunium				=	Uqu		= unknown		( 1410,	"Unquadunium"			, 141, 233),
	Uqb		, Unquadbium				=	Uqb		= unknown		( 1420,	"Unquadbium"			, 142, 236),
	Uqt		, Unquadtrium				=	Uqt		= unknown		( 1430,	"Unquadtrium"			, 143, 239),
	Uqq		, Unquadquadium				=	Uqq		= unknown		( 1440,	"Unquadquadium"			, 144, 242),
	Dn		, Duranium					=	Dn		= element		( 1450,	"DuraniumElemental"		, 145, 190,  1200,  2100, 20.0	     , SET_DULL				,  75, 175, 175, 255).add(	G_DUST_ORES, UUM, MELTING														).aspects(TC.METALLUM, 2, TC.TUTAMEN, 1).setLocal("Elemental Duranium"), Uqp = Dn, Unquadpentium = Dn.addIdenticalNames("Unquadpentium"), // material refined from dolamide ore
	Uqh		, Unquadhexium				=	Uqh		= unknown		( 1460,	"Unquadhexium"			, 146, 248),
	Uqs		, Unquadseptium				=	Uqs		= unknown		( 1470,	"Unquadseptium"			, 147, 251),
	Uqo		, Unquadoctium				=	Uqo		= unknown		( 1480,	"Unquadoctium"			, 148, 254),
	Uqe		, Unquadennium				=	Uqe		= unknown		( 1490,	"Unquadennium"			, 149, 257),
	Upn		, Unpentnilium				=	Upn		= unknown		( 1500,	"Unpentnilium"			, 150, 260),
	Upu		, Unpentunium				=	Upu		= unknown		( 1510,	"Unpentunium"			, 151, 263),
	Vb		, Vibranium					=	Vb		= element		( 1520,	"Vibranium"				, 152, 266,  4852,  9415,  3.23978365, SET_EMERALD			, 200, 128, 255, 100).add(	G_GEM_ORES_TRANSPARENT, GLOWING													).aspects(TC.VITREUS, 10, TC.SENSUS, 10).visDefault(), Upb = Vb, Unpentbium = Vb.addIdenticalNames("Unpentbium").setQuality(1000.0F, 512, 15),
	Upt		, Unpenttrium				=	Upt		= unknown		( 1530,	"Unpenttrium"			, 153, 269),
	Upq		, Unpentquadium				=	Upq		= unknown		( 1540,	"Unpentquadium"			, 154, 272),
	Upp		, Unpentpentium				=	Upp		= unknown		( 1550,	"Unpentpentium"			, 155, 276),
	Uph		, Unpenthexium				=	Uph		= unknown		( 1560,	"Unpenthexium"			, 156, 280),
	Ups		, Unpentseptium				=	Ups		= unknown		( 1570,	"Unpentseptium"			, 157, 284),
	Upo		, Unpentoctium				=	Upo		= unknown		( 1580,	"Unpentoctium"			, 158, 288),
	Upe		, Unpentennium				=	Upe		= unknown		( 1590,	"Unpentennium"			, 159, 292),
	Uhn		, Unhexnilium				=	Uhn		= unknown		( 1600,	"Unhexnilium"			, 160, 296),
	Uhu		, Unhexunium				=	Uhu		= unknown		( 1610,	"Unhexunium"			, 161, 300),
	Uhb		, Unhexbium					=	Uhb		= unknown		( 1620,	"Unhexbium"				, 162, 304),
	Uht		, Unhextrium				=	Uht		= unknown		( 1630,	"Unhextrium"			, 163, 308),
	Uhq		, Unhexquadium				=	Uhq		= unknown		( 1640,	"Unhexquadium"			, 164, 312),
	Uhp		, Unhexpentium				=	Uhp		= unknown		( 1650,	"Unhexpentium"			, 165, 316),
	Uhh		, Unhexhexium				=	Uhh		= unknown		( 1660,	"Unhexhexium"			, 166, 320),
	Uhs		, Unhexseptium				=	Uhs		= unknown		( 1670,	"Unhexseptium"			, 167, 324),
	Uho		, Unhexoctium				=	Uho		= unknown		( 1680,	"Unhexoctium"			, 168, 328),
	Uhe		, Unhexennium				=	Uhe		= unknown		( 1690,	"Unhexennium"			, 169, 332),
	Usn		, Unseptnilium				=	Usn		= unknown		( 1700,	"Unseptnilium"			, 170, 336),
	Usu		, Unseptunium				=	Usu		= unknown		( 1710,	"Unseptunium"			, 171, 340),
	Usb		, Unseptbium				=	Usb		= unknown		( 1720,	"Unseptbium"			, 172, 344),
	Ust		, Unsepttrium				=	Ust		= unknown		( 1730,	"Unsepttrium"			, 173, 348),
	Nq		, Naquadah					=	Nq		= element		( 1740,	"Naquadah"				, 174, 352,   500,  2000, 21.0	     , SET_METALLIC			,  50,  50,  50, 255).add(	G_INGOT_MACHINE_ORES, UUM, SMITHABLE, MELTING, EXTRUDER							).aspects(TC.METALLUM, 3, TC.RADIO, 1, TC.NEBRISUM, 1).setRGBaLiquid(  0, 255,   0, 255), Usq = Nq, Unseptquadium = Nq.addIdenticalNames("Unseptquadium").setQuality(6.0F, 1280, 4),
	Nq_528	, NaquadahEnriched			=	Nq_528	= element		( 1741,	"Naquadah-Enriched"		, 174, 354,   500,  2000, 22.0	     , SET_METALLIC			,  60,  60,  60, 255).add(	G_INGOT_ORES, SMITHABLE, MELTING, EXTRUDER, EXPLOSIVE							).aspects(TC.METALLUM, 3, TC.RADIO, 2, TC.NEBRISUM, 2).setRGBaLiquid( 64, 255,  64, 255).setLocal("Enriched Naquadah").setQuality(6.0F, 1280, 4),
	Nq_522	, Naquadria					=	Nq_522	= element		( 1742,	"Naquadria"				, 174, 348,   500,  2000, 20.0	     , SET_SHINY			,  30,  30,  30, 255).add(	G_INGOT_ORES, SMITHABLE, MELTING, EXTRUDER, EXPLOSIVE							).aspects(TC.METALLUM, 4, TC.RADIO, 3, TC.NEBRISUM, 3).setRGBaLiquid(128, 255, 128, 255).setQuality(1.0F, 512, 4),
	Usp		, Unseptpentium				=	Usp		= unknown		( 1750,	"Unseptpentium"			, 175, 356),
	Ush		, Unsepthexium				=	Ush		= unknown		( 1760,	"Unsepthexium"			, 176, 360),
	Uss		, Unseptseptium				=	Uss		= unknown		( 1770,	"Unseptseptium"			, 177, 364),
	Uso		, Unseptoctium				=	Uso		= unknown		( 1780,	"Unseptoctium"			, 178, 368),
	Use		, Unseptennium				=	Use		= unknown		( 1790,	"Unseptennium"			, 179, 372),
	Uon		, Unoctnilium				=	Uon		= unknown		( 1800,	"Unoctnilium"			, 180, 376),
	Uou		, Unoctunium				=	Uou		= unknown		( 1810,	"Unoctunium"			, 181, 380),
	Uob		, Unoctbium					=	Uob		= unknown		( 1820,	"Unoctbium"				, 182, 384),
	Uot		, Unocttrium				=	Uot		= unknown		( 1830,	"Unocttrium"			, 183, 388),
	Uoq		, Unoctquadium				=	Uoq		= unknown		( 1840,	"Unoctquadium"			, 184, 392),
	Uop		, Unoctpentium				=	Uop		= unknown		( 1850,	"Unoctpentium"			, 185, 396),
	Uoh		, Unocthexium				=	Uoh		= unknown		( 1860,	"Unocthexium"			, 186, 400),
	Uos		, Unoctseptium				=	Uos		= unknown		( 1870,	"Unoctseptium"			, 187, 405),
	Uoo		, Unoctoctium				=	Uoo		= unknown		( 1880,	"Unoctoctium"			, 188, 410),
	Uoe		, Unoctennium				=	Uoe		= unknown		( 1890,	"Unoctennium"			, 189, 415),
	Uen		, Unennilium				=	Uen		= unknown		( 1900,	"Unennilium"			, 190, 420),
	Ueu		, Unennunium				=	Ueu		= unknown		( 1910,	"Unennunium"			, 191, 425),
	Ueb		, Unennbium					=	Ueb		= unknown		( 1920,	"Unennbium"				, 192, 430),
	Uet		, Unenntrium				=	Uet		= unknown		( 1930,	"Unenntrium"			, 193, 435),
	Ueq		, Unennquadium				=	Ueq		= unknown		( 1940,	"Unennquadium"			, 194, 440),
	Uep		, Unennpentium				=	Uep		= unknown		( 1950,	"Unennpentium"			, 195, 445),
	Ueh		, Unennhexium				=	Ueh		= unknown		( 1960,	"Unennhexium"			, 196, 450),
	Ues		, Unennseptium				=	Ues		= unknown		( 1970,	"Unennseptium"			, 197, 455),
	Ueo		, Unennoctium				=	Ueo		= unknown		( 1980,	"Unennoctium"			, 198, 460),
	Uee		, Unennennium				=	Uee		= unknown		( 1990,	"Unennennium"			, 199, 465),
	Bnn		, Binilnilium				=	Bnn		= unknown		( 2000,	"Binilnilium"			, 200, 470),
	Bnu		, Binilunium				=	Bnu		= unknown		( 2010,	"Binilunium"			, 201, 475),
	Bnb		, Binilbium					=	Bnb		= unknown		( 2020,	"Binilbium"				, 202, 480),
	Bnt		, Biniltrium				=	Bnt		= unknown		( 2030,	"Biniltrium"			, 203, 485),
	Bnq		, Binilquadium				=	Bnq		= unknown		( 2040,	"Binilquadium"			, 204, 490),
	Bnp		, Binilpentium				=	Bnp		= unknown		( 2050,	"Binilpentium"			, 205, 495),
	Bnh		, Binilhexium				=	Bnh		= unknown		( 2060,	"Binilhexium"			, 206, 500),
	Bns		, Binilseptium				=	Bns		= unknown		( 2070,	"Binilseptium"			, 207, 505),
	Bno		, Biniloctium				=	Bno		= unknown		( 2080,	"Biniloctium"			, 208, 510),
	Bne		, Binilennium				=	Bne		= unknown		( 2090,	"Binilennium"			, 209, 515),
	Bun		, Biunnilium				=	Bun		= unknown		( 2100,	"Biunnilium"			, 210, 520),
	Buu		, Biununium					=	Buu		= unknown		( 2110,	"Biununium"				, 211, 525),
	Bub		, Biunbium					=	Bub		= unknown		( 2120,	"Biunbium"				, 212, 530),
	But		, Biuntrium					=	But		= unknown		( 2130,	"Biuntrium"				, 213, 535),
	Buq		, Biunquadium				=	Buq		= unknown		( 2140,	"Biunquadium"			, 214, 540),
	Bup		, Biunpentium				=	Bup		= unknown		( 2150,	"Biunpentium"			, 215, 545),
	Buh		, Biunhexium				=	Buh		= unknown		( 2160,	"Biunhexium"			, 216, 550),
	Bus		, Biunseptium				=	Bus		= unknown		( 2170,	"Biunseptium"			, 217, 555),
	Buo		, Biunoctium				=	Buo		= unknown		( 2180,	"Biunoctium"			, 218, 560),
	Bue		, Biunennium				=	Bue		= unknown		( 2190,	"Biunennium"			, 219, 565),
	Bbn		, Bibinilium				=	Bbn		= unknown		( 2200,	"Bibinilium"			, 220, 570),
	Bbu		, Bibiunium					=	Bbu		= unknown		( 2210,	"Bibiunium"				, 221, 575),
	Ad		, Adamantium				= 	Ad		= element		( 2220,	"Adamantium"			, 222, 580,  5425, 14528, 13.35624762, SET_SHINY			, 255, 255, 255, 255).add(	G_INGOT_MACHINE_ORES, SMITHABLE, MELTING, EXTRUDER, MAGICAL, MAGNETIC_PASSIVE	).aspects(TC.METALLUM, 10, TC.PRAECANTIO, 10).setQuality(10.0F, 5120, 5).addIdenticalNames("Adamant"), Bbb = Ad, Bibibium = Ad.addIdenticalNames("Bibibium"),
	Bbt		, Bibitrium					=	Bbt		= unknown		( 2230,	"Bibitrium"				, 223, 585),
	Bbq		, Bibiquadium				=	Bbq		= unknown		( 2240,	"Bibiquadium"			, 224, 590),
	Bbp		, Bibipentium				=	Bbp		= unknown		( 2250,	"Bibipentium"			, 225, 595),
	Bbh		, Bibihexium				=	Bbh		= unknown		( 2260,	"Bibihexium"			, 226, 600),
	Bbs		, Bibiseptium				=	Bbs		= unknown		( 2270,	"Bibiseptium"			, 227, 605),
	Bbo		, Bibioctium				=	Bbo		= unknown		( 2280,	"Bibioctium"			, 228, 610),
	Bbe		, Bibiennium				=	Bbe		= unknown		( 2290,	"Bibiennium"			, 229, 615),
	Btn		, Bitrinilium				=	Btn		= unknown		( 2300,	"Bitrinilium"			, 230, 620),
	Btu		, Bitriunium				=	Btu		= unknown		( 2310,	"Bitriunium"			, 231, 625),
	Btb		, Bitribium					=	Btb		= unknown		( 2320,	"Bitribium"				, 232, 630),
	Btt		, Bitritrium				=	Btt		= unknown		( 2330,	"Bitritrium"			, 233, 635),
	Btq		, Bitriquadium				=	Btq		= unknown		( 2340,	"Bitriquadium"			, 234, 640),
	Btp		, Bitripentium				=	Btp		= unknown		( 2350,	"Bitripentium"			, 235, 645),
	Bth		, Bitrihexium				=	Bth		= unknown		( 2360,	"Bitrihexium"			, 236, 650),
	Bts		, Bitriseptium				=	Bts		= unknown		( 2370,	"Bitriseptium"			, 237, 655),
	Bto		, Bitrioctium				=	Bto		= unknown		( 2380,	"Bitrioctium"			, 238, 660),
	Mcg		, MacGuffium				=	Mcg		= element		( 2390,	"Mac-Guffium"			, 239, 665,   200,  1000,  3.122     , SET_SHINY			, 200,  50, 150, 255).add(	G_CONTAINERS																	).aspects(TC.ALIENIS, 8, TC.PERMUTATIO, 8, TC.SPIRITUS, 8, TC.AURAM, 8, TC.VITIUM, 8, TC.RADIO, 8, TC.MAGNETO, 8, TC.ELECTRUM, 8, TC.NEBRISUM, 8, TC.STRONTIO, 8), Bte = Mcg, Bitriennium = Mcg.addIdenticalNames("Bitriennium"),
	Bqn		, Biquadnilium				=	Bqn		= unknown		( 2400,	"Biquadnilium"			, 240, 670),
	Bqu		, Biquadunium				=	Bqu		= unknown		( 2410,	"Biquadunium"			, 241, 675),
	Bqb		, Biquadbium				=	Bqb		= unknown		( 2420,	"Biquadbium"			, 242, 680),
	Bqt		, Biquadtrium				=	Bqt		= unknown		( 2430,	"Biquadtrium"			, 243, 685),
	Bqq		, Biquadquadium				=	Bqq		= unknown		( 2440,	"Biquadquadium"			, 244, 690),
	Bqp		, Biquadpentium				=	Bqp		= unknown		( 2450,	"Biquadpentium"			, 245, 695),
	Bqh		, Biquadhexium				=	Bqh		= unknown		( 2460,	"Biquadhexium"			, 246, 700),
	Bqs		, Biquadseptium				=	Bqs		= unknown		( 2470,	"Biquadseptium"			, 247, 705),
	Bqo		, Biquadoctium				=	Bqo		= unknown		( 2480,	"Biquadoctium"			, 248, 710),
	Bqe		, Biquadennium				=	Bqe		= unknown		( 2490,	"Biquadennium"			, 249, 715),
	Bpn		, Bipentnilium				=	Bpn		= unknown		( 2500,	"Bipentnilium"			, 250, 720),
	Bpu		, Bipentunium				=	Bpu		= unknown		( 2510,	"Bipentunium"			, 251, 725),
	Bpb		, Bipentbium				=	Bpb		= unknown		( 2520,	"Bipentbium"			, 252, 730),
	Bpt		, Bipenttrium				=	Bpt		= unknown		( 2530,	"Bipenttrium"			, 253, 735),
	Bpq		, Bipentquadium				=	Bpq		= unknown		( 2540,	"Bipentquadium"			, 254, 740),
	Bpp		, Bipentpentium				=	Bpp		= unknown		( 2550,	"Bipentpentium"			, 255, 745),
	Bph		, Bipenthexium				=	Bph		= unknown		( 2560,	"Bipenthexium"			, 256, 750),
	Bps		, Bipentseptium				=	Bps		= unknown		( 2570,	"Bipentseptium"			, 257, 755),
	Bpo		, Bipentoctium				=	Bpo		= unknown		( 2580,	"Bipentoctium"			, 258, 760),
	Bpe		, Bipentennium				=	Bpe		= unknown		( 2590,	"Bipentennium"			, 259, 765),
	Bhn		, Bihexnilium				=	Bhn		= unknown		( 2600,	"Bihexnilium"			, 260, 770),
	Bhu		, Bihexunium				=	Bhu		= unknown		( 2610,	"Bihexunium"			, 261, 775),
	Bhb		, Bihexbium					=	Bhb		= unknown		( 2620,	"Bihexbium"				, 262, 780),
	Bht		, Bihextrium				=	Bht		= unknown		( 2630,	"Bihextrium"			, 263, 785),
	Bhq		, Bihexquadium				=	Bhq		= unknown		( 2640,	"Bihexquadium"			, 264, 790),
	Bhp		, Bihexpentium				=	Bhp		= unknown		( 2650,	"Bihexpentium"			, 265, 795),
	Bhh		, Bihexhexium				=	Bhh		= unknown		( 2660,	"Bihexhexium"			, 266, 800),
	Bhs		, Bihexseptium				=	Bhs		= unknown		( 2670,	"Bihexseptium"			, 267, 805),
	Bho		, Bihexoctium				=	Bho		= unknown		( 2680,	"Bihexoctium"			, 268, 810),
	Bhe		, Bihexennium				=	Bhe		= unknown		( 2690,	"Bihexennium"			, 269, 815),
	Bsn		, Biseptnilium				=	Bsn		= unknown		( 2700,	"Biseptnilium"			, 270, 820),
	Bsu		, Biseptunium				=	Bsu		= unknown		( 2710,	"Biseptunium"			, 271, 825),
	Bsb		, Biseptbium				=	Bsb		= unknown		( 2720,	"Biseptbium"			, 272, 830),
	Bst		, Bisepttrium				=	Bst		= unknown		( 2730,	"Bisepttrium"			, 273, 835),
	Bsq		, Biseptquadium				=	Bsq		= unknown		( 2740,	"Biseptquadium"			, 274, 840),
	Bsp		, Biseptpentium				=	Bsp		= unknown		( 2750,	"Biseptpentium"			, 275, 845),
	Bsh		, Bisepthexium				=	Bsh		= unknown		( 2760,	"Bisepthexium"			, 276, 850),
	Bss		, Biseptseptium				=	Bss		= unknown		( 2770,	"Biseptseptium"			, 277, 855),
	Bso		, Biseptoctium				=	Bso		= unknown		( 2780,	"Biseptoctium"			, 278, 860),
	Bse		, Biseptennium				=	Bse		= unknown		( 2790,	"Biseptennium"			, 279, 865),
	Bon		, Bioctnilium				=	Bon		= unknown		( 2800,	"Bioctnilium"			, 280, 870),
	Bou		, Bioctunium				=	Bou		= unknown		( 2810,	"Bioctunium"			, 281, 875),
	Bob		, Bioctbium					=	Bob		= unknown		( 2820,	"Bioctbium"				, 282, 880),
	Bot		, Biocttrium				=	Bot		= unknown		( 2830,	"Biocttrium"			, 283, 885),
	Boq		, Bioctquadium				=	Boq		= unknown		( 2840,	"Bioctquadium"			, 284, 890),
	Bop		, Bioctpentium				=	Bop		= unknown		( 2850,	"Bioctpentium"			, 285, 895),
	Boh		, Biocthexium				=	Boh		= unknown		( 2860,	"Biocthexium"			, 286, 900),
	Bos		, Bioctseptium				=	Bos		= unknown		( 2870,	"Bioctseptium"			, 287, 905),
	Boo		, Bioctoctium				=	Boo		= unknown		( 2880,	"Bioctoctium"			, 288, 910),
	Boe		, Bioctennium				=	Boe		= unknown		( 2890,	"Bioctennium"			, 289, 915),
	Ben		, Biennilium				=	Ben		= unknown		( 2900,	"Biennilium"			, 290, 920),
	Beu		, Biennunium				=	Beu		= unknown		( 2910,	"Biennunium"			, 291, 925),
	Beb		, Biennbium					=	Beb		= unknown		( 2920,	"Biennbium"				, 292, 930),
	Bet		, Bienntrium				=	Bet		= unknown		( 2930,	"Bienntrium"			, 293, 935),
	Beq		, Biennquadium				=	Beq		= unknown		( 2940,	"Biennquadium"			, 294, 940),
	Bep		, Biennpentium				=	Bep		= unknown		( 2950,	"Biennpentium"			, 295, 945),
	Beh		, Biennhexium				=	Beh		= unknown		( 2960,	"Biennhexium"			, 296, 950),
	Bes		, Biennseptium				=	Bes		= unknown		( 2970,	"Biennseptium"			, 297, 955),
	Beo		, Biennoctium				=	Beo		= unknown		( 2980,	"Biennoctium"			, 298, 960),
	Bee		, Biennennium				=	Bee		= unknown		( 2990,	"Biennennium"			, 299, 965),
	Tnn		, Trinilnilium				=	Tnn		= unknown		( 3000,	"Trinilnilium"			, 300, 970),
	Tnu		, Trinilunium				=	Tnu		= unknown		( 3010,	"Trinilunium"			, 301, 975),
	Tnb		, Trinilbium				=	Tnb		= unknown		( 3020,	"Trinilbium"			, 302, 980),
	Tnt		, Triniltrium				=	Tnt		= unknown		( 3030,	"Triniltrium"			, 303, 985),
	Tnq		, Trinilquadium				=	Tnq		= unknown		( 3040,	"Trinilquadium"			, 304, 990),
	Tnp		, Trinilpentium				=	Tnp		= unknown		( 3050,	"Trinilpentium"			, 305, 995),
	Tnh		, Trinilhexium				=	Tnh		= unknown		( 3060,	"Trinilhexium"			, 306,1000),
	Tns		, Trinilseptium				=	Tns		= unknown		( 3070,	"Trinilseptium"			, 307,1005),
	Tno		, Triniloctium				=	Tno		= unknown		( 3080,	"Triniloctium"			, 308,1010),
	Tne		, Trinilennium				=	Tne		= unknown		( 3090,	"Trinilennium"			, 309,1015),
	Tun		, Triunnilium				=	Tun		= unknown		( 3100,	"Triunnilium"			, 310,1020),
	Tuu		, Triununium				=	Tuu		= unknown		( 3110,	"Triununium"			, 311,1025),
	Tub		, Triunbium					=	Tub		= unknown		( 3120,	"Triunbium"				, 312,1030),
	Tut		, Triuntrium				=	Tut		= unknown		( 3130,	"Triuntrium"			, 313,1035),
	Tuq		, Triunquadium				=	Tuq		= unknown		( 3140,	"Triunquadium"			, 314,1040),
	Tup		, Triunpentium				=	Tup		= unknown		( 3150,	"Triunpentium"			, 315,1045),
	Tuh		, Triunhexium				=	Tuh		= unknown		( 3160,	"Triunhexium"			, 316,1050),
	Tus		, Triunseptium				=	Tus		= unknown		( 3170,	"Triunseptium"			, 317,1055),
	Tuo		, Triunoctium				=	Tuo		= unknown		( 3180,	"Triunoctium"			, 318,1060),
	Tue		, Triunennium				=	Tue		= unknown		( 3190,	"Triunennium"			, 319,1065),
	Tbn		, Tribinilium				=	Tbn		= unknown		( 3200,	"Tribinilium"			, 320,1070),
	Tbu		, Tribiunium				=	Tbu		= unknown		( 3210,	"Tribiunium"			, 321,1075),
	Tbb		, Tribibium					=	Tbb		= unknown		( 3220,	"Tribibium"				, 322,1080),
	Tbt		, Tribitrium				=	Tbt		= unknown		( 3230,	"Tribitrium"			, 323,1085),
	Tbq		, Tribiquadium				=	Tbq		= unknown		( 3240,	"Tribiquadium"			, 324,1090),
	Tbp		, Tribipentium				=	Tbp		= unknown		( 3250,	"Tribipentium"			, 325,1095),
	Tbh		, Tribihexium				=	Tbh		= unknown		( 3260,	"Tribihexium"			, 326,1100),
	Tbs		, Tribiseptium				=	Tbs		= unknown		( 3270,	"Tribiseptium"			, 327,1105),
	Tbo		, Tribioctium				=	Tbo		= unknown		( 3280,	"Tribioctium"			, 328,1110),
	Tbe		, Tribiennium				=	Tbe		= unknown		( 3290,	"Tribiennium"			, 329,1115),
	Ttn		, Tritrinilium				=	Ttn		= unknown		( 3300,	"Tritrinilium"			, 330,1120),
	Ttu		, Tritriunium				=	Ttu		= unknown		( 3310,	"Tritriunium"			, 331,1125),
	Ttb		, Tritribium				=	Ttb		= unknown		( 3320,	"Tritribium"			, 332,1130),
	Ttt		, Tritritrium				=	Ttt		= unknown		( 3330,	"Tritritrium"			, 333,1135),
	Ttq		, Tritriquadium				=	Ttq		= unknown		( 3340,	"Tritriquadium"			, 334,1140),
	Ttp		, Tritripentium				=	Ttp		= unknown		( 3350,	"Tritripentium"			, 335,1145),
	Tth		, Tritrihexium				=	Tth		= unknown		( 3360,	"Tritrihexium"			, 336,1150),
	Tts		, Tritriseptium				=	Tts		= unknown		( 3370,	"Tritriseptium"			, 337,1155),
	Tto		, Tritrioctium				=	Tto		= unknown		( 3380,	"Tritrioctium"			, 338,1160),
	Tte		, Tritriennium				=	Tte		= unknown		( 3390,	"Tritriennium"			, 339,1165),
	Tqn		, Triquadnilium				=	Tqn		= unknown		( 3400,	"Triquadnilium"			, 340,1170),
	Tqu		, Triquadunium				=	Tqu		= unknown		( 3410,	"Triquadunium"			, 341,1175),
	Tqb		, Triquadbium				=	Tqb		= unknown		( 3420,	"Triquadbium"			, 342,1180),
	Tqt		, Triquadtrium				=	Tqt		= unknown		( 3430,	"Triquadtrium"			, 343,1185),
	Tqq		, Triquadquadium			=	Tqq		= unknown		( 3440,	"Triquadquadium"		, 344,1190),
	Tqp		, Triquadpentium			=	Tqp		= unknown		( 3450,	"Triquadpentium"		, 345,1195),
	Tqh		, Triquadhexium				=	Tqh		= unknown		( 3460,	"Triquadhexium"			, 346,1200),
	Tqs		, Triquadseptium			=	Tqs		= unknown		( 3470,	"Triquadseptium"		, 347,1205),
	Tqo		, Triquadoctium				=	Tqo		= unknown		( 3480,	"Triquadoctium"			, 348,1210),
	Tqe		, Triquadennium				=	Tqe		= unknown		( 3490,	"Triquadennium"			, 349,1215),
	Tpn		, Tripentnilium				=	Tpn		= unknown		( 3500,	"Tripentnilium"			, 350,1220),
	Tpu		, Tripentunium				=	Tpu		= unknown		( 3510,	"Tripentunium"			, 351,1225),
	Tpb		, Tripentbium				=	Tpb		= unknown		( 3520,	"Tripentbium"			, 352,1230),
	Tpt		, Tripenttrium				=	Tpt		= unknown		( 3530,	"Tripenttrium"			, 353,1235),
	Tpq		, Tripentquadium			=	Tpq		= unknown		( 3540,	"Tripentquadium"		, 354,1240),
	Tpp		, Tripentpentium			=	Tpp		= unknown		( 3550,	"Tripentpentium"		, 355,1245),
	Tph		, Tripenthexium				=	Tph		= unknown		( 3560,	"Tripenthexium"			, 356,1250),
	Tps		, Tripentseptium			=	Tps		= unknown		( 3570,	"Tripentseptium"		, 357,1255),
	Tpo		, Tripentoctium				=	Tpo		= unknown		( 3580,	"Tripentoctium"			, 358,1260),
	Tpe		, Tripentennium				=	Tpe		= unknown		( 3590,	"Tripentennium"			, 359,1265),
	Thn		, Trihexnilium				=	Thn		= unknown		( 3600,	"Trihexnilium"			, 360,1270),
	Thu		, Trihexunium				=	Thu		= unknown		( 3610,	"Trihexunium"			, 361,1275),
	Thb		, Trihexbium				=	Thb		= unknown		( 3620,	"Trihexbium"			, 362,1280),
	Tht		, Trihextrium				=	Tht		= unknown		( 3630,	"Trihextrium"			, 363,1285),
	Thq		, Trihexquadium				=	Thq		= unknown		( 3640,	"Trihexquadium"			, 364,1290),
	Thp		, Trihexpentium				=	Thp		= unknown		( 3650,	"Trihexpentium"			, 365,1295),
	Thh		, Trihexhexium				=	Thh		= unknown		( 3660,	"Trihexhexium"			, 366,1300),
	Ths		, Trihexseptium				=	Ths		= unknown		( 3670,	"Trihexseptium"			, 367,1305),
	Tho		, Trihexoctium				=	Tho		= unknown		( 3680,	"Trihexoctium"			, 368,1310),
	The		, Trihexennium				=	The		= unknown		( 3690,	"Trihexennium"			, 369,1315),
	Tsn		, Triseptnilium				=	Tsn		= unknown		( 3700,	"Triseptnilium"			, 370,1320),
	Tsu		, Triseptunium				=	Tsu		= unknown		( 3710,	"Triseptunium"			, 371,1325),
	Gt		, Gravitonium				=	Gt		= element		( 3720,	"Gravitonium"			, 372,1330,   112,  1275, 1768.866761, SET_SHINY			,   0,  50,   0, 255).add(	G_CONTAINERS																	).aspects(TC.TERRA, 10, TC.POTENTIA, 10), Tsb = Gt, Triseptbium = Gt.addIdenticalNames("Triseptbium"),
	Tst		, Trisepttrium				=	Tst		= unknown		( 3730,	"Trisepttrium"			, 373,1335),
	Tsq		, Triseptquadium			=	Tsq		= unknown		( 3740,	"Triseptquadium"		, 374,1340),
	Tsp		, Triseptpentium			=	Tsp		= unknown		( 3750,	"Triseptpentium"		, 375,1345),
	Tsh		, Trisepthexium				=	Tsh		= unknown		( 3760,	"Trisepthexium"			, 376,1350),
	Tss		, Triseptseptium			=	Tss		= unknown		( 3770,	"Triseptseptium"		, 377,1355),
	Tso		, Triseptoctium				=	Tso		= unknown		( 3780,	"Triseptoctium"			, 378,1360),
	Tse		, Triseptennium				=	Tse		= unknown		( 3790,	"Triseptennium"			, 379,1365),
	Ton		, Trioctnilium				=	Ton		= unknown		( 3800,	"Trioctnilium"			, 380,1370),
	Tou		, Trioctunium				=	Tou		= unknown		( 3810,	"Trioctunium"			, 381,1375),
	Tob		, Trioctbium				=	Tob		= unknown		( 3820,	"Trioctbium"			, 382,1380),
	Tot		, Triocttrium				=	Tot		= unknown		( 3830,	"Triocttrium"			, 383,1385),
	Toq		, Trioctquadium				=	Toq		= unknown		( 3840,	"Trioctquadium"			, 384,1390),
	Top		, Trioctpentium				=	Top		= unknown		( 3850,	"Trioctpentium"			, 385,1395),
	Toh		, Triocthexium				=	Toh		= unknown		( 3860,	"Triocthexium"			, 386,1400),
	Tos		, Trioctseptium				=	Tos		= unknown		( 3870,	"Trioctseptium"			, 387,1405),
	Too		, Trioctoctium				=	Too		= unknown		( 3880,	"Trioctoctium"			, 388,1410),
	Toe		, Trioctennium				=	Toe		= unknown		( 3890,	"Trioctennium"			, 389,1415),
	Ten		, Triennilium				=	Ten		= unknown		( 3900,	"Triennilium"			, 390,1420),
	Teu		, Triennunium				=	Teu		= unknown		( 3910,	"Triennunium"			, 391,1425),
	Teb		, Triennbium				=	Teb		= unknown		( 3920,	"Triennbium"			, 392,1430),
	Tet		, Trienntrium				=	Tet		= unknown		( 3930,	"Trienntrium"			, 393,1435),
	Teq		, Triennquadium				=	Teq		= unknown		( 3940,	"Triennquadium"			, 394,1440),
	Tep		, Triennpentium				=	Tep		= unknown		( 3950,	"Triennpentium"			, 395,1445),
	Teh		, Triennhexium				=	Teh		= unknown		( 3960,	"Triennhexium"			, 396,1450),
	Tes		, Triennseptium				=	Tes		= unknown		( 3970,	"Triennseptium"			, 397,1455),
	Teo		, Triennoctium				=	Teo		= unknown		( 3980,	"Triennoctium"			, 398,1460),
	Tee		, Triennennium				=	Tee		= unknown		( 3990,	"Triennennium"			, 399,1465);
	
	public static final OreDictMaterial
	Neutronium				= unused		("Neutronium"		).setQuality(6.0F, 81920, 6)	.add(IGNORE_IN_COLOR_LOG),
	
	Primitive				= unused		("Primitive"		).aspects(TC.MACHINA	, 1)	.add(DONT_SHOW_THIS_COMPONENT, IGNORE_IN_COLOR_LOG).setAllToTheOutputOf(null, 0, 1),
	Basic					= unused		("Basic"			).aspects(TC.MACHINA	, 2)	.add(DONT_SHOW_THIS_COMPONENT, IGNORE_IN_COLOR_LOG).setAllToTheOutputOf(null, 0, 1),
	Good					= unused		("Good"				).aspects(TC.MACHINA	, 3)	.add(DONT_SHOW_THIS_COMPONENT, IGNORE_IN_COLOR_LOG).setAllToTheOutputOf(null, 0, 1),
	Advanced				= unused		("Advanced"			).aspects(TC.MACHINA	, 4)	.add(DONT_SHOW_THIS_COMPONENT, IGNORE_IN_COLOR_LOG).setAllToTheOutputOf(null, 0, 1),
	Data					= unused		("Data"				).aspects(TC.MACHINA	, 4)	.add(DONT_SHOW_THIS_COMPONENT, IGNORE_IN_COLOR_LOG).setAllToTheOutputOf(null, 0, 1),
	Elite					= unused		("Elite"			).aspects(TC.MACHINA	, 5)	.add(DONT_SHOW_THIS_COMPONENT, IGNORE_IN_COLOR_LOG).setAllToTheOutputOf(null, 0, 1),
	Master					= unused		("Master"			).aspects(TC.MACHINA	, 6)	.add(DONT_SHOW_THIS_COMPONENT, IGNORE_IN_COLOR_LOG).setAllToTheOutputOf(null, 0, 1),
	Ultimate				= unused		("Ultimate"			).aspects(TC.MACHINA	, 7)	.add(DONT_SHOW_THIS_COMPONENT, IGNORE_IN_COLOR_LOG).setAllToTheOutputOf(null, 0, 1),
	Quantum					= unused		("Quantum"			).aspects(TC.ORDO		, 8)	.add(DONT_SHOW_THIS_COMPONENT, IGNORE_IN_COLOR_LOG).setAllToTheOutputOf(null, 0, 1),
	Superconductor			= unused		("Superconductor"	).aspects(TC.ELECTRUM	, 8)	.add(DONT_SHOW_THIS_COMPONENT, IGNORE_IN_COLOR_LOG).setAllToTheOutputOf(null, 0, 1),
	Infinite				= unused		("Infinite"			)								.add(DONT_SHOW_THIS_COMPONENT, IGNORE_IN_COLOR_LOG).setAllToTheOutputOf(null, 0, 1);
	
	public static final OreDictMaterial
	Water					= lqudelec		( 9800, "Water"					, SET_FLUID				, 100, 100, 255, 255).add(UUM, FOOD, UNRECYCLABLE, MELTING)													  														.setMcfg( 0, H				, 2*U, O				, 1*U)																									.aspects(TC.AQUA, 2).liquid(FL.Water.make(1000)).setStatsEnergetic(CS.C, CS.C+100).setDensity(1.0).setOriginalMod(MD.MC), H2O = Water,
	Snow					= dust			( 9801, "Snow"					, SET_FINE				, 250, 250, 250, 255).add(UUM, FOOD, UNRECYCLABLE, MORTAR)																											.setMcfg( 0, H				, 2*U, O				, 1*U)																									.aspects(TC.GELUM, 1).setSmelting(H2O, U).setStatsEnergetic(CS.C, CS.C+100).setDensity(1.0).setOriginalMod(MD.MC),
	Ice						= create		( 9802, "Ice"					, SET_CUBE_SHINY		, 200, 200, 255, 255).add(G_GEM_TRANSPARENT, UUM, FOOD, CONTAINERS, UNRECYCLABLE, BRITTLE, NO_ADVANCED_TOOLS, MORTAR)												.setMcfg( 0, H				, 2*U, O				, 1*U)																									.aspects(TC.GELUM, 2).setSmelting(H2O, U).setStatsEnergetic(CS.C, CS.C+100).setDensity(1.0).setOriginalMod(MD.MC).setQuality( 2.0F,	4, 0),
	FreshWater				= lqud			( 9803, "Fresh Water"			, SET_FLUID				, 110, 110, 255, 255).add(UUM, FOOD, UNRECYCLABLE)																													.setMcfg( 0, H				, 2*U, O				, 1*U)																									.aspects(TC.AQUA, 2).setStatsEnergetic(CS.C, CS.C+100).setDensity(1.0),
	HolyWater				= lqud			( 9805, "Holy Water"			, SET_FLUID				, 120, 120, 255, 255).add(GLOWING)																																	.setMcfg( 0, H				, 2*U, O				, 1*U)																									.aspects(TC.AQUA, 2, TC.AURAM, 1).setStatsEnergetic(CS.C, CS.C+100).setDensity(1.0),
	SeaWater				= lqud			( 9806, "Sea Water"				, SET_FLUID				,  90,  90, 255, 255).add(UNRECYCLABLE, LIQUID)																														.setMcfg( 0, H				, 2*U, O				, 1*U)																									.aspects(TC.AQUA, 2).setStatsEnergetic(CS.C, CS.C+100).setDensity(1.0),
	Air						= gas			( 9830, "Air"					, SET_GAS				, 169, 208, 245,  15).add(TRANSPARENT, GASSES)										  																				.setMcfg( 0, N				,40*U, O				,11*U, Ar				, 1*U)																			.setStatsEnergetic( 100,  200),
	
	RockSalt				= oredustelec	( 8203, "Rock Salt"				, SET_CUBE				, 240, 200, 200, 255).add(UUM, NO_ADVANCED_TOOLS, BRITTLE, MORTAR)												  													.setMcfg( 0, K 				, 1*U, Cl				, 1*U)																									.aspects(TC.FAMES, 1).setStatsEnergetic(1040, 1690), KCl = RockSalt,
	Salt					= oredustelec	( 8204, "Salt"					, SET_CUBE				, 250, 250, 250, 255).add(UUM, NO_ADVANCED_TOOLS, BRITTLE, MORTAR, FOOD)												   											.setMcfg( 0, Na				, 1*U, Cl				, 1*U)																									.aspects(TC.FAMES, 1).setStatsEnergetic(1074, 1686), NaCl = Salt,
	IodineSalt				= oredustelec	( 8242, "Iodine Salt"			, SET_CUBE				, 240, 200, 240, 255).add(UUM, NO_ADVANCED_TOOLS, BRITTLE, MORTAR)												  													.setMcfg( 0, I 				, 1*U, Cl				, 1*U)																									.aspects(TC.TEMPESTAS, 1).setStatsEnergetic(300, 370).setOriginalMod(MD.GT), ICl = IodineSalt,
	SilverIodide			= oredustelec	( 8243, "Silver Iodide"			, SET_CUBE				, 240, 200, 100, 255).add(UUM, NO_ADVANCED_TOOLS, BRITTLE, MORTAR)												  													.setMcfg( 0, Ag 			, 1*U, I				, 1*U)																									.aspects(TC.TEMPESTAS, 2).setStatsEnergetic(831, 1779).setOriginalMod(MD.RoC), AgI = SilverIodide,
	Saltpeter				= oredustelec	( 8205, "Saltpeter"				, SET_FINE				, 230, 230, 230, 255).add(UUM, FLAMMABLE)									   																						.setMcfg( 0, K 				, 1*U, N 				, 1*U, O				, 3*U)																			.aspects(TC.IGNIS, 2).setOriginalMod(MD.RC), KNO3 = Saltpeter,
	SiliconDioxide			= oredust		( 8000, "Silicon Dioxide"		, SET_QUARTZ			, 200, 200, 200, 255).add(UUM, BRITTLE, QUARTZ, CRYSTALLISABLE, NO_ADVANCED_TOOLS, FURNACE)																			.setMcfg( 0, Si				, 1*U, O 				, 2*U)																									.setStatsEnergetic(1986, 3220).setOriginalMod(MD.IC2), SiO2 = SiliconDioxide,
	Carborundum				= metalore		( 8003, "Carborundum"			, SET_QUARTZ			,  77,  77,  77, 255).add(UUM, BRITTLE, QUARTZ)																														.setMcfg( 0, Si				, 1*U, C 				, 1*U)																									.setStatsEnergetic(3000, 3100).setOriginalMod(MD.IHL).alloyElectrolyzer(), SiC = Carborundum,
	LithiumOxide			= dustelec		( 8004, "Lithium Oxide"			, SET_ROUGH				, 222, 222, 234, 255).add(UUM)																																		.setMcfg( 0, Li				, 2*U, O 				, 1*U)																									.setOriginalMod(MD.IHL), Li2O = LithiumOxide,
	Ferrite					= metalore		( 8005, "Ferrite"				, SET_METALLIC			, 120, 120, 130, 255).add(UUM, DECOMPOSABLE, ELECTROLYSER)																											.setMcfg( 0, Li				, 2*U, Fe 				, 2*U, O				, 4*U)																			.setOriginalMod(MD.IHL), Li2Fe2O4 = Ferrite,
	Datolite				= dustelec		( 8006, "Datolite"				, SET_ROUGH				, 222, 255, 222, 255).add(UUM)																																		.setMcfg( 0, H				, 2*U, Ca 				, 2*U, B 				, 2*U, Si 				, 2*U, O 				,10*U)							.setOriginalMod(MD.IHL), H2Ca2B2Si2O10 = Datolite,
	HydrogenBorate			= dustelec		( 8007, "Hydrogen Borate"		, SET_FINE				, 234, 234, 255, 255).add(UUM)																																		.setMcfg( 0, H				, 3*U, B				, 1*U, O 				, 3*U)																			.setOriginalMod(MD.IHL), H3BO3 = HydrogenBorate, BoricAcid = HydrogenBorate.addIdenticalNames("BoricAcid"),
	VanadiumPentoxide		= oredustelec	( 8234, "Vanadium Pentoxide"	, SET_FINE				,  50,  50,  50, 255).add(UUM)																																		.setMcfg( 0, V				, 2*U, O				, 5*U)																									.setSmelting(V, U/7), V2O5 = VanadiumPentoxide,
	Calcite					= oredustelec	( 9107, "Calcite"				, SET_DULL				, 250, 230, 220, 255).add(UUM, MELTING)																																.setMcfg( 0, Ca				, 1*U, C 				, 1*U, O				, 3*U)																			.setStatsEnergetic(1612, 3000), CaCO3 = Calcite,
	CalciumSulfate			= dustelec		( 8274, "Calcium Sulfate"		, SET_CUBE				, 240, 220, 210, 255).add(UUM)																																		.setMcfg( 0, Ca				, 1*U, S				, 1*U, O				, 4*U)																			.setStatsEnergetic(1730, 3000).addIdenticalNames("CalciumSulphate"), CaSO4 = CalciumSulfate,
	SodiumPersulfate		= dustelec		( 9822, "Sodium Persulfate"		, SET_CUBE				, 130, 180, 250, 255).add(UUM)																																		.setMcfg( 0, Na				, 1*U, S				, 1*U, O				, 4*U)																			.addIdenticalNames("SodiumPersulphate"), NaSO4 = SodiumPersulfate,
	SodiumSulfide			= dustelec		( 9823, "Sodium Sulfide"		, SET_CUBE				, 220, 220, 100, 255).add(UUM)																																		.setMcfg( 0, Na				, 2*U, S				, 1*U)																									.addIdenticalNames("SodiumSulphide"), Na2S = SodiumSulfide,
	SodiumSulfite			= dustelec		( 8269, "Sodium Sulfite"		, SET_CUBE				, 190, 190, 140, 255).add(UUM)																																		.setMcfg( 0, Na				, 2*U, S				, 1*U, O				, 3*U)																			.addIdenticalNames("SodiumSulphite"), Na2SO3 = SodiumSulfite,
	SodiumSulfate			= dustelec		( 8270, "Sodium Sulfate"		, SET_CUBE				, 190, 190, 140, 255).add(UUM)																																		.setMcfg( 0, Na				, 2*U, S				, 1*U, O				, 4*U)																			.addIdenticalNames("SodiumSulphate"), Na2SO4 = SodiumSulfate, // Can be used to store Heat very well.
	SodiumHydroxide			= dustelec		( 8268, "Sodium Hydroxide"		, SET_CUBE				, 220, 250, 220, 255).add(UUM)																																		.setMcfg( 0, Na				, 1*U, O				, 1*U, H				, 1*U)																			.setOriginalMod(MD.IHL), NaOH = SodiumHydroxide,
	SodiumBisulfate			= dustelec		( 8230, "Sodium Bisulfate"		, SET_FINE				, 240, 240, 255, 255).add(UUM)																																		.setMcfg( 0, Na				, 1*U, H				, 1*U, S				, 1*U, O				, 4*U)													.addIdenticalNames("SodiumBisulphate"), NaHSO4 = SodiumBisulfate,
	SodiumPyrosulfate		= dustelec		( 8231, "Sodium Pyrosulfate"	, SET_FINE				, 240, 240, 255, 255).add(UUM)																																		.setMcfg( 0, Na				, 2*U, S				, 2*U, O				, 7*U)																			.addIdenticalNames("SodiumPyrosulphate"), Na2S2O7 = SodiumPyrosulfate,
	PotassiumSulfide		= dustelec		( 8272, "Potassium Sulfide"		, SET_CUBE				, 100, 220, 220, 255).add(UUM)																																		.setMcfg( 0, K				, 2*U, S				, 1*U)																									.addIdenticalNames("PotassiumSulphide"), K2S = PotassiumSulfide,
	PotassiumSulfate		= dustelec		( 8271, "Potassium Sulfate"		, SET_CUBE				, 140, 190, 190, 255).add(UUM)																																		.setMcfg( 0, K				, 2*U, S				, 1*U, O				, 4*U)																			.setOriginalMod(MD.IHL).addIdenticalNames("PotassiumSulphate"), K2SO4 = PotassiumSulfate,
	PotassiumBisulfate		= dustelec		( 8232, "Potassium Bisulfate"	, SET_FINE				, 255, 240, 240, 255).add(UUM)																																		.setMcfg( 0, K				, 1*U, H				, 1*U, S				, 1*U, O				, 4*U)													.addIdenticalNames("PotassiumBisulphate"), KHSO4 = PotassiumBisulfate,
	PotassiumPyrosulfate	= dustelec		( 8233, "Potassium Pyrosulfate"	, SET_FINE				, 255, 240, 240, 255).add(UUM)																																		.setMcfg( 0, K				, 2*U, S				, 2*U, O				, 7*U)																			.addIdenticalNames("PotassiumPyrosulphate"), K2S2O7 = PotassiumPyrosulfate,
	Phosphate				= oredustelec	( 8207, "Phosphate"				, SET_DULL				, 255, 255,   0, 255).add(UUM, FLAMMABLE, EXPLOSIVE)							  																					.setMcfg( 0, P 				, 1*U, O 				, 4*U)																									.setStatsEnergetic( 400,  800), PO4 = Phosphate,
	
	HydrogenPeroxide		= lqud			( 9809, "Hydrogen Peroxide"		, SET_FLUID				,  20,  20, 255, 255).add(UUM, LIQUID)													  																			.setMcfg( 0, H				, 2*U, O				, 2*U)																									.aspects(TC.AQUA, 3).setStatsEnergetic(CS.C, CS.C+150).setDensity(1.0).setOriginalMod(MD.IHL), H2O2 = HydrogenPeroxide,
	Glyceryl				= lqudchemelec	( 9821, "Glyceryl"				, SET_FLUID				,   0, 150, 150, 255).add(UUM, LIQUID, FLAMMABLE, EXPLOSIVE)																										.setMcfg( 0, C				, 3*U, H				, 5*U, N				, 3*U, O				, 9*U)													.setStatsEnergetic( 287,  323),
	Glycerol				= lqudchemelec	( 9828, "Glycerol"				, SET_FLUID				,   0, 180, 180, 255).add(UUM, LIQUID, FLAMMABLE)																													.setMcfg( 0, C				, 3*U, H				, 8*U, O				, 3*U)																			.setStatsEnergetic( 291,  563),
	SulfuricAcid			= lqudacidelec	( 9824, "Sulfuric Acid"			, SET_FLUID				, 255, 128,   0, 255).add(UUM, LIQUID)																																.setMcfg( 0, H				, 2*U, S				, 1*U, O				, 4*U)																			.setStatsEnergetic( 200,  400).setOriginalMod(MD.FZ).addIdenticalNames("SulphuricAcid"), H2SO4 = SulfuricAcid,
	NitricAcid				= lqudacidelec	( 9825, "Nitric Acid"			, SET_FLUID				, 128, 255,   0, 255).add(UUM, LIQUID)																																.setMcfg( 0, H				, 1*U, N				, 1*U, O				, 3*U)																			.setStatsEnergetic( 200,  400), HNO3 = NitricAcid,
	
	ChloroauricAcid			= lqudacidelec	( 8400, "Chloroauric Acid"		, SET_FLUID				, 255, 200,  70, 255).add(UUM, LIQUID)																																.setMcfg( 0, Au				, 1*U, Cl				, 4*U, H				, 1*U)																			.setStatsEnergetic( 200,  400).setOriginalMod(MD.GT),
	ChloroplatinicAcid		= lqudacidelec	( 8401, "Chloroplatinic Acid"	, SET_FLUID				, 255,  70,  70, 255).add(UUM, LIQUID)																																.setMcfg( 0, Pt				, 1*U, Cl				, 6*U, H				, 2*U)																			.setStatsEnergetic( 200,  400).setOriginalMod(MD.GT),
	StannicChloride			= lqudacidelec	( 8402, "Stannic Chloride"		, SET_FLUID				, 210, 250, 250, 255).add(UUM, LIQUID)																																.setMcfg( 0, Sn				, 1*U, Cl				, 4*U)																									.setStatsEnergetic( 200,  400).setOriginalMod(MD.GT),
	BlackVitriol			= lqudacidelec	( 8403, "Black Vitriol"			, SET_FLUID				,  66,  66,  66, 255).add(UUM, LIQUID)																																.setMcfg( 0, Fe				, 1*U, S				, 1*U)																									.setStatsEnergetic( 200,  400).setOriginalMod(MD.GT),
	BlueVitriol				= lqudacidelec	( 8404, "Blue Vitriol"			, SET_FLUID				,  66,  66, 222, 255).add(UUM, LIQUID)																																.setMcfg( 0, Cu				, 1*U, S				, 1*U, O				, 4*U)																			.setStatsEnergetic( 200,  400).setOriginalMod(MD.GT).addIdenticalNames("RomanVitriol", "CyprusVitriol"),
	GreenVitriol			= lqudacidelec	( 8405, "Green Vitriol"			, SET_FLUID				,  66, 222,  66, 255).add(UUM, LIQUID)																																.setMcfg( 0, Fe				, 1*U, S				, 1*U, O				, 4*U)																			.setStatsEnergetic( 200,  400).setOriginalMod(MD.GT),
	RedVitriol				= lqudacidelec	( 8406, "Red Vitriol"			, SET_FLUID				, 222,  66,  66, 255).add(UUM, LIQUID)																																.setMcfg( 0, Co				, 1*U, S				, 1*U, O				, 4*U)																			.setStatsEnergetic( 200,  400).setOriginalMod(MD.GT),
	PinkVitriol				= lqudacidelec	( 8407, "Pink Vitriol"			, SET_FLUID				, 222, 111, 111, 255).add(UUM, LIQUID)																																.setMcfg( 0, Mg				, 1*U, S				, 1*U, O				, 4*U)																			.setStatsEnergetic( 200,  400).setOriginalMod(MD.GT),
	CyanVitriol				= lqudacidelec	( 8408, "Cyan Vitriol"			, SET_FLUID				, 111, 222, 222, 255).add(UUM, LIQUID)																																.setMcfg( 0, Ni				, 1*U, S				, 1*U, O				, 4*U)																			.setStatsEnergetic( 200,  400).setOriginalMod(MD.GT),
	WhiteVitriol			= lqudacidelec	( 8409, "White Vitriol"			, SET_FLUID				, 222, 222, 222, 255).add(UUM, LIQUID)																																.setMcfg( 0, Zn				, 1*U, S				, 1*U, O				, 4*U)																			.setStatsEnergetic( 200,  400).setOriginalMod(MD.GT),
	GrayVitriol				= lqudacidelec	( 8410, "Gray Vitriol"			, SET_FLUID				, 111, 111, 111, 255).add(UUM, LIQUID)																																.setMcfg( 0, Mn				, 1*U, S				, 1*U, O				, 4*U)																			.setStatsEnergetic( 200,  400).setOriginalMod(MD.GT),
	MartianVitriol			= lqudacidelec	( 8411, "Martian Vitriol"		, SET_FLUID				, 222,  66, 222, 255).add(UUM, LIQUID)																																.setMcfg( 0, Fe				, 2*U, S				, 3*U, O				,12*U)																			.setStatsEnergetic( 200,  400).setOriginalMod(MD.GT),
	VitriolOfClay			= lqudacidelec	( 8412, "Vitriol Of Clay"		, SET_FLUID				,  66, 222, 222, 255).add(UUM, LIQUID)																																.setMcfg( 0, Al				, 2*U, S				, 3*U, O				,12*U)																			.setStatsEnergetic( 200,  400).setOriginalMod(MD.GT),
	
	CarbonMonoxide			= gaschemelec	( 9838, "Carbon Monoxide"		, SET_GAS				,  10,  10,  10, 255).add(UUM, GASSES)													   																			.setMcfg( 0, C				, 1*U, O				, 1*U)																									.setStatsEnergetic( 100,  200), CO  = CarbonMonoxide,
	CarbonDioxide			= gaschemelec	( 9836, "Carbon Dioxide"		, SET_GAS				,  40,  40,  40, 255).add(UUM, GASSES)													   																			.setMcfg( 0, C				, 1*U, O				, 2*U)																									.setStatsEnergetic( 100,  200), CO2 = CarbonDioxide,
	NitrogenMonoxide		= gaschemelec	( 9837, "Nitrogen Monoxide"		, SET_GAS				, 100, 175, 255, 255).add(UUM, GASSES)													   																			.setMcfg( 0, N				, 1*U, O				, 1*U)																									.setStatsEnergetic( 100,  200), NO  = NitrogenMonoxide,
	NitrogenDioxide			= gaschemelec	( 9831, "Nitrogen Dioxide"		, SET_GAS				, 120, 190, 255, 255).add(UUM, GASSES)												   																				.setMcfg( 0, N				, 1*U, O				, 2*U)																									.setStatsEnergetic( 100,  200), NO2 = NitrogenDioxide,
	SulfurDioxide			= gaschemelec	( 9834, "Sulfur Dioxide"		, SET_GAS				, 255, 200,   0, 255).add(UUM, GASSES)													   																			.setMcfg( 0, S				, 1*U, O				, 2*U)																									.setStatsEnergetic( 100,  200).addIdenticalNames("SulphurDioxide"), SO2 = SulfurDioxide,
	SulfurTrioxide			= gaschemelec	( 9835, "Sulfur Trioxide"		, SET_GAS				, 255, 220,   0, 255).add(UUM, GASSES)													   																			.setMcfg( 0, S				, 1*U, O				, 3*U)																									.setStatsEnergetic( 100,  200).addIdenticalNames("SulphurTrioxide"), SO3 = SulfurTrioxide,
	Methane					= gaschemelec	( 9832, "Methane"				, SET_GAS				, 250, 200, 250, 255).add(UUM, GASSES, FLAMMABLE)																													.setMcfg( 0, C				, 1*U, H				, 4*U)																									.setStatsEnergetic( 100,  200), CH4 = Methane,
	HydrochloricAcid		= gasacidelec	( 9826, "Hydrochloric Acid"		, SET_GAS				,   0, 255, 128, 255).add(UUM, GASSES)																																.setMcfg( 0, H				, 1*U, Cl				, 1*U)																									.setStatsEnergetic( 100,  200), HCl = HydrochloricAcid,
	HydrogenFluoride		= gasacidelec	( 9829, "Hydrogen Fluoride"		, SET_GAS				,   0, 240, 240, 255).add(UUM, GASSES)																																.setMcfg( 0, H				, 1*U, F				, 1*U)																									.setStatsEnergetic( 189,  292), HF = HydrogenFluoride,
	
	HeliumNeon				= gaschemcent	( 9839, "Helium-Neon"			, SET_FLUID				, 255,   0, 128, 255).add(UUM, GASSES)													   																			.setMcfg( 0, He				, 1*U, Ne				, 1*U)																									, HeNe = HeliumNeon,
	
	AquaRegia				= lqudacidcent	( 9827, "Aqua Regia"			, SET_FLUID				,  64, 255,  64, 255).add(UUM, LIQUID)																																.setMcfg( 0, HNO3			, 5*U, HCl				, 8*U)																									.setStatsEnergetic( 200,  400).setOriginalMod(MD.FZ),
	SaltWater				= lqudelec		( 9804, "SaltWater"				, SET_FLUID				, 255,   0, 255, 255).add(UUM, LIQUID, UNRECYCLABLE)																												.setMcfg( 0, H2O			, 3*U, NaCl				, 1*U)																									.aspects(TC.AQUA, 2).setStatsEnergetic(CS.C, CS.C+100).setDensity(1.0).setLocal("Saltwater"),
	
	CobaltHexahydrate		= dustcent		( 8229,	"Cobalt Hexahydrate"	, SET_ROUGH				,  80,  80, 250, 255).add(UUM)																																		.setMcfg( 0, Co				, 1*U, H2O				, 6*U)																									,
	MethaneIce				= dustcent		( 9833, "Methane Ice"			, SET_SHINY				, 225, 200, 250, 255).add(G_CONTAINERS, FLAMMABLE)																													.setMcfg( 2, CH4			, 1*U, Ice				, 2*U)																									,
	NitroCarbon				= create		( 9820, "Nitro Carbon"			, SET_FLUID				,   0,  75, 100, 255).add(G_CONTAINERS, ELECTROLYSER, DECOMPOSABLE, EXPLOSIVE, FLAMMABLE, UUM)												   						.setMcfg( 0, N				, 1*U, C				, 1*U)																									,
	
	Lava					= create		( 9810,	"Lava"					, SET_STONE				, 255,  64,   0, 255).add(G_CONTAINERS, MELTING, GLOWING, LIGHTING)																																																																.setStatsEnergetic(1300, 4000).liquid(FL.Lava.make(1000), U*9).setOriginalMod(MD.MC),
	Biomass					= create		( 9840, "Biomass"				, SET_FLUID				,   0, 255,   0, 255).add(G_CONTAINERS, FLAMMABLE)																																																																				.setStatsEnergetic( 200,  400).aspects(TC.HERBA, 2).setOriginalMod(MD.FR).addIdenticalNames("BioMass"),
	BioFuel					= create		( 9841, "Bio Fuel"				, SET_FLUID				, 200, 128,   0, 255).add(G_CONTAINERS, FLAMMABLE)																																																																				.setStatsEnergetic( 100,  400).aspects(TC.HERBA, 1, TC.POTENTIA, 1).setOriginalMod(MD.FR).setLocal("Bio Diesel"),
	Ethanol					= create		( 9842, "Ethanol"				, SET_FLUID				, 255, 128,   0, 255).add(G_CONTAINERS, FLAMMABLE)																																																																				.setStatsEnergetic( 100,  400).aspects(TC.HERBA, 1, TC.POTENTIA, 1).setOriginalMod(MD.FR),
	Oil						= create		( 9850, "Oil"					, SET_FLUID				,  10,  10,  10, 255).add(G_CONTAINERS, FLAMMABLE)																																																																				.setStatsEnergetic( 100,  400).aspects(TC.MORTUUS, 2, TC.LUX, 1).setOriginalMod(MD.BC),
	Oilsands				= oredust		( 9851,	"Oilsands"				, SET_SAND				,  10,  10,  10, 255).add(FLAMMABLE)																																																																							.setStatsEnergetic( 100,  400).aspects(TC.MORTUUS, 2, TC.LUX, 1),
	CrudeOil				= oredust		( 9852,	"Crude Oil"				, SET_DULL				,  10,  10,  10, 255).add(G_CONTAINERS, FLAMMABLE)																																																																				.setStatsEnergetic( 100,  400).aspects(TC.MORTUUS, 2, TC.LUX, 1),
	Fuel					= create		( 9860, "Fuel"					, SET_FLUID				, 255, 255,   0, 255).add(G_CONTAINERS, EXPLOSIVE, FLAMMABLE)																																																																	.setStatsEnergetic( 100,  400).aspects(TC.MORTUUS, 1, TC.POTENTIA, 1).setOriginalMod(MD.BC).setLocal("Fuel Oil").addIdenticalNames("FuelOil"),
	NitroFuel				= create		( 9861, "Nitro-Fuel"			, SET_FLUID				, 200, 255,   0, 255).add(G_CONTAINERS, EXPLOSIVE, FLAMMABLE)																										.setMcfg( 0, Glyceryl		, 1*U, Fuel				, 4*U)																									.setStatsEnergetic( 100,  400).aspects(TC.MORTUUS, 1, TC.POTENTIA, 2).setOriginalMod(MD.GT),
	Kerosine				= create		( 9862, "Kerosine"				, SET_FLUID				,   0,   0, 255, 255).add(G_CONTAINERS, EXPLOSIVE, FLAMMABLE)																																																																	.setStatsEnergetic( 100,  400).aspects(TC.VOLATUS, 1, TC.POTENTIA, 1),
	Diesel					= create		( 9863, "Diesel"				, SET_FLUID				, 255, 255,   0, 255).add(G_CONTAINERS, EXPLOSIVE, FLAMMABLE)																																																																	.setStatsEnergetic( 100,  400).aspects(TC.MORTUUS, 1, TC.POTENTIA, 1),
	Petrol					= create		( 9864, "Petrol"				, SET_FLUID				, 255,   0,   0, 255).add(G_CONTAINERS, EXPLOSIVE, FLAMMABLE)																																																																	.setStatsEnergetic( 100,  400).aspects(TC.MORTUUS, 1, TC.POTENTIA, 1).addIdenticalNames("Gasoline"),
	Propane					= create		( 9865, "Propane"				, SET_FLUID				, 255,  20,  20, 255).add(G_CONTAINERS, EXPLOSIVE, FLAMMABLE)																																																																	.setStatsEnergetic( 100,  200).aspects(TC.MORTUUS, 1, TC.POTENTIA, 1),
	Butane					= create		( 9866, "Butane"				, SET_FLUID				, 255,  40,  40, 255).add(G_CONTAINERS, EXPLOSIVE, FLAMMABLE)																																																																	.setStatsEnergetic( 100,  200).aspects(TC.MORTUUS, 1, TC.POTENTIA, 1),
	Creosote				= create		( 9870, "Creosote"				, SET_FLUID				, 128,  64,   0, 255).add(G_CONTAINERS, FLAMMABLE, LIQUID)																																																																		.setStatsEnergetic( 100,  400).aspects(TC.ARBOR, 2, TC.LUX, 1).setOriginalMod(MD.RC).addIdenticalNames("Creosote Oil"),
	FishOil					= create		( 9871, "Fish Oil"				, SET_FLUID				, 255, 196,   0, 255).add(G_CONTAINERS, FLAMMABLE, FOOD)																																																																		.setStatsEnergetic( 100,  400).aspects(TC.CORPUS, 2, TC.LUX, 1),
	SeedOil					= create		( 9872, "Seed Oil"				, SET_FLUID				, 196, 255,   0, 255).add(G_CONTAINERS, FLAMMABLE, FOOD)																																																																		.setStatsEnergetic( 100,  400).aspects(TC.GRANUM, 2, TC.LUX, 1).setOriginalMod(MD.FR),
	HempOil					= create		( 9873, "Hemp Oil"				, SET_FLUID				, 196, 255,   0, 255).add(G_CONTAINERS, FLAMMABLE, FOOD)																																																																		.setStatsEnergetic( 100,  400).aspects(TC.GRANUM, 2, TC.LUX, 1),
	LinOil					= create		( 9874, "Lin Oil"				, SET_FLUID				, 196, 255,   0, 255).add(G_CONTAINERS, FLAMMABLE, FOOD)																																																																		.setStatsEnergetic( 100,  400).aspects(TC.GRANUM, 2, TC.LUX, 1),
	SunflowerOil			= create		( 9875, "Sunflower Oil"			, SET_FLUID				, 216, 189,  17, 255).add(G_CONTAINERS, FLAMMABLE, FOOD)																																																																		.setStatsEnergetic( 100,  400).aspects(TC.GRANUM, 2, TC.LUX, 1),
	NutOil					= create		( 9876, "Nut Oil"				, SET_FLUID				, 235, 173,  70, 255).add(G_CONTAINERS, FLAMMABLE, FOOD)																																																																		.setStatsEnergetic( 100,  400).aspects(TC.GRANUM, 2, TC.LUX, 1),
	OliveOil				= create		( 9877, "Olive Oil"				, SET_FLUID				,  63, 146,   0, 255).add(G_CONTAINERS, FLAMMABLE, FOOD)																																																																		.setStatsEnergetic( 100,  400).aspects(TC.HERBA, 2, TC.LUX, 1),
	FryingOilHot			= create		( 9880, "FryingOilHot"			, SET_FLUID				, 200, 196,   0, 255).add(G_CONTAINERS, FLAMMABLE, FOOD)																																																																		.setStatsEnergetic( 100,  400).aspects(TC.AQUA, 1, TC.IGNIS, 1).setLocal("Hot Frying Oil"),
	Glue					= create		( 9881, "Glue"					, SET_FLUID				, 200, 196,   0, 255).add(G_CONTAINERS, FLAMMABLE)																																																																				.setStatsEnergetic( 150,  500).aspects(TC.LIMUS, 2),
	Lubricant				= create		( 9882, "Lubricant"				, SET_FLUID				, 255, 196,   0, 255).add(G_CONTAINERS)																																																																							.setStatsEnergetic( 150,  500).aspects(TC.AQUA, 1, TC.MACHINA, 1),
	ConstructionFoam		= create		( 9883,	"Construction Foam"		, SET_DULL				, 128, 128, 128, 255).add(G_CONTAINERS, DUSTS, STONE, BRITTLE, MELTING, NO_ADVANCED_TOOLS)																																																										.setStatsEnergetic( 500, 2000).setOriginalMod(MD.IC2).setLocal("C-Foam"),
	UUAmplifier				= create		( 9884,	"UU-Amplifier"			, SET_FLUID				, 196,   0, 255, 255).add(G_CONTAINERS)																																																																							.setStatsEnergetic(  50,  500).setOriginalMod(MD.GT),
	UUMatter				= create		( 9885,	"UU-Matter"				, SET_FLUID				,  96,   0, 128, 255).add(G_CONTAINERS, GLOWING)																																																																				.setStatsEnergetic(  50,  500).setOriginalMod(MD.IC2),
	Latex					= create		( 9886, "Latex"					, SET_FLUID				, 250, 250, 250, 255).add(G_CONTAINERS)																																																																							.aspects(TC.AQUA, 1, TC.LIMUS, 1);
	
	public static final OreDictMaterial
	Glass					= create		( 8001, "Glass"					, SET_GLASS				, 250, 250, 250,  35).add(G_GEM_TRANSPARENT,CENTRIFUGE,DECOMPOSABLE,FURNACE,CRYSTAL,UNRECYCLABLE,MELTING,EXTRUDER,EXTRUDER_SIMPLE,BRITTLE,NO_ADVANCED_TOOLS, UUM)	.setMcfg( 0, SiO2			, 1*U)																															.aspects(TC.VITREUS, 2).setQuality( 1.0F, 4,  0).setMeltingPoint(1200).setOriginalMod(MD.MC),
	Flint					= create		( 8002, "Flint"					, SET_FLINT				,   0,  32,  64, 255).add(G_GEM, STONE, BRITTLE, DIRTY_DUSTS, CENTRIFUGE, DECOMPOSABLE, MORTAR, NO_ADVANCED_TOOLS)													.setMcfg( 0, SiO2			, 1*U)																															.aspects(TC.TERRA, 1, TC.INSTRUMENTUM, 1).setQuality( 2.5F, 48,  1).setOriginalMod(MD.MC);

	public static final OreDictMaterial
	RareEarth				= oredust		( 9100, "Rare Earth"			, SET_FINE				, 128, 128, 100, 255).add(																				)																																																						.aspects(TC.VITREUS, 1, TC.LUCRUM, 1).setOriginalMod(MD.GT),
	Chalk					= oredustelec	( 9112, "Chalk"					, SET_FINE				, 250, 250, 250, 255).add(FURNACE, MORTAR																).setSmelting(CaCO3, 2*U/3)									.setMcfg( 0, Ca				, 1*U, C 				, 1*U, O				, 3*U)																			,
	Dolomite				= oredustelec	( 9163, "Dolomite"				, SET_FLINT				, 225, 205, 205, 255).add(FURNACE, MORTAR																).setSmelting(CaCO3, U/2)									.setMcfg( 0, Ca				, 1*U, Mg				, 1*U, C				, 2*U, O				, 6*U)													, // CaMg(CO3)2
	Asbestos				= oredustelec	( 9103, "Asbestos"				, SET_DULL				, 230, 230, 230, 255).add(UUM																			)															.setMcfg( 0, Mg				, 3*U, Si				, 2*U, H2O				, 6*U, O				, 7*U)													, // Mg3Si2O5(OH)4
	Talc					= oredustelec	( 9168, "Talc"					, SET_DULL				,  90, 180,  90, 255)																																				.setMcfg( 0, Mg				, 3*U, Si				, 4*U, H2O				, 3*U, O				,11*U)													, // H2Mg3(SiO3)4
	Soapstone				= oredustelec	( 9169, "Soapstone"				, SET_DULL				,  95, 145,  95, 255)																																				.setMcfg( 0, Mg				, 3*U, Si				, 4*U, H2O				, 3*U, O				,11*U)													, // H2Mg3(SiO3)4
	Pyrite					= oredustelec	( 9125, "Pyrite"				, SET_ROUGH				, 150, 120,  40, 255).add(BLACKLISTED_SMELTER, MORTAR, MAGNETIC_PASSIVE, UUM							).setSmelting(Fe,   U/4)									.setMcfg( 0, Fe				, 1*U, S 				, 2*U)																									,
	PotassiumFeldspar		= oredustelec	( 9140, "Potassium Feldspar"	, SET_FINE				, 120,  40,  40, 255)																																				.setMcfg( 0, K				, 1*U, Al				, 1*U, Si				, 3*U, O				, 8*U)													,
	Biotite					= oredustelec	( 9141, "Biotite"				, SET_METALLIC			,  20,  30,  20, 255)																																				.setMcfg( 0, K				, 1*U, Mg				, 3*U, Al				, 3*U, F				, 2*U, Si				, 3*U, O				,10*U)	,
	Gneiss					= create		( 9170,	"Gneiss"				, SET_STONE				, 136, 136, 142, 255).add(G_STONE, STONE																)																																																						.setQuality( 4.0F, 16, 1),
	Greenschist				= create		( 9171,	"Greenschist"			, SET_STONE				, 105, 170, 105, 255).add(G_STONE, STONE																)																																																						.setQuality( 4.0F, 24, 1).setLocal("Green Schist"),
	Greenstone				= create		( 9172,	"Greenstone"			, SET_STONE				,  52, 252,  52, 255).add(G_STONE, STONE																)																																																						,
	Greywacke				= create		( 9173,	"Greywacke"				, SET_STONE				, 176, 176, 176, 255).add(G_STONE, STONE																)																																																						.setQuality( 4.0F, 16, 1),
	Komatiite				= create		( 9177,	"Komatiite"				, SET_STONE				, 170, 170, 105, 255).add(G_STONE, STONE																)																																																						.setQuality( 4.0F, 16, 1),
	Siltstone				= create		( 9178,	"Siltstone"				, SET_STONE				, 250, 205, 205, 255).add(G_STONE, STONE																)																																																						.setQuality( 4.0F, 16, 0),
	Rhyolite				= create		( 9179,	"Rhyolite"				, SET_STONE				, 121, 121, 121, 255).add(G_STONE, STONE																)																																																						.setQuality( 4.0F, 16, 1),
	Quartzite				= create		( 9180,	"Quartzite"				, SET_QUARTZ			, 230, 205, 205, 255).add(G_QUARTZ_ORES, CRYSTALLISABLE, QUARTZ, BLACKLISTED_SMELTER, MORTAR			).setSmelting(SiO2, U/3)																																																.setQuality( 3.7F, 32, 1),
	Migmatite				= create		( 9181,	"Migmatite"				, SET_STONE				,  70,  40,  40, 255).add(G_STONE, STONE																)																																																						.setQuality( 4.0F, 16, 1),
	Epidote					= create		( 9182,	"Epidote"				, SET_STONE				, 128, 128, 128, 255).add(G_STONE, STONE																)																																																						,
	Emery					= oredust		( 9183,	"Emery"					, SET_STONE				, 128, 128, 128, 255)																																																																											,
	Blueschist				= create		( 9184,	"Blueschist"			, SET_STONE				, 105, 105, 170, 255).add(G_STONE, STONE																)																																																						.setQuality( 4.0F, 24, 1).setLocal("Blue Schist"),
	Bluestone				= create		( 9185,	"Bluestone"				, SET_STONE				,  52,  52, 252, 255).add(G_STONE, STONE																)																																																						,
	Chert					= create		( 9186,	"Chert"					, SET_STONE				, 105,  10,  10, 255).add(G_STONE, STONE																)																																																						.setQuality( 4.0F, 16, 0),
	Dacite					= create		( 9187,	"Dacite"				, SET_STONE				, 131, 131, 131, 255).add(G_STONE, STONE																)																																																						.setQuality( 4.0F, 16, 1),
	Andesite				= create		( 9188,	"Andesite"				, SET_STONE				, 191, 191, 191, 255).add(G_STONE, STONE																)																																																						.setQuality( 4.0F, 16, 1),
	Shale					= create		( 9190,	"Shale"					, SET_STONE				, 142, 142, 168, 255).add(G_STONE, STONE																)																																																						.setQuality( 4.0F, 16, 0),
	Eclogite				= create		( 9191,	"Eclogite"				, SET_STONE				,  90,  40,  40, 255).add(G_STONE, STONE																)																																																						.setQuality( 4.0F, 16, 1);
	
	public static final OreDictMaterial
	Ash						= dustcent		( 8200, "Ashes"					, SET_DULL				, 150, 150, 150, 255).add(MORTAR, UUM)														   																		.setMcfg( 0, C 				, 1*U)																															.aspects(TC.PERDITIO, 1).addIdenticalNames("Ash").setOriginalMod(MD.FR),
	DarkAsh					= dustelec		( 8201, "Dark Ashes"			, SET_DULL				,  50,  50,  50, 255).add(MORTAR, UUM)																																.setMcfg( 1, C 				, 2*U)																															.aspects(TC.IGNIS, 1, TC.PERDITIO, 1).addIdenticalNames("DarkAsh", "AshDark", "AshesDark").setOriginalMod(MD.GT),
	VolcanicAsh				= dustcent		( 8202, "Volcanic Ashes"		, SET_FLINT				,  60,  50,  50, 255).add(MORTAR)																																	.setMcfg( 0, Flint			, 6*U, Fe				, 1*U, Mg				, 1*U)																			.aspects(TC.IGNIS, 2, TC.PERDITIO, 1).addIdenticalNames("VolcanicAsh", "AshVolcanic", "AshesVolcanic"),
	Graphite				= oredustdcmp	( 9174,	"Graphite"				, SET_DULL				, 128, 128, 128, 255).add(BLACKLISTED_SMELTER, BRITTLE, MORTAR, UUM										)															.setMcfg( 0, C				, 1*U)																															.setQuality( 5.0F, 32, 2).setSmelting(C, U/2).setBurning(Ash, U/4).setStatsEnergetic(2000, C.mBoilingPoint),
	Niter					= create		( 8206, "Niter"					, SET_FLINT				, 255, 200, 200, 255).add(G_GEM_ORES, CENTRIFUGE, DECOMPOSABLE, CRYSTAL, BRITTLE)																					.setMcfg( 0, KNO3			, 1*U)																															,
	Phosphorus				= create		( 8208, "Phosphorus"			, SET_FLINT				, 255, 255,   0, 255).add(G_GEM_ORES, CENTRIFUGE, DECOMPOSABLE, FLAMMABLE, EXPLOSIVE, UUM)																			.setMcfg( 0, Ca				, 3*U, Phosphate		, 2*U)																									.setOriginalMod(MD.FR),
	Apatite					= create		( 8209, "Apatite"				, SET_DIAMOND			, 120, 180, 250, 255).add(G_GEM_ORES, ELECTROLYSER, DECOMPOSABLE, FLAMMABLE, CRYSTAL, BRITTLE, CRYSTALLISABLE, MORTAR, UUM)											.setMcfg( 0, Ca				, 5*U, Phosphate		, 3*U, Cl				, 1*U)																			.setOriginalMod(MD.FR).aspects(TC.MESSIS, 2),
	Phosphorite				= create		( 8226, "Phosphorite"			, SET_DIAMOND			,  50,  50,  65, 255).add(G_GEM_ORES, ELECTROLYSER, DECOMPOSABLE, FLAMMABLE, CRYSTAL, BRITTLE, CRYSTALLISABLE, MORTAR, UUM)											.setMcfg( 0, Ca				, 5*U, Phosphate		, 3*U, F				, 1*U)																			.aspects(TC.MESSIS, 2),
	Basalz					= create		( 8247,	"Basalz"				, SET_SHINY				, 100,  81,  81, 255).add(G_BLAZE, GLOWING, MAGICAL, BRITTLE, NO_ADVANCED_TOOLS, MORTAR)																																																										.aspects(TC.PRAECANTIO, 2, TC.POTENTIA, 4).setOriginalMod(MD.TE),
	Blitz					= create		( 8248,	"Blitz"					, SET_SHINY				, 214, 189,  21, 255).add(G_BLAZE, GLOWING, MAGICAL, BRITTLE, NO_ADVANCED_TOOLS, MORTAR)																																																										.aspects(TC.PRAECANTIO, 2, TC.AER, 4).setOriginalMod(MD.TE),
	Blizz					= create		( 8210,	"Blizz"					, SET_SHINY				, 220, 233, 255, 255).add(G_BLAZE, GLOWING, MAGICAL, BRITTLE, NO_ADVANCED_TOOLS, MORTAR)																																																										.aspects(TC.PRAECANTIO, 2, TC.GELUM, 4).setOriginalMod(MD.TE),
	Blaze					= create		( 8211, "Blaze"					, SET_POWDER			, 255, 200,   0, 255).add(G_BLAZE, GLOWING, MAGICAL, BRITTLE, NO_ADVANCED_TOOLS, MORTAR, CENTRIFUGE, DECOMPOSABLE, UNBURNABLE, BURNING, MELTING)					.setMcfg( 2, DarkAsh		, 1*U, S				, 1*U, Ma				, 1*U)																			.aspects(TC.PRAECANTIO, 2, TC.IGNIS, 4).setQuality(2.0F, 16, 1).setMeltingPoint(4000).setOriginalMod(MD.MC),
	Obsidian				= create		( 8214, "Obsidian"				, SET_STONE				,  80,  50, 100, 255).add(G_STONE, STONE, BRITTLE, ELECTROLYSER, DECOMPOSABLE, NO_ADVANCED_TOOLS)																	.setMcfg(64, Mg				, 1*U, Fe				, 1*U, Si				, 2*U, O 				, 8*U)													.setSmelting(Lava, U).setStatsEnergetic(1300, 4000).setQuality(3.0F, 32, 3).setOriginalMod(MD.MC),
	Clay					= dustelec		( 8215, "Clay"					, SET_ROUGH				, 200, 200, 220, 255).add(MORTAR)														   																			.setMcfg( 0, Na				, 2*U, Li				, 1*U, Al				, 2*U, Si				, 2*U)													.setMeltingPoint(2000).setOriginalMod(MD.MC),
	Ceramic					= dust			( 8225, "Ceramic"				, SET_ROUGH				, 220, 130,  70, 255).add(PLATES, BRITTLE, MORTAR)														   															.setMcfg( 0, Clay			, 1*U)																															.steal(Clay).setMeltingPoint(2000).setPulver(Clay, U/2).setCompressing(null, 0).setBending(null, 0).setForging(null, 0).setSmashing(null, 0).setOriginalMod(MD.GT),
	Porcelain				= mixdust		( 8273, "Porcelain"				, SET_ROUGH				, 195, 195, 222, 255).add(PLATES, BRITTLE, MORTAR)														   															.setMcfg( 0, Clay			, 2*U, SiO2				, 1*U, PotassiumFeldspar, 1*U)																			.setMeltingPoint(1800).setCompressing(null, 0).setBending(null, 0).setForging(null, 0).setSmashing(null, 0).setOriginalMod(MD.IHL),
	Paper					= dust			( 8216, "Paper"					, SET_PAPER				, 250, 250, 250, 255).add(MULTIPLATES, MORTAR)																																																																					.aspects(TC.COGNITO, 1).setBurning(Ash, U / 9).setFurnaceBurnTime(TICKS_PER_SMELT/ 8).setOriginalMod(MD.MC),
	Rubber					= create		( 8217, "Rubber"				, SET_SHINY				,   0,   0,   0, 255).add(G_INGOT_MACHINE, APPROXIMATE, FLAMMABLE, EXTRUDER, EXTRUDER_SIMPLE, MORTAR, BOUNCY, STRETCHY, FURNACE, UUM)								.setMcfg( 0, C 				, 5*U, H 				, 8*U)																									.aspects(TC.MOTUS, 2).setMeltingPoint(410).setBurning(Ash, U / 9).setSmelting(null, 2*U/3).setQuality(1.5F, 16, 0).setOriginalMod(MD.IC2),
	Plastic					= create		( 8218, "Plastic"				, SET_DULL				, 200, 200, 200, 255).add(G_INGOT_MACHINE, APPROXIMATE, FLAMMABLE, EXTRUDER, EXTRUDER_SIMPLE, MORTAR, BOUNCY, BRITTLE, FURNACE, UUM)								.setMcfg( 0, C 				, 1*U, H 				, 2*U)																									.aspects(TC.MOTUS, 2).setMeltingPoint(423).setBurning(Ash, U / 9).setSmelting(null, 2*U/3).setQuality(3.0F, 32, 1).setOriginalMod(MD.MFR),
	Bone					= dustelec		( 8219, "Bone"					, SET_DULL				, 250, 250, 250, 255).add(MORTAR, UUM)													   																			.setMcfg( 3, Ca				, 1*U)																															.aspects(TC.MORTUUS, 2, TC.CORPUS, 1).setOriginalMod(MD.MC),
	Gunpowder				= dust			( 8220, "Gunpowder"				, SET_DULL				, 128, 128, 128, 255).add(EXPLOSIVE, FLAMMABLE, UUM)																												.setMcfg( 3, C				, 2*U, S				, 1*U, KNO3				, 1*U)																			.aspects(TC.PERDITIO, 3, TC.IGNIS, 4).setBurning(Ash, U / 9).setOriginalMod(MD.MC),
	Wood					= create		( 8221, "Wood"					, SET_WOOD				, 100,  50,   0, 255).add(G_WOOD, WOOD, FLAMMABLE, NO_ADVANCED_TOOLS, APPROXIMATE, UUM)																				.setMcfg( 0, C 				, 6*U, H2O				,15*U)																									.aspects(TC.ARBOR, 2).setFurnaceBurnTime(TICKS_PER_SMELT/ 2).setBurning(Ash, U / 9).setSmelting(Ash, U / 4).setQuality(2.0F, 16, 0).setStatsEnergetic(400, 500).setOriginalMod(MD.MC),
	WoodSealed				= create		( 8222, "WoodSealed"			, SET_WOOD				,  80,  40,   0, 255).add(G_WOOD, WOOD, FLAMMABLE, NO_ADVANCED_TOOLS, COATED)																						.setMcfg( 0, Wood			, 1*U)																															.aspects(TC.ARBOR, 2, TC.FABRICO, 1).setFurnaceBurnTime(TICKS_PER_SMELT/ 2).setAllToTheOutputOf(Wood).setQuality(3.0F, 24, 0).setStatsEnergetic(500, 600).setLocal("Sealed Wood"),
	WoodPolished			= create		( 8267, "WoodPolished"			, SET_WOOD				,  60,  30,   0, 255).add(G_WOOD, WOOD, FLAMMABLE, NO_ADVANCED_TOOLS, COATED)																						.setMcfg( 0, Wood			, 1*U)																															.aspects(TC.ARBOR, 2, TC.FABRICO, 1).setFurnaceBurnTime(TICKS_PER_SMELT/ 2).setAllToTheOutputOf(Wood).setQuality(3.0F, 24, 0).setStatsEnergetic(500, 600).setLocal("Polished Wood"),
	LiveRoot				= dustcent		( 8223, "LiveRoot"				, SET_WOOD				, 220, 200,   0, 255).add(WOOD, MAGICAL, MORTAR)																													.setMcfg( 3, Wood			, 3*U, Ma				, 1*U)																									.aspects(TC.ARBOR, 2, TC.VICTUS, 2, TC.PRAECANTIO, 1).setLocal("Liveroot").setFurnaceBurnTime(TICKS_PER_SMELT).setBurning(Ash, U / 9).setStatsEnergetic(1178, 2465).setOriginalMod(MD.TF),
	WoodRubber				= create		( 8224, "WoodRubber"			, SET_WOOD				, 120,  60,   0, 255).add(G_WOOD, WOOD, FLAMMABLE, NO_ADVANCED_TOOLS, APPROXIMATE)																					.setMcfg( 0, C 				, 1*U, O 				, 1*U, H 				, 1*U)																			.aspects(TC.ARBOR, 2).setFurnaceBurnTime(TICKS_PER_SMELT/ 4).setBurning(Ash, U / 4).setSmelting(Ash, U / 2).setQuality(1.5F, 12, 0).setStatsEnergetic(350, 450).setLocal("Rubber Wood").setOriginalMod(MD.IC2),
	Peanutwood				= create		(-8227,	"Peanut Wood"			, SET_WOOD				, 120,  60,   0, 255).add(G_WOOD, WOOD, FLAMMABLE)																													.setMcfg( 0, Wood			, 1*U)																															.aspects(TC.ARBOR, 2).setFurnaceBurnTime(TICKS_PER_SMELT/ 2).setAllToTheOutputOf(Wood).steal(Wood).setStatsEnergetic(350, 450).addIdenticalNames("Peanutwood"),
	Dynamite				= dust			( 8249, "Dynamite"				, SET_ROUGH				, 111, 131, 111, 255).add(EXPLOSIVE, FLAMMABLE, UUM)																												.setMcfg( 0, Glyceryl		, 1*U, Wood				, 1*U)																									.aspects(TC.PERDITIO, 4, TC.IGNIS, 3).setBurning(Ash, U / 9),
	Asphalt					= dust			( 8266, "Asphalt"				, SET_ROUGH				,  88,  88,  99, 255).add(FURNACE, MELTING, EXTRUDER, EXTRUDER_SIMPLE, MOLTEN)														   																																															.aspects(TC.TERRA, 1, TC.ITER, 1),
	Indigo					= dust			( 8228,	"Indigo"				, SET_LEAF				, 255, 128, 255, 255).add(FLAMMABLE)																																																																							.aspects(TC.SENSUS, 1).setOriginalMod(MD.RP),
	Wax						= dust			( 8235, "Wax"					, SET_FOOD				, 250, 250, 250, 255).add(FOILS, PLATES, FURNACE, MELTING, MORTAR, EXTRUDER, EXTRUDER_SIMPLE)																																																									.aspects(TC.PERDITIO	, 1).setMeltingPoint( 350),
	WaxBee					= dust			( 8236, "WaxBee"				, SET_FOOD				, 250, 250, 100, 255).add(FOILS, PLATES, FURNACE, MELTING, MORTAR, EXTRUDER, EXTRUDER_SIMPLE, FOOD)																																																								.aspects(TC.BESTIA		, 1).setMeltingPoint( 350).setOriginalMod(MD.FR).setLocal("Bees Wax").addIdenticalNames("BeesWax", "Beeswax", "BeeWax", "Beewax"),
	WaxRefractory			= dust			( 8237, "WaxRefractory"			, SET_FOOD				, 250,  50,  50, 255).add(FOILS, PLATES, FURNACE, MELTING, MORTAR, EXTRUDER, EXTRUDER_SIMPLE, UNBURNABLE)																																																						.aspects(TC.GELUM		, 1).setMeltingPoint(2600).setOriginalMod(MD.FR).setLocal("Refractory Wax").addIdenticalNames("RefractoryWax", "Refractorywax"),
	WaxParaffin				= dust			( 8238, "WaxParaffin"			, SET_FOOD				, 210, 210, 250, 255).add(FOILS, PLATES, FURNACE, MELTING, MORTAR, EXTRUDER, EXTRUDER_SIMPLE)																																																									.aspects(TC.FABRICO		, 1).setMeltingPoint( 350).setOriginalMod(MD.GT).setLocal("Paraffin Wax").addIdenticalNames("ParaffinWax", "Paraffinwax"),
	WaxPlant				= dust			( 8239, "WaxPlant"				, SET_FOOD				, 210, 250, 210, 255).add(FOILS, PLATES, FURNACE, MELTING, MORTAR, EXTRUDER, EXTRUDER_SIMPLE, FOOD)																																																								.aspects(TC.HERBA		, 1).setMeltingPoint( 350).setOriginalMod(MD.HaC).setLocal("Plant Wax"),
	WaxMagic				= dust			( 8240, "WaxMagic"				, SET_FOOD				, 200,  80, 200, 255).add(FOILS, PLATES, FURNACE, MELTING, MORTAR, EXTRUDER, EXTRUDER_SIMPLE, MAGICAL)																																																							.aspects(TC.PRAECANTIO	, 1).setMeltingPoint( 350).setLocal("Magic Wax"),
	Tallow					= dust			( 8244, "Tallow"				, SET_FOOD				, 220, 200, 100, 255).add(MELTING, MAGICAL, MOLTEN)																																																																				.aspects(TC.CORPUS, 2, TC.HUMANUS, 1, TC.PRAECANTIO, 1).setMeltingPoint(350).setLocal("Magic Tallow"),
	Leather					= create		( 8241, "Leather"				, SET_ROUGH				, 141,  65,  37, 255).add(FLAMMABLE)																																																																							.aspects(TC.PANNUS, 1, TC.TUTAMEN, 1).setBurning(Ash, U/9).setSmelting(Ash, U/9).setOriginalMod(MD.MC),
	
	Black					= dust			( 8250, "Black"					, SET_FOOD				,  32,  32,  32, 255)																																																																											.aspects(TC.SENSUS, 1).setOriginalMod(MD.MC),
	Red						= dust			( 8251, "Red"					, SET_FOOD				, 255,   0,   0, 255)																																																																											.aspects(TC.SENSUS, 1).setOriginalMod(MD.MC),
	Green					= dust			( 8252, "Green"					, SET_FOOD				,   0, 255,   0, 255)																																																																											.aspects(TC.SENSUS, 1).setOriginalMod(MD.MC),
	Brown					= dust			( 8253, "Brown"					, SET_FOOD				,  96,  64,   0, 255)																																																																											.aspects(TC.SENSUS, 1).setOriginalMod(MD.MC),
	Blue					= dust			( 8254, "Blue"					, SET_FOOD				,   0,   0, 255, 255)																																																																											.aspects(TC.SENSUS, 1).setOriginalMod(MD.MC),
	Purple					= dust			( 8255, "Purple"				, SET_FOOD				, 128,   0, 128, 255)																																																																											.aspects(TC.SENSUS, 1).setOriginalMod(MD.MC),
	Cyan					= dust			( 8256, "Cyan"					, SET_FOOD				,   0, 255, 255, 255)																																																																											.aspects(TC.SENSUS, 1).setOriginalMod(MD.MC),
	LightGray				= dust			( 8257, "Light Gray"			, SET_FOOD				, 192, 192, 192, 255)																																																																											.aspects(TC.SENSUS, 1).setOriginalMod(MD.MC),
	Gray					= dust			( 8258, "Gray"					, SET_FOOD				, 128, 128, 128, 255)																																																																											.aspects(TC.SENSUS, 1).setOriginalMod(MD.MC),
	Pink					= dust			( 8259, "Pink"					, SET_FOOD				, 255, 192, 192, 255)																																																																											.aspects(TC.SENSUS, 1).setOriginalMod(MD.MC),
	Lime					= dust			( 8260, "Lime"					, SET_FOOD				, 128, 255, 128, 255)																																																																											.aspects(TC.SENSUS, 1).setOriginalMod(MD.MC),
	Yellow					= dust			( 8261, "Yellow"				, SET_FOOD				, 255, 255,   0, 255)																																																																											.aspects(TC.SENSUS, 1).setOriginalMod(MD.MC),
	LightBlue				= dust			( 8262, "Light Blue"			, SET_FOOD				, 128, 128, 255, 255)																																																																											.aspects(TC.SENSUS, 1).setOriginalMod(MD.MC),
	Magenta					= dust			( 8263, "Magenta"				, SET_FOOD				, 255,   0, 255, 255)																																																																											.aspects(TC.SENSUS, 1).setOriginalMod(MD.MC),
	Orange					= dust			( 8264, "Orange"				, SET_FOOD				, 255, 128,   0, 255)																																																																											.aspects(TC.SENSUS, 1).setOriginalMod(MD.MC),
	White					= dust			( 8265, "White"					, SET_FOOD				, 255, 255, 255, 255)																																																																											.aspects(TC.SENSUS, 1).setOriginalMod(MD.MC);
	
	static {
		H2O.setSolidifying(Ice, U);
		Lava.setSolidifying(Obsidian, U).setDensity(Obsidian.mGramPerCubicCentimeter);
	}
	
	public static final OreDictMaterial
	MeatCooked				= dust			( 9701, "MeatCooked"			, SET_FINE				, 150,  60,  20, 255).add(FOOD, MORTAR, INGOTS, MEAT, MELTING, EXTRUDER, EXTRUDER_SIMPLE)																																																										.aspects(TC.CORPUS, 2).setStatsEnergetic(477, 550).setLocal("Cooked Meat").addIdenticalNames("Meat").setOriginalMod(MD.MFR),
	MeatRaw					= dust			( 9700, "MeatRaw"				, SET_FINE				, 255, 100, 100, 255).add(FOOD, MORTAR, INGOTS, MEAT, MELTING, EXTRUDER, EXTRUDER_SIMPLE, FURNACE)																																																								.aspects(TC.CORPUS, 1).setStatsEnergetic(477, 550).setLocal("Raw Meat").setSmelting(MeatCooked, U).setForging(MeatCooked, U).setOriginalMod(MD.MFR),
	MeatRotten				= dust			( 9710, "MeatRotten"			, SET_FINE				, 255, 150, 100, 255).add(FOOD, MORTAR, INGOTS, MEAT, ROTTEN)																																																																	.aspects(TC.CORPUS, 1, TC.MORTUUS, 1).setStatsEnergetic(477, 550).setLocal("Rotten Meat").setOriginalMod(MD.MC),
	FishCooked				= dust			( 9711, "FishCooked"			, SET_FINE				, 150, 120,  20, 255).add(FOOD, MORTAR, INGOTS, MEAT, MELTING, EXTRUDER, EXTRUDER_SIMPLE)																																																										.aspects(TC.CORPUS, 2).setStatsEnergetic(477, 550).setLocal("Cooked Fishmeal"),
	FishRaw					= dust			( 9712, "FishRaw"				, SET_FINE				, 255, 150, 100, 255).add(FOOD, MORTAR, INGOTS, MEAT, MELTING, EXTRUDER, EXTRUDER_SIMPLE, FURNACE)																																																								.aspects(TC.CORPUS, 1).setStatsEnergetic(477, 550).setLocal("Raw Fishmeal").setSmelting(FishCooked, U).setForging(FishCooked, U).setOriginalMod(MD.MaCu),
	FishRotten				= dust			( 9713, "FishRotten"			, SET_FINE				, 220, 200, 100, 255).add(FOOD, MORTAR, INGOTS, MEAT, ROTTEN)																																																																	.aspects(TC.CORPUS, 1, TC.MORTUUS, 1).setStatsEnergetic(477, 550).setLocal("Rotten Fishmeal"),
	Wheat					= dust			( 9702, "Wheat"					, SET_POWDER			, 255, 255, 196, 255).add(FOOD, MORTAR, FLAMMABLE)																																																																				.aspects(TC.MESSIS, 2).setBurning(Ash, U / 9).setOriginalMod(MD.MC).addIdenticalNames("Flour"),
	Sugar					= dustelec		( 9703, "Sugar"					, SET_CUBE				, 250, 250, 250, 255).add(FOOD, MORTAR, FURNACE, MELTING, UUM)																										.setMcfg( 0, C				, 2*U, H2O				, 5*U, O				,25*U)																			.aspects(TC.HERBA, 1, TC.AQUA, 1, TC.AER, 1).setStatsEnergetic(320, 500).setOriginalMod(MD.MC),
	Barley					= dust			( 9704, "Barley"				, SET_POWDER			, 196, 255, 196, 255).add(FOOD, MORTAR, FLAMMABLE)																																																																				.aspects(TC.MESSIS, 2).setBurning(Ash, U / 9).setOriginalMod(MD.HaC),
	Rye						= dust			( 9705, "Rye"					, SET_POWDER			, 255, 230, 180, 255).add(FOOD, MORTAR, FLAMMABLE)																																																																				.aspects(TC.MESSIS, 2).setBurning(Ash, U / 9).setOriginalMod(MD.HaC),
	Rice					= dust			( 9706, "Rice"					, SET_POWDER			, 252, 252, 240, 255).add(FOOD, MORTAR, FLAMMABLE)																																																																				.aspects(TC.MESSIS, 2).setBurning(Ash, U / 9).setOriginalMod(MD.HaC),
	Oat						= dust			( 9707, "Oat"					, SET_POWDER			, 240, 240, 222, 255).add(FOOD, MORTAR, FLAMMABLE)																																																																				.aspects(TC.MESSIS, 2).setBurning(Ash, U / 9).setOriginalMod(MD.HaC).addIdenticalNames("Oats"),
	Corn					= dust			( 9708, "Corn"					, SET_POWDER			, 250, 240, 111, 255).add(FOOD, MORTAR, FLAMMABLE)																																																																				.aspects(TC.MESSIS, 2).setBurning(Ash, U / 9).setOriginalMod(MD.HaC),
	Potato					= dust			( 9709, "Potato"				, SET_POWDER			, 240, 240, 164, 255).add(FOOD, MORTAR)																																																																							.aspects(TC.MESSIS, 2).setBurning(Ash, U / 9).setOriginalMod(MD.MC),
	Tofu					= dust			( 9778, "Tofu"					, SET_FOOD				, 222, 222, 222, 255).add(FOOD, MORTAR, INGOTS, MELTING, EXTRUDER, EXTRUDER_SIMPLE)																																																												.aspects(TC.FAMES, 1, TC.HERBA, 1).setOriginalMod(MD.HaC).setStatsEnergetic(422, 500),
	SoylentGreen			= dust			( 9779, "Soylent Green"			, SET_FOOD				,   0, 222,   0, 255).add(FOOD, MORTAR, INGOTS, MELTING, EXTRUDER, EXTRUDER_SIMPLE)																																																												.aspects(TC.FAMES, 1, TC.CORPUS, 1).setOriginalMod(MD.GT).setStatsEnergetic(422, 500).setLocal("Emerald Green"),
	Cheese					= oredust		( 9780, "Cheese"				, SET_FINE				, 255, 234,   0, 255).add(MORTAR, INGOTS, FOOD, MELTING, EXTRUDER, EXTRUDER_SIMPLE, FURNACE)																																																									.aspects(TC.FAMES, 1).setStatsEnergetic(320, 500),
	Chili					= dust			( 9781, "Chili"					, SET_FINE				, 200,   0,   0, 255).add(FOOD, MORTAR)																																																																							.aspects(TC.IGNIS, 1),
	Cocoa					= dust			( 9782, "Cocoa"					, SET_FINE				, 190,  95,   0, 255).add(FOOD, MORTAR)																																																																							.aspects(TC.SANO, 1).setOriginalMod(MD.MC),
	Chocolate				= mixdust		( 9783, "Chocolate"				, SET_FINE				, 100,  50,   0, 255).add(FOOD, MORTAR, FURNACE, INGOTS, MELTING, EXTRUDER, EXTRUDER_SIMPLE)																		.setMcfg( 0, Cocoa			, 1*U, Sugar			, 1*U)																									.aspects(TC.SANO, 1, TC.FAMES, 1).setStatsEnergetic(CS.C+40, 400),
	Coffee					= dust			( 9784, "Coffee"				, SET_FINE				, 150,  75,   0, 255).add(FOOD, MORTAR)																																																																							.aspects(TC.MOTUS, 1).setOriginalMod(MD.IC2).addIdenticalNames("CoffeeDust"),
	Cinnamon				= dust			( 9785, "Cinnamon"				, SET_FINE				, 122,  83,  53, 255).add(FOOD, MORTAR)																																																																							.aspects(TC.FAMES, 1).setOriginalMod(MD.HaC),
	Nutmeg					= dust			( 9786, "Nutmeg"				, SET_FINE				, 240, 220, 180, 255).add(FOOD, MORTAR)																																																																							.aspects(TC.FAMES, 1).setFurnaceBurnTime(TICKS_PER_SMELT/ 2).setOriginalMod(MD.HaC),
	Peanut					= dust			( 9795, "Peanut"				, SET_FINE				, 240, 210, 160, 255).add(FOOD, MORTAR)																																																																							.aspects(TC.FAMES, 1).setFurnaceBurnTime(TICKS_PER_SMELT/ 2).setOriginalMod(MD.GT),
	Hazelnut				= dust			( 9796, "Hazelnut"				, SET_FINE				, 240, 200, 140, 255).add(FOOD, MORTAR)																																																																							.aspects(TC.FAMES, 1).setFurnaceBurnTime(TICKS_PER_SMELT/ 2).setOriginalMod(MD.GT),
	Pistachio				= dust			( 9797, "Pistachio"				, SET_FINE				, 200, 250, 140, 255).add(FOOD, MORTAR)																																																																							.aspects(TC.FAMES, 1).setFurnaceBurnTime(TICKS_PER_SMELT/ 2).setOriginalMod(MD.GT),
	Vanilla					= dust			( 9787, "Vanilla"				, SET_FINE				, 110,  80,  40, 255).add(FOOD, MORTAR)																																																																							.aspects(TC.FAMES, 1).setOriginalMod(MD.HaC),
	PepperBlack				= dust			( 9788, "PepperBlack"			, SET_FINE				,  40,  40,  40, 255).add(FOOD, MORTAR)																																																																							.aspects(TC.FAMES, 1).setOriginalMod(MD.HaC).addIdenticalNames("Pepper").setLocal("Black Pepper"),
	Curry					= dust			( 9789, "Curry"					, SET_FINE				, 240, 190, 100, 255).add(FOOD, MORTAR)																																																																							.aspects(TC.FAMES, 1).setOriginalMod(MD.HaC),
	Milk					= create		( 9790, "Milk"					, SET_FINE				, 254, 254, 254, 255).add(G_CONTAINERS, DUSTS, FOOD, MORTAR)																																																																	.aspects(TC.SANO, 2).setStatsEnergetic(CS.C, CS.C+100).setOriginalMod(MD.MC),
	Honey					= create		( 9791, "Honey"					, SET_FINE				, 250, 200,   0, 255).add(G_CONTAINERS, DUSTS, FOOD, MORTAR)																																																																	.aspects(TC.SANO, 1).setStatsEnergetic(CS.C, CS.C+100).setOriginalMod(MD.FR),
	Honeydew				= create		( 9793, "Honeydew"				, SET_FINE				, 210, 100,   0, 255).add(G_CONTAINERS, DUSTS, FOOD, MORTAR)																																																																	.aspects(TC.SANO, 1).setStatsEnergetic(CS.C, CS.C+100).setOriginalMod(MD.FR),
	Tea						= dust			( 9792, "Tea"					, SET_FINE				, 100, 250, 100, 255).add(FOOD, MORTAR)																																																																							.aspects(TC.SANO, 1),
	Mint					= dust			( 9794, "Mint"					, SET_FINE				, 150, 250, 150, 255).add(FOOD, MORTAR)																																																																							.aspects(TC.HERBA, 1);
	
	public static final OreDictMaterial
	Sand					= create		( 8100,	"Sand"					, SET_SAND				, 250, 250, 200, 255).add(FURNACE, UNRECYCLABLE)																																																																				.setOriginalMod(MD.MC).setSmelting(Glass, U),
	SoulSand				= create		( 8101,	"Soulsand"				, SET_SAND				, 100, 100,  80, 255)																																																																											.setOriginalMod(MD.MC),
	SluiceSand				= oredust		( 8102, "Sluice Sand"			, SET_SAND				, 165, 165, 120, 255)																																																																											.aspects(TC.TERRA, 1).setOriginalMod(MD.GT),
	PlatinumGroupSludge		= oredust		( 8103, "Platinum Group Sludge"	, SET_SAND				,  50,  50,  80, 255)																																																																											.aspects(TC.LUCRUM, 2).setOriginalMod(MD.GT);
	
	public static final OreDictMaterial
	Diamond					= create		( 8300, "Diamond"				, SET_DIAMOND			, 200, 255, 255, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, UUM)																				.setMcfg( 1, C 				, 4*U)																															.aspects(TC.VITREUS, 3, TC.LUCRUM					, 4).steal(C).setQuality( 8.0F, 1280,  3).setSmelting(C, U*2).setBurning(Ash, U).setStatsEnergetic(4200, C.mBoilingPoint).setOriginalMod(MD.MC),
	
	Emerald					= create		( 8301, "Emerald"				, SET_EMERALD			,  80, 255,  80, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, Be				, 3*U, Al				, 2*U, Si				, 6*U, O				,18*U)													.aspects(TC.VITREUS, 3, TC.LUCRUM					, 5).setQuality( 7.0F,   256,  2).setSmelting(null, 0).setOriginalMod(MD.MC),
	Aquamarine				= create		( 8323, "Aquamarine"			, SET_EMERALD			, 200, 200, 255, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, Be				, 3*U, Al				, 2*U, Si				, 6*U, O				,18*U)													.aspects(TC.VITREUS, 3, TC.LUCRUM, 3, TC.AQUA		, 2).setQuality( 7.0F,   256,  2).setSmelting(null, 0).setOriginalMod(MD.GT),
	Morganite				= create		( 8324, "Morganite"				, SET_EMERALD			, 255, 200, 200, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, Be				, 3*U, Al				, 2*U, Si				, 6*U, O				,18*U)													.aspects(TC.VITREUS, 3, TC.LUCRUM, 3, TC.TEMPESTAS	, 2).setQuality( 7.0F,   256,  2).setSmelting(null, 0).setOriginalMod(MD.GT),
	
	Sapphire				= create		( 8304, "Sapphire"				, SET_GEM_VERTICAL		, 120, 120, 160, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, Al				, 2*U, O 				, 3*U)																									.aspects(TC.VITREUS, 3, TC.LUCRUM					, 5).setQuality( 7.0F,   512,  3).setSmelting(null, 0).setOriginalMod(MD.RP).addIdenticalNames("Saphire"),
	GreenSapphire			= create		( 8305, "Green Sapphire"		, SET_GEM_VERTICAL		, 100, 200, 130, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, Al				, 2*U, O 				, 3*U, Mg				, 1*U)																			.aspects(TC.VITREUS, 3, TC.LUCRUM, 4, TC.SANO		, 1).setQuality( 7.0F,   512,  3).setSmelting(null, 0).setOriginalMod(MD.RP),
	YellowSapphire			= create		( 8315, "Yellow Sapphire"		, SET_GEM_VERTICAL		, 220, 220,  50, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, Al				, 2*U, O 				, 3*U, OREMATS.Rutile	, 1*U)																			.aspects(TC.VITREUS, 3, TC.LUCRUM, 4, TC.TUTAMEN	, 1).setQuality( 7.0F,   512,  3).setSmelting(null, 0).setOriginalMod(MD.GT),
	OrangeSapphire			= create		( 8314, "Orange Sapphire"		, SET_GEM_VERTICAL		, 220, 150,  50, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, Al				, 2*U, O 				, 3*U, Cu				, 1*U)																			.aspects(TC.VITREUS, 3, TC.LUCRUM, 4, TC.PERMUTATIO	, 1).setQuality( 7.0F,   512,  3).setSmelting(null, 0).setOriginalMod(MD.GT),
	BlueSapphire			= create		( 8328, "Blue Sapphire"			, SET_GEM_VERTICAL		, 100, 100, 200, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, Al				, 2*U, O 				, 3*U, Fe				, 1*U)																			.aspects(TC.VITREUS, 3, TC.LUCRUM, 4, TC.METALLUM	, 1).setQuality( 7.0F,   512,  3).setSmelting(null, 0).setOriginalMod(MD.GT),
	Ruby					= create		( 8302, "Ruby"					, SET_RUBY				, 255, 100, 100, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, Al				, 2*U, O				, 3*U, Cr				, 1*U)																			.aspects(TC.VITREUS, 3, TC.LUCRUM, 4, TC.MACHINA	, 1).setQuality( 7.0F,   512,  3).setSmelting(null, 0).setOriginalMod(MD.RP),
	
	Spinel					= create		( 8326,	"Spinel"				, SET_RUBY				,   0, 100,   0, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, Mg				, 1*U, Al				, 2*U, O				, 4*U)																			.aspects(TC.VITREUS, 2, TC.LUCRUM					, 3).setQuality( 7.0F,   384,  2).setSmelting(null, 0),
	BalasRuby				= create		( 8303,	"Balas Ruby"			, SET_RUBY				, 255, 100, 100, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM).setLocal("Ruby").steal(Ruby)									.setMcfg( 0, Mg				, 1*U, Cr				, 2*U, O				, 4*U)																			.aspects(TC.VITREUS, 2, TC.LUCRUM, 2, TC.STRONTIO	, 2).setQuality( 7.0F,   384,  2).setSmelting(null, 0).setOriginalMod(MD.RP).addIdenticalNames("FoolsRuby"),
	
	Almandine				= create		( 9101, "Almandine"				, SET_RUBY				, 255,   0,   0, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, Al				, 2*U, Fe				, 3*U, Si				, 3*U, O				,12*U)													.aspects(TC.VITREUS, 3, TC.LUCRUM, 1, TC.METALLUM	, 1).setQuality( 7.0F,   128,  2).setSmelting(null, 0).setOriginalMod(MD.GT).addIdenticalNames("GarnetRed"),
	Andradite				= create		( 9102, "Andradite"				, SET_RUBY				, 150, 120,   0, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)						 										.setMcfg( 0, Ca				, 3*U, Fe				, 2*U, Si				, 3*U, O				,12*U)													.aspects(TC.VITREUS, 3, TC.LUCRUM, 1, TC.TUTAMEN	, 1).setQuality( 7.0F,   256,  2).setSmelting(null, 0).setOriginalMod(MD.GT).addIdenticalNames("GarnetYellow"),
	Grossular				= create		( 9119, "Grossular"				, SET_RUBY				, 200, 100,   0, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, Ca				, 3*U, Al				, 2*U, Si				, 3*U, O				,12*U)													.aspects(TC.VITREUS, 3, TC.LUCRUM, 1, TC.VOLATUS	, 1).setQuality( 7.0F,   128,  2).setSmelting(null, 0).setOriginalMod(MD.GT),
	Pyrope					= create		( 9127, "Pyrope"				, SET_RUBY				, 120,  50, 100, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)													  			.setMcfg( 0, Al				, 2*U, Mg				, 3*U, Si				, 3*U, O				,12*U)													.aspects(TC.VITREUS, 3, TC.LUCRUM, 1, TC.SANO		, 1).setQuality( 7.0F,   128,  2).setSmelting(null, 0).setOriginalMod(MD.GT),
	Spessartine				= create		( 9129, "Spessartine"			, SET_RUBY				, 255, 100, 100, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)													   			.setMcfg( 0, Al				, 2*U, Mn				, 3*U, Si				, 3*U, O				,12*U)													.aspects(TC.VITREUS, 3, TC.LUCRUM, 1, TC.METALLUM	, 1).setQuality( 7.0F,   128,  2).setSmelting(null, 0).setOriginalMod(MD.GT),
	Uvarovite				= create		( 9135, "Uvarovite"				, SET_RUBY				, 180, 255, 180, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)													 			.setMcfg( 0, Ca				, 3*U, Cr				, 2*U, Si				, 3*U, O				,12*U)													.aspects(TC.VITREUS, 3, TC.LUCRUM, 1, TC.MACHINA	, 1).setQuality( 8.0F,   128,  2).setSmelting(null, 0).setOriginalMod(MD.GT),
	
	Topaz					= create		( 8306, "Topaz"					, SET_GEM_HORIZONTAL	, 255, 128,   0, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, Al				, 2*U, Si				, 1*U, F 				, 2*U, H2O				, 3*U, O				, 5*U)							.aspects(TC.VITREUS, 4, TC.LUCRUM					, 6).setQuality( 7.0F,   256,  3).setSmelting(null, 0),
	BlueTopaz				= create		( 8307, "Blue Topaz"			, SET_GEM_HORIZONTAL	, 123, 150, 220, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, Al				, 2*U, Si				, 1*U, F 				, 2*U, H2O				, 3*U, O				, 5*U)							.aspects(TC.VITREUS, 4, TC.LUCRUM					, 6).setQuality( 7.0F,   256,  3).setSmelting(null, 0),
	Tanzanite				= create		( 8308, "Tanzanite"				, SET_GEM_HORIZONTAL	,  64,   0, 200, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, Ca				, 2*U, Al				, 3*U, Si				, 3*U, H				, 1*U, O				,13*U)							.aspects(TC.VITREUS, 3, TC.LUCRUM					, 5).setQuality( 7.0F,   256,  2).setSmelting(null, 0),
	Jasper					= create		( 8309,	"Jasper"				, SET_EMERALD			, 200,  80,  80, 150).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, SiO2			, 2*U, Fe				, 1*U)																									.aspects(TC.VITREUS, 2, TC.LUCRUM, 2, TC.METALLUM	, 1).setQuality( 7.0F,   256,  2).setSmelting(null, 0).setOriginalMod(MD.TFC),
	Olivine					= create		( 8311, "Olivine"				, SET_RUBY				, 150, 255, 150, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, Mg				, 2*U, Fe				, 1*U, SiO2				, 2*U)																			.aspects(TC.VITREUS, 2, TC.LUCRUM					, 4).setQuality( 7.0F,   256,  2).setSmelting(null, 0).addIdenticalNames("Peridot").setOriginalMod(MD.GT),
	Opal					= create		( 8312, "Opal"					, SET_OPAL				,   0,   0, 255, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, SiO2			, 1*U)																															.aspects(TC.VITREUS, 3, TC.LUCRUM					, 5).setQuality( 7.0F,   256,  2).setSmelting(null, 0),
	Amethyst				= create		( 8313, "Amethyst"				, SET_RUBY				, 200,  50, 200, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, SiO2			, 4*U, Fe				, 1*U)																									.aspects(TC.VITREUS, 4, TC.LUCRUM					, 6).setQuality( 7.0F,   256,  3).setSmelting(null, 0),
	Amber					= create		( 8310,	"Amber"					, SET_RUBY				, 255, 180,   0, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MORTAR)																																																														.aspects(TC.VITREUS, 1, TC.VINCULUM					, 2).setQuality( 4.0F,   128,  2).setSmelting(null, 0).setOriginalMod(MD.TC),
	Jade					= create		( 8321,	"Jade"					, SET_SHINY				,   0, 100,  30, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE)																																																																.aspects(TC.VITREUS, 3, TC.LUCRUM, 6).setSmelting(null, 0),
	Craponite				= create		( 8322, "Craponite"				, SET_FLINT				, 255, 170, 185, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER)																																																									.aspects(TC.VITREUS, 3, TC.LUCRUM, 3, TC.STRONTIO, 2).setQuality( 7.0F,	256,  2).setSmelting(null, 0).setOriginalMod(MD.GT),
	Dioptase				= create		( 8325, "Dioptase"				, SET_EMERALD			,   0, 180, 180, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																.setMcfg( 0, Cu				, 1*U, Si				, 1*U, O				, 3*U, H2O				, 3*U)													.aspects(TC.VITREUS, 3, TC.LUCRUM, 3, TC.PERMUTATIO, 2).setQuality( 7.0F,  256,  2).setSmelting(null, 0).setOriginalMod(MD.GT),
	
	Vinteum					= create		( 8316, "Vinteum"				, SET_EMERALD			,  80,  80, 255, 255).add(G_GEM_ORES, MAGICAL, CRYSTAL, BRITTLE, DECOMPOSABLE, UNBURNABLE)																			.setMcfg( 0, Ma				, 1*U)																															.aspects(TC.VITREUS, 2, TC.PRAECANTIO, 1).setQuality(10.0F,	128,  3).setOriginalMod(MD.ARS).visDefault(),
	VinteumPurified			= create		( 8327, "VinteumPurified"		, SET_EMERALD			, 230, 100, 255, 255).add(G_GEM_ORES, MAGICAL, CRYSTAL, BRITTLE, DECOMPOSABLE, UNBURNABLE)																			.setMcfg( 0, Ma				, 1*U)																															.aspects(TC.VITREUS, 2, TC.PRAECANTIO, 2).setQuality(12.0F,	256,  3).setOriginalMod(MD.ARS).visDefault().setLocal("Purified Vinteum"),
	ArcaneAsh				= dust			( 8367, "Arcane Ashes"			, SET_FINE				, 150,  50, 180, 255).add(MAGICAL)																																																																								.aspects(TC.PRAECANTIO, 1).setOriginalMod(MD.ARS).addIdenticalNames("ArcaneAsh").visDefault(),
	ArcaneCompound			= dust			( 8366, "Arcane Compound"		, SET_ROUGH				, 180, 140,  50, 255).add(MAGICAL, FURNACE)																																																																						.aspects(TC.PRAECANTIO, 1).setSmelting(ArcaneAsh, U*2).setOriginalMod(MD.ARS).visDefault(),
	
	EnderAmethyst			= create		( 8329, "AmethystEnder"			, SET_FLINT				, 210,  50, 210, 127).add(G_GEM_ORES, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, UUM)																			.setMcfg( 5, SiO2			, 4*U, Fe				, 1*U, Ma				, 1*U)																			.aspects(TC.VITREUS, 4, TC.LUCRUM, 4, TC.ALIENIS, 2).setQuality(10.0F, 2560, 3).setSmelting(null, 0).setLocal("Ender Amethyst").setOriginalMod(MD.BoP).visDefault(),
	EnderPearl				= create		( 8318, "EnderPearl"			, SET_SHINY				, 108, 220, 200, 255).add(G_PEARL_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, MAGICAL, PEARL, MELTING)												.setMcfg(10, Be				, 1*U, K				, 4*U, N				, 5*U, Ma				, 6*U)													.aspects(TC.ALIENIS, 4, TC.ITER, 4, TC.PRAECANTIO, 2).setQuality( 1.0F,	 16,  1).setLocal("Enderpearl").addIdenticalNames("Ender").setStatsEnergetic(2723, 3785).setOriginalMod(MD.MC),
	EnderEye				= create		( 8319, "EnderEye"				, SET_SHINY				, 160, 250, 230, 255).add(G_PEARL_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, CENTRIFUGE, MAGICAL, PEARL, MELTING)													.setMcfg( 1, EnderPearl		, 1*U, Blaze			, 1*U)																									.aspects(TC.SENSUS, 4, TC.ALIENIS, 4, TC.ITER, 4, TC.PRAECANTIO, 3, TC.IGNIS, 2).setQuality( 1.0F,	 16,  1).setLocal("Endereye").setStatsEnergetic(3447, 4978).setOriginalMod(MD.MC),
	NetherStar				= create		( 8320,	"Nether Star"			, SET_NETHERSTAR		, 255, 255, 255, 255).add(G_GEM, CRYSTAL, BRITTLE, UNBURNABLE, MAGICAL, GLOWING, MELTING)																																																										.setQuality( 1.0F,   5120,  4).setOriginalMod(MD.MC),
	
	Dilithium				= create		( 8317,	"Dilithium"				, SET_DIAMOND			, 153, 255, 255, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, CRYSTALLISABLE, QUARTZ)																																																										.setSmelting(null, 0).setOriginalMod(MD.MO);
	
	public static final OreDictMaterial
	Lazurite				= create		( 8330, "Lazurite"				, SET_LAPIS				, 100, 120, 255, 255).add(G_GEM_ORES, DENSEPLATES, ELECTROLYSER, DECOMPOSABLE, BRITTLE, CRYSTAL, CRYSTALLISABLE, MORTAR, UUM)									 	.setMcfg( 0, Al				, 6*U, Si				, 6*U, Ca				, 8*U, Na				, 8*U)													,
	Sodalite				= create		( 8331, "Sodalite"				, SET_LAPIS				,  20,  20, 255, 255).add(G_GEM_ORES, DENSEPLATES, ELECTROLYSER, DECOMPOSABLE, BRITTLE, CRYSTAL, CRYSTALLISABLE, MORTAR, UUM)										.setMcfg( 0, Al				, 3*U, Si				, 3*U, Na				, 4*U, Cl				, 1*U)													,
	Lapis					= create		( 8332, "Lapis"					, SET_LAPIS				,  70,  70, 220, 255).add(G_GEM_ORES, DENSEPLATES, CENTRIFUGE, DECOMPOSABLE, BRITTLE, CRYSTAL, CRYSTALLISABLE, MORTAR, UUM)											.setMcfg( 0, Lazurite		,12*U, Sodalite			, 2*U, Pyrite			, 1*U, CaCO3			, 1*U)													.aspects(TC.SENSUS, 1).setOriginalMod(MD.MC),
	Redstone				= oredustcent	( 8333, "Redstone"				, SET_ROUGH				, 200,   0,   0, 255).add(PLATES, BRITTLE, MELTING, UNBURNABLE, PULVERIZING_CINNABAR, MORTAR, UUM)																	.setMcfg( 0, Pyrite			, 5*U, Hg				, 3*U, SiO2				, 1*U, Ruby				, 1*U)													.aspects(TC.MACHINA, 1, TC.POTENTIA, 2).setStatsEnergetic(500, 1500).setOriginalMod(MD.MC),
	Graphene				= dustcent		( 9175,	"Graphene"				, SET_DULL				, 128, 128, 128, 255).add(BLACKLISTED_SMELTER, MORTAR, UUM)																											.setMcfg( 0, C				, 1*U)																															.setQuality( 6.0F,	 32,  1).setSmelting(C, U/2).setBurning(Ash, U/4).setStatsEnergetic(4300, 4400),
	Coal					= create		( 8334, "Coal"					, SET_LIGNITE			,  70,  70,  70, 255).add(G_GEM_ORES, ELECTROLYSER, DECOMPOSABLE, BRITTLE, FLAMMABLE, FURNACE, MORTAR, UUM).setBurning(DarkAsh, U/4)								.setMcfg( 1, C 				, 2*U)																															.setFurnaceBurnTime(TICKS_PER_SMELT* 8).setDensity(0.929).aspects(TC.POTENTIA, 2, TC.IGNIS, 2).setOriginalMod(MD.MC),
	CoalCoke				= create		( 8349, "Coal Coke"				, SET_LIGNITE			, 140, 140, 170, 255).add(G_GEM_ORES, ELECTROLYSER, DECOMPOSABLE, BRITTLE, FLAMMABLE, FURNACE, MORTAR, UUM).setBurning(DarkAsh, U/4)								.setMcfg( 1, C 				, 2*U)																															.setFurnaceBurnTime(TICKS_PER_SMELT*16).setDensity(0.929).aspects(TC.POTENTIA, 3, TC.IGNIS, 1).setOriginalMod(MD.RC).steal(Coal).addIdenticalNames("Coke"),
	HydratedCoal			= mixdust		( 8335, "Hydrated Coal"			, SET_LIGNITE			,  70,  70, 100, 255).add(BRITTLE, FURNACE, MORTAR, UUM)																											.setMcfg( 8, Coal			, 8*U, H2O				, 1*U)																									.setOriginalMod(MD.IC2),
	Charcoal				= create		( 8336, "Charcoal"				, SET_LIGNITE			, 100,  70,  70, 255).add(G_GEM, ELECTROLYSER, DECOMPOSABLE, BRITTLE, FLAMMABLE, FURNACE, MORTAR, UUM).setBurning(Ash, U/4)											.setMcfg( 0, C 				, 1*U)																															.setFurnaceBurnTime(TICKS_PER_SMELT* 8).setDensity(0.929).aspects(TC.POTENTIA, 2, TC.IGNIS, 2).setOriginalMod(MD.MC),
	Lignite					= create		( 8337, "Lignite"				, SET_LIGNITE			, 100,  70,  70, 255).add(G_GEM_ORES, ELECTROLYSER, DECOMPOSABLE, BRITTLE, FLAMMABLE, FURNACE, MORTAR, UUM).setBurning(DarkAsh, U/9)								.setMcfg( 7, C				, 2*U, H2O				, 4*U, DarkAsh			, 1*U)																			.setFurnaceBurnTime(TICKS_PER_SMELT* 2).setDensity(0.865).setLocal("Lignite Coal"),
	LigniteCoke				= create		( 8365, "Lignite Coke"			, SET_LIGNITE			, 140, 100, 100, 255).add(G_GEM_ORES, ELECTROLYSER, DECOMPOSABLE, BRITTLE, FLAMMABLE, FURNACE, MORTAR, UUM).setBurning(DarkAsh, U/9)								.setMcfg( 7, C				, 2*U, DarkAsh			, 1*U)																									.setFurnaceBurnTime(TICKS_PER_SMELT* 4).setDensity(0.865).setLocal("Lignite Coke"),
	Peat					= dust			( 8360, "Peat"					, SET_LIGNITE			,  64,  40,  14, 255).add(INGOTS, MULTIINGOTS, BRITTLE, FLAMMABLE, MORTAR).setBurning(Ash, U)																																																									.setFurnaceBurnTime(TICKS_PER_SMELT*10).aspects(TC.POTENTIA, 2, TC.IGNIS, 2).setOriginalMod(MD.FR),
	PeatBituminous			= dust			( 8361, "PeatBituminous"		, SET_LIGNITE			,  80,  40,  10, 255).add(INGOTS, MULTIINGOTS, BRITTLE, FLAMMABLE, MORTAR)																																																														.setFurnaceBurnTime(TICKS_PER_SMELT*12).aspects(TC.POTENTIA, 3, TC.IGNIS, 3, TC.PERDITIO, 3).setOriginalMod(MD.FR).setLocal("Bituminous Peat"),
	Anthracite				= create		( 8362, "Anthracite"			, SET_LIGNITE			,  90,  90,  90, 255).add(G_GEM_ORES, ELECTROLYSER, DECOMPOSABLE, BRITTLE, FLAMMABLE, UUM).setBurning(DarkAsh, U/2)													.setMcfg( 1, C 				, 4*U)																															.setFurnaceBurnTime(TICKS_PER_SMELT*24).aspects(TC.POTENTIA, 3, TC.IGNIS, 3).setOriginalMod(MD.RoC),
	Prismane				= create		( 8363, "Prismane"				, SET_LIGNITE			, 115, 110, 110, 255).add(G_GEM_ORES, ELECTROLYSER, DECOMPOSABLE, BRITTLE, FLAMMABLE, UUM).setBurning(DarkAsh, U)													.setMcfg( 1, C 				, 8*U)																															.setFurnaceBurnTime(TICKS_PER_SMELT*48).aspects(TC.POTENTIA, 4, TC.IGNIS, 4).setOriginalMod(MD.RoC),
	Lonsdaleite				= create		( 8364, "Lonsdaleite"			, SET_DIAMOND			, 140, 130, 130, 255).add(G_GEM_ORES, ELECTROLYSER, DECOMPOSABLE, BRITTLE, FLAMMABLE, UUM).setBurning(DarkAsh, U*2)													.setMcfg( 1, C 				,16*U)																															.setFurnaceBurnTime(TICKS_PER_SMELT*96).aspects(TC.POTENTIA, 6, TC.IGNIS, 6).setOriginalMod(MD.RoC),
	Monazite				= create		( 8338, "Monazite"				, SET_DIAMOND			,  50,  70,  50, 255).add(G_GEM_ORES, ELECTROLYSER, DECOMPOSABLE, CRYSTAL, BRITTLE, CRYSTALLISABLE, BLACKLISTED_SMELTER)											.setMcfg( 0, RareEarth		, 1*U, Phosphate		, 1*U)																									, // Wikipedia: (Ce, La, Nd, Th, Sm, Gd)PO4 Monazite also smelt-extract to Helium, it is brown like the rare earth Item Monazite sand deposits are inevitably of the monazite-(Ce) composition. Typically, the lanthanides in such monazites contain about 4548% cerium, about 24% lanthanum, about 17% neodymium, about 5% praseodymium, and minor quantities of samarium, gadolinium, and yttrium. Europium concentrations tend to be low, about 0.05% Thorium content of monazite is variable and sometimes can be up to 2030%
	Teslatite				= oredustcent	( 8339,	"Teslatite"				, SET_SHINY				,  60, 180, 200, 255).add(PLATES, MORTAR, BRITTLE, UNBURNABLE, MELTING, UUM)																						.setMcfg( 0, Sodalite		, 5*U, Cu				, 3*U, SiO2				, 1*U, Ar				, 1*U)													.aspects(TC.ELECTRUM, 2).setStatsEnergetic(1500, 3000).visDefault(),
	Nikolite				= oredustcent	( 8340,	"Nikolite"				, SET_SHINY				,  60, 180, 200, 255).add(PLATES, MORTAR, BRITTLE, UNBURNABLE, MELTING, UUM)																						.setMcfg( 0, Sodalite		, 5*U, Cu				, 3*U, SiO2				, 1*U, Ar				, 1*U)													.aspects(TC.ELECTRUM, 2).setStatsEnergetic(1500, 3000).visDefault().setOriginalMod(MD.RP),
	Electrotine				= oredustcent	( 8359,	"Electrotine"			, SET_SHINY				,  60, 180, 200, 255).add(PLATES, MORTAR, BRITTLE, UNBURNABLE, MELTING, UUM)																						.setMcfg( 0, Sodalite		, 5*U, Cu				, 3*U, SiO2				, 1*U, Ar				, 1*U)													.aspects(TC.ELECTRUM, 2).setStatsEnergetic(1500, 3000).visDefault().setOriginalMod(MD.PR),
	Glowstone				= oredustcent	( 8341,	"Glowstone"				, SET_SHINY				, 255, 255,   0, 255).add(PLATES, MORTAR, BRITTLE, UNBURNABLE, MELTING, UUM, CRYSTAL, GLOWING, LIGHTING)															.setMcfg( 0, Phosphorite	, 5*U, Au				, 3*U, SiO2				, 1*U, He				, 1*U)													.aspects(TC.LUX, 2, TC.SENSUS, 1).setOriginalMod(MD.MC),
	GlowstoneCeres			= oredustcent	( 8368,	"GlowstoneCeres"		, SET_SHINY				,  70,  90,  70, 255).add(PLATES, MORTAR, BRITTLE, UNBURNABLE, MELTING, UUM, CRYSTAL, GLOWING, LIGHTING)															.setMcfg( 0, Phosphorite	, 5*U, Au				, 3*U, SiO2				, 1*U, He				, 1*U)													.aspects(TC.LUX, 2, TC.SENSUS, 1, TC.HERBA		, 1).setOriginalMod(MD.GC_GALAXYSPACE).visDefault().setLocal("Ceres Glowstone"),
	GlowstoneIo				= oredustcent	( 8369,	"GlowstoneIo"			, SET_SHINY				, 180,  20,   0, 255).add(PLATES, MORTAR, BRITTLE, UNBURNABLE, MELTING, UUM, CRYSTAL, GLOWING, LIGHTING)															.setMcfg( 0, Phosphorite	, 5*U, Au				, 3*U, SiO2				, 1*U, He				, 1*U)													.aspects(TC.LUX, 2, TC.SENSUS, 1, TC.IGNIS		, 1).setOriginalMod(MD.GC_GALAXYSPACE).visDefault().setLocal("Io Glowstone"),
	GlowstoneEnceladus		= oredustcent	( 8370,	"GlowstoneEnceladus"	, SET_SHINY				,   0, 250, 250, 255).add(PLATES, MORTAR, BRITTLE, UNBURNABLE, MELTING, UUM, CRYSTAL, GLOWING, LIGHTING)															.setMcfg( 0, Phosphorite	, 5*U, Au				, 3*U, SiO2				, 1*U, He				, 1*U)													.aspects(TC.LUX, 2, TC.SENSUS, 1, TC.AQUA		, 1).setOriginalMod(MD.GC_GALAXYSPACE).visDefault().setLocal("Enceladus Glowstone"),
	GlowstoneProteus		= oredustcent	( 8371,	"GlowstoneProteus"		, SET_SHINY				,  62,  62,  62, 255).add(PLATES, MORTAR, BRITTLE, UNBURNABLE, MELTING, UUM, CRYSTAL, GLOWING, LIGHTING)															.setMcfg( 0, Phosphorite	, 5*U, Au				, 3*U, SiO2				, 1*U, He				, 1*U)													.aspects(TC.LUX, 2, TC.SENSUS, 1, TC.TENEBRAE	, 1).setOriginalMod(MD.GC_GALAXYSPACE).visDefault().setLocal("Proteus Glowstone"),
	GlowstonePluto			= oredustcent	( 8372,	"GlowstonePluto"		, SET_SHINY				, 123, 150, 220, 255).add(PLATES, MORTAR, BRITTLE, UNBURNABLE, MELTING, UUM, CRYSTAL, GLOWING, LIGHTING)															.setMcfg( 0, Phosphorite	, 5*U, Au				, 3*U, SiO2				, 1*U, He				, 1*U)													.aspects(TC.LUX, 2, TC.SENSUS, 1, TC.GELUM		, 1).setOriginalMod(MD.GC_GALAXYSPACE).visDefault().setLocal("Pluto Glowstone"),
	Ectoplasm				= dust			( 8373,	"Ectoplasm"				, SET_FOOD				, 220, 255, 255, 200).add(MAGICAL, LIQUID, GLOWING)																																																																				.aspects(TC.SPIRITUS, 4).setOriginalMod(MD.RT).setStatsEnergetic(400, 3000),
	Firestone				= create		( 8342,	"Firestone"				, SET_QUARTZ			, 200,  20,   0, 255).add(G_QUARTZ_ORES, CRYSTAL, BRITTLE, CRYSTALLISABLE, MAGICAL, QUARTZ, UNBURNABLE, BURNING).setQuality( 6.0F, 1280, 3)																																														.setOriginalMod(MD.RC).visDefault(),
	Force					= create		( 8343,	"Force"					, SET_DIAMOND			, 255, 255,   0, 255).add(G_GEM_ORES, G_INGOT_MACHINE_ORES).add(CRYSTAL, MAGICAL, UNBURNABLE, GLOWING).setQuality(10.0F, 128, 3)																																																.aspects(TC.POTENTIA, 4).visDefault(),
	Forcicium				= create		( 8344,	"Forcicium"				, SET_DIAMOND			,  50,  50,  70, 255).add(G_QUARTZ_ORES, MAGICAL, CRYSTAL, BRITTLE, CRYSTALLISABLE)																																																												.aspects(TC.POTENTIA, 2).visDefault(),
	Forcillium				= create		( 8345,	"Forcillium"			, SET_DIAMOND			,  50,  50,  70, 255).add(G_QUARTZ_ORES, MAGICAL, CRYSTAL, BRITTLE, CRYSTALLISABLE)																																																												.aspects(TC.POTENTIA, 2).visDefault(),
	NetherQuartz			= create		( 8346,	"Nether Quartz"			, SET_QUARTZ			, 230, 210, 210, 255).add(G_QUARTZ_ORES, MORTAR, BRITTLE, QUARTZ, BLACKLISTED_SMELTER, UUM, CRYSTALLISABLE)															.setMcfg( 0, SiO2			, 1*U)																															.aspects(TC.POTENTIA, 1, TC.VITREUS, 1).setQuality( 2.5F, 32, 1).setSmelting(SiO2, U).setOriginalMod(MD.MC),
	CertusQuartz			= create		( 8347,	"Certus Quartz"			, SET_QUARTZ			, 210, 210, 230, 255).add(G_QUARTZ_ORES, MORTAR, BRITTLE, QUARTZ, BLACKLISTED_SMELTER, UUM, CRYSTALLISABLE)															.setMcfg( 0, SiO2			, 1*U)																															.aspects(TC.POTENTIA, 1, TC.VITREUS, 1).setQuality( 5.0F, 32, 1).setSmelting(SiO2, U).setOriginalMod(MD.AE),
	ChargedCertusQuartz		= create		( 8348,	"Charged Certus Quartz"	, SET_QUARTZ			, 210, 210, 230, 255).add(G_QUARTZ_ORES, MORTAR, BRITTLE, QUARTZ, BLACKLISTED_SMELTER, UUM, GLOWING)																.setMcfg( 0, CertusQuartz	, 1*U)																															.aspects(TC.POTENTIA, 2, TC.VITREUS, 1).steal(CertusQuartz).setSmelting(SiO2, U).setPulver(CertusQuartz, U).setOriginalMod(MD.AE),
	InfusedDull				= create		(-8350,	"Infused Dull"			, SET_SHARDS			, 100, 100, 100, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MAGICAL, UNBURNABLE)																																																										.aspects(TC.PRAECANTIO, 1, TC.VACUOS	, 2).setQuality(32.0F,   64,  3).setOriginalMod(MD.TC).visDefault(),
	InfusedVis				= create		(-8351,	"Infused Vis"			, SET_SHARDS			, 255,   0, 255, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MAGICAL, UNBURNABLE, GLOWING)																																																								.aspects(TC.PRAECANTIO, 1, TC.AURAM		, 2).setQuality( 8.0F,   64,  3).setOriginalMod(MD.TC).visDefault(),
	InfusedAir				= create		( 8352,	"Infused Air"			, SET_SHARDS			, 255, 255,   0, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MAGICAL, UNBURNABLE, GLOWING)																																																								.aspects(TC.PRAECANTIO, 1, TC.AER		, 2).setQuality( 8.0F,   64,  3).setOriginalMod(MD.TC).visDefault(),
	InfusedFire				= create		( 8353,	"Infused Fire"			, SET_SHARDS			, 255,   0,   0, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MAGICAL, UNBURNABLE, GLOWING)																																																								.aspects(TC.PRAECANTIO, 1, TC.IGNIS		, 2).setQuality( 8.0F,   64,  3).setOriginalMod(MD.TC).visDefault(),
	InfusedEarth			= create		( 8354,	"Infused Earth"			, SET_SHARDS			,   0, 255,   0, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MAGICAL, UNBURNABLE, GLOWING)																																																								.aspects(TC.PRAECANTIO, 1, TC.TERRA		, 2).setQuality( 8.0F,  256,  3).setOriginalMod(MD.TC).visDefault(),
	InfusedWater			= create		( 8355,	"Infused Water"			, SET_SHARDS			,   0,   0, 255, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MAGICAL, UNBURNABLE, GLOWING)																																																								.aspects(TC.PRAECANTIO, 1, TC.AQUA		, 2).setQuality( 8.0F,   64,  3).setOriginalMod(MD.TC).visDefault(),
	InfusedEntropy			= create		( 8356,	"Infused Entropy"		, SET_SHARDS			,  62,  62,  62, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MAGICAL, UNBURNABLE, GLOWING)																																																								.aspects(TC.PRAECANTIO, 1, TC.PERDITIO	, 2).setQuality(32.0F,   64,  4).setOriginalMod(MD.TC).visDefault(),
	InfusedOrder			= create		( 8357,	"Infused Order"			, SET_SHARDS			, 252, 252, 252, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MAGICAL, UNBURNABLE, GLOWING)																																																								.aspects(TC.PRAECANTIO, 1, TC.ORDO		, 2).setQuality( 8.0F,   64,  3).setOriginalMod(MD.TC).visDefault(),
	InfusedBalance			= create		(-8358,	"Infused Balance"		, SET_SHARDS			, 252, 252, 252, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MAGICAL, UNBURNABLE, GLOWING)																																																								.aspects(TC.PRAECANTIO, 1, TC.ORDO, 1, TC.PERDITIO, 1, TC.IGNIS, 1, TC.AER, 1, TC.AQUA, 1, TC.TERRA, 1).setQuality(32.0F,  256,  4).setOriginalMod(MD.TC).visDefault();
	
	public static final OreDictMaterial
	Petrotheum				= mixdust		( 8245, "Petrotheum"			, SET_DULL				,  86,  76,  82, 255).add(CONTAINERS, MELTING, MORTAR)																												.setMcfg( 2, Clay			, 1*U, Obsidian			, 1*U, Redstone			, 1*U, Basalz			, 1*U)													.aspects(TC.PRAECANTIO, 2, TC.POTENTIA, 1).setStatsEnergetic(400, 2000).setOriginalMod(MD.TE),
	Aerotheum				= mixdust		( 8246, "Aerotheum"				, SET_SHINY				, 200, 176,  83, 255).add(CONTAINERS, MELTING, MORTAR)																												.setMcfg( 2, Sand			, 1*U, KNO3				, 1*U, Redstone			, 1*U, Blitz			, 1*U)													.aspects(TC.PRAECANTIO, 2, TC.AER, 1).setStatsEnergetic(299, 300).setOriginalMod(MD.TE),
	Pyrotheum				= mixdust		( 8212, "Pyrotheum"				, SET_FIERY				, 255, 200, 120, 255).add(CONTAINERS, MELTING, MORTAR)																												.setMcfg( 2, Coal			, 1*U, S				, 1*U, Redstone			, 1*U, Blaze			, 1*U)													.aspects(TC.PRAECANTIO, 2, TC.IGNIS, 1).setStatsEnergetic(3800, 6400).setOriginalMod(MD.TE),
	Cryotheum				= mixdust		( 8213, "Cryotheum"				, SET_SHINY				, 200, 220, 255, 255).add(CONTAINERS, MELTING, MORTAR)																												.setMcfg( 8, Snow			, 1*U, KNO3				, 4*U, Redstone			, 4*U, Blizz			, 4*U)													.aspects(TC.PRAECANTIO, 2, TC.GELUM, 1).setStatsEnergetic(40, 1000).setOriginalMod(MD.TE);
	
	public static final OreDictMaterial
	Stone					= create		( 8500, "Stone"					, SET_STONE				, 205, 205, 205, 255).add(G_STONE, STONE, BRITTLE, NO_ADVANCED_TOOLS, MORTAR, FURNACE, MELTING, UNRECYCLABLE, EXTRUDER, EXTRUDER_SIMPLE, MOLTEN)																																												.aspects(TC.TERRA, 1).setQuality(4.0F, 16, 1).setMeltingPoint(1100).setRGBaLiquid(192,  96,  64, 255).setOriginalMod(MD.MC),
	Concrete				= create		( 8501, "Concrete"				, SET_STONE				, 100, 100, 100, 255).add(G_STONE, STONE, BRITTLE, NO_ADVANCED_TOOLS, MORTAR, FURNACE, MELTING)																		.setMcfg( 0, Stone			, 1*U)																															.aspects(TC.TERRA, 1).setQuality(3.0F, 32, 0).setMeltingPoint(500).setOriginalMod(MD.GT),
	Netherrack				= create		( 8502,	"Netherrack"			, SET_STONE				, 200,   0,   0, 255).add(G_STONE, STONE, BRITTLE, NO_ADVANCED_TOOLS, MORTAR, FURNACE, UNBURNABLE, FLAMMABLE, BLACKLISTED_SMELTER)																																																.aspects(TC.IGNIS, 1).setStatsEnergetic(1500, 3000).setOriginalMod(MD.MC),
	NetherBrick				= create		( 8503,	"Nether Brick"			, SET_STONE				, 100,   0,   0, 255).add(G_BRICK, STONE, BRITTLE, NO_ADVANCED_TOOLS, MORTAR, FURNACE, UNBURNABLE)																																																								.aspects(TC.IGNIS, 1).setQuality(3.0F, 24, 1).setPulver(Netherrack, U).addIdenticalNames("BrickNether").setStatsEnergetic(1800, 3000).setOriginalMod(MD.MC),
	Endstone				= create		( 8504,	"Endstone"				, SET_STONE				, 217, 222, 158, 255).add(G_STONE, STONE, BRITTLE, NO_ADVANCED_TOOLS, MORTAR)																																																													.aspects(TC.TENEBRAE, 1).setMeltingPoint(1200).setOriginalMod(MD.MC),
	Gabbro					= create		( 9176,	"Gabbro"				, SET_STONE				,  65,  60,  60, 255).add(G_STONE, STONE, BRITTLE, NO_ADVANCED_TOOLS, MORTAR, CENTRIFUGE, DECOMPOSABLE)																.setMcfg( 0, Olivine		, 1*U, CaCO3			, 3*U, Flint			, 8*U, DarkAsh			, 4*U)													.aspects(TC.TENEBRAE, 1).setQuality(5.0F, 32, 1).setMeltingPoint(1673).setOriginalMod(MD.UB),
	Basalt					= create		( 8505, "Basalt"				, SET_STONE				,  30,  20,  20, 255).add(G_STONE, STONE, BRITTLE, NO_ADVANCED_TOOLS, MORTAR, CENTRIFUGE, DECOMPOSABLE)																.setMcfg( 0, Olivine		, 1*U, CaCO3			, 3*U, Flint			, 8*U, DarkAsh			, 4*U)													.aspects(TC.TENEBRAE, 1).setQuality(5.0F, 32, 1).setMeltingPoint(1673).setOriginalMod(MD.RP),
	Marble					= create		( 8506, "Marble"				, SET_STONE				, 200, 200, 200, 255).add(G_STONE, STONE, BRITTLE, NO_ADVANCED_TOOLS, MORTAR, CENTRIFUGE, DECOMPOSABLE, FURNACE)													.setMcfg( 0, Mg				, 1*U, CaCO3			, 7*U)																									.aspects(TC.PERFODIO, 1).setQuality(3.0F, 16, 1).setSmelting(CaCO3, 2*U/3).setOriginalMod(MD.RP),
	Limestone				= create		( 9189,	"Limestone"				, SET_STONE				, 230, 200, 130, 255).add(G_STONE, STONE, BRITTLE, NO_ADVANCED_TOOLS, MORTAR, CENTRIFUGE, DECOMPOSABLE, FURNACE)													.setMcfg( 0, CaCO3			, 1*U)																															.aspects(TC.TERRA, 1).setQuality(3.0F, 16, 1).setSmelting(CaCO3, U/2),
	GraniteRed				= create		( 8507, "GraniteRed"			, SET_STONE				, 160,  60,  70, 255).add(G_STONE, STONE, BRITTLE, NO_ADVANCED_TOOLS, MORTAR, ELECTROLYSER, DECOMPOSABLE)															.setMcfg( 0, Al				, 2*U, PotassiumFeldspar, 1*U, O				, 3*U)																			.aspects(TC.TUTAMEN, 1).setQuality(4.0F, 64, 3).setLocal("Red Granite").setMeltingPoint(1500),
	GraniteBlack			= create		( 8508, "GraniteBlack"			, SET_STONE				,  20,  20,  20, 255).add(G_STONE, STONE, BRITTLE, NO_ADVANCED_TOOLS, MORTAR, CENTRIFUGE, DECOMPOSABLE)																.setMcfg( 0, SiO2			, 4*U, Biotite			, 1*U)																									.aspects(TC.TUTAMEN, 1).setQuality(4.0F, 64, 3).setLocal("Black Granite").setMeltingPoint(1500).addIdenticalNames("Granite"),
	Redrock					= create		( 8509, "Redrock"				, SET_STONE				, 255,  80,  50, 255).add(G_STONE, STONE, BRITTLE, NO_ADVANCED_TOOLS, MORTAR, CENTRIFUGE, DECOMPOSABLE)																.setMcfg( 0, CaCO3			, 2*U, Flint			, 1*U, Clay				, 1*U)																			.aspects(TC.TERRA, 1).setQuality(4.0F, 16, 1).addIdenticalNames("RedRock"),
	Gravel					= dust			( 8510,	"Gravel"				, SET_ROUGH				, 205, 205, 205, 255).add(STONE, BRITTLE, NO_ADVANCED_TOOLS, MORTAR)																																																															.aspects(TC.PERDITIO, 1).setOriginalMod(MD.MC),
	Diorite					= create		( 8511,	"Diorite"				, SET_STONE				, 240, 240, 240, 255).add(G_STONE, STONE, BRITTLE, NO_ADVANCED_TOOLS, MORTAR)																																																													.aspects(TC.TERRA, 1).setQuality(4.0F, 16, 1).setOriginalMod(MD.MC),
	
	Bedrock					= create		( 8599, "Bedrock"				, SET_STONE				,  64,  64,  64, 255).add(G_STONE, STONE, BRITTLE, NO_ADVANCED_TOOLS, MELTING)																																																													.aspects(TC.TERRA, 5).setQuality(8.0F, 2048, 5).setMeltingPoint(4000).setRGBaLiquid(128,  96,  64, 255).setOriginalMod(MD.MC).visDefault();
	
	static {Netherrack.setSmelting(NetherBrick, U);}
	
	public static final OreDictMaterial
	WroughtIron				= metalmachnd	( 8643, "Wrought Iron"			, SET_METALLIC			, 200, 180, 180, 255).add(RAILS, MORTAR, MAGNETIC_PASSIVE, UUM, MOLTEN)																								.setMcfg( 0, Fe				, 1*U)																															.aspects(TC.METALLUM, 2, TC.INSTRUMENTUM, 1).setPulver(Fe, U).steal(Fe).setQuality( 6.0F,  384,  2).addIdenticalNames("WrougtIron").setStatsEnergetic(Fe.mMeltingPoint + 200, Fe.mBoilingPoint).setRGBaLiquid(255, 80, 40, 255).setOriginalMod(MD.GT).alloyCentrifuge(),
	
	Alduorite				= metalore		( 8760, "Alduorite"				, SET_SHINY				, 159, 180, 180, 255)																																																																											.aspects(TC.METALLUM, 2, TC.PERMUTATIO							, 1).setOriginalMod(MD.MET).visDefault().setMeltingPoint(1567),
	Infuscolium				= metalore		( 8761, "Infuscolium"			, SET_METALLIC			, 146,  33,  86, 255)																																																																											.aspects(TC.METALLUM, 2, TC.PRAECANTIO							, 1).setOriginalMod(MD.MET).visDefault().setMeltingPoint(1828),
	Rubracium				= metalore		( 8762, "Rubracium"				, SET_METALLIC			, 151,  45,  45, 255)																																																																											.aspects(TC.METALLUM, 2, TC.LUCRUM								, 1).setOriginalMod(MD.MET).visDefault().setMeltingPoint(1847),
	Meutoite				= metalore		( 8763, "Meutoite"				, SET_METALLIC			,  95,  82, 105, 255)																																																																											.aspects(TC.METALLUM, 2, TC.ALIENIS								, 1).setOriginalMod(MD.MET).visDefault().setMeltingPoint(1837),
	Lemurite				= metalore		( 8764, "Lemurite"				, SET_METALLIC			, 219, 219, 219, 255)																																																																											.aspects(TC.METALLUM, 2, TC.MOTUS								, 1).setOriginalMod(MD.MET).visDefault().setMeltingPoint(1179),
	Ceruclase				= metalore		( 8765, "Ceruclase"				, SET_METALLIC			, 140, 189, 208, 255).setQuality( 6.0F, 1280, 2)																																																																				.aspects(TC.METALLUM, 2, TC.TEMPESTAS							, 1).setOriginalMod(MD.MET).visDefault().setMeltingPoint(1867),
	Atlarus					= metalore		( 8766, "Atlarus"				, SET_METALLIC			, 204, 179,   0, 255).setQuality( 4.0F, 4480, 4)																																																																				.aspects(TC.METALLUM, 2, TC.COGNITO								, 1).setOriginalMod(MD.MET).visDefault().setMeltingPoint(3276),
	Oureclase				= metalore		( 8767, "Oureclase"				, SET_METALLIC			, 183,  98,  21, 255).setQuality( 6.0F, 1920, 3)																																																																				.aspects(TC.METALLUM, 2, TC.AER									, 1).setOriginalMod(MD.MET).visDefault().setMeltingPoint(2789),
	Kalendrite				= metalore		( 8768, "Kalendrite"			, SET_METALLIC			, 170,  91, 189, 255).setQuality( 5.0F, 2560, 3)																																																																				.aspects(TC.METALLUM, 2, TC.ORDO								, 1).setOriginalMod(MD.MET).visDefault().setMeltingPoint(2679),
	Orichalcum				= metalore		( 8769, "Orichalcum"			, SET_METALLIC			,  84, 122,  56, 255).setQuality( 4.5F, 3456, 3)																																																																				.aspects(TC.METALLUM, 2, TC.TERRA								, 1).setOriginalMod(MD.MET).visDefault().setMeltingPoint(2897),
	Carmot					= metalore		( 8770, "Carmot"				, SET_METALLIC			, 217, 205, 140, 255).setQuality(16.0F,  128, 1)																																																																				.aspects(TC.METALLUM, 2, TC.NEBRISUM							, 1).setOriginalMod(MD.MET).visDefault().setMeltingPoint(1178),
	Sanguinite				= metalore		( 8771, "Sanguinite"			, SET_METALLIC			, 185,   0,   0, 255).setQuality( 3.0F, 4480, 4)																																																																				.aspects(TC.METALLUM, 2, TC.SANO								, 1).setOriginalMod(MD.MET).visDefault().setMeltingPoint(3104),
	Vyroxeres				= metalore		( 8772, "Vyroxeres"				, SET_METALLIC			,  85, 224,   1, 255).setQuality( 9.0F,  768, 3)																																																																				.aspects(TC.METALLUM, 2, TC.VENENUM								, 1).setOriginalMod(MD.MET).visDefault().setMeltingPoint(2348),
	Eximite					= metalore		( 8773, "Eximite"				, SET_METALLIC			, 124,  90, 150, 255).setQuality( 5.0F, 2560, 3)																																																																				.aspects(TC.METALLUM, 2, TC.ITER								, 1).setOriginalMod(MD.MET).visDefault().setMeltingPoint(2758),
	Prometheum				= metalore		( 8774, "Prometheum"			, SET_METALLIC			,  90, 129,  86, 255).setQuality( 8.0F,  512, 1)																																																																				.aspects(TC.METALLUM, 2, TC.POTENTIA							, 1).setOriginalMod(MD.MET).visDefault().setMeltingPoint(1067),
	Ignatius				= metalore		( 8775, "Ignatius"				, SET_METALLIC			, 255, 169,  83, 255).setQuality(12.0F,  512, 2)																																																																				.aspects(TC.METALLUM, 2, TC.IGNIS								, 1).setOriginalMod(MD.MET).visDefault().setMeltingPoint(1978),
	Vulcanite				= metalore		( 8776, "Vulcanite"				, SET_METALLIC			, 255, 132,  72, 255).setQuality( 5.0F, 3840, 3)																																																																				.aspects(TC.METALLUM, 2, TC.VITREUS								, 1).setOriginalMod(MD.MET).visDefault().setMeltingPoint(2978),
	DeepIron				= metalore		( 8641, "Deep Iron"				, SET_METALLIC			,  73,  91, 105, 255).setQuality( 6.0F,  384, 2).add(MAGNETIC_PASSIVE)																																																															.aspects(TC.METALLUM, 2, TC.MAGNETO								, 1).setOriginalMod(MD.MET).visDefault().stealStatsEnergetic(Fe),
	ShadowIron				= metalore		( 8670, "Shadow Iron"			, SET_METALLIC			,  95,  76,  63, 255).setQuality( 6.0F,  384, 2).add(MAGNETIC_PASSIVE)																																																															.aspects(TC.METALLUM, 2, TC.TENEBRAE							, 1).setOriginalMod(MD.MET).visDefault().stealStatsEnergetic(WroughtIron),
	Adamantine				= metalore		( 8784, "Adamantine"			, SET_METALLIC			, 255,   0,  64, 255).setQuality(10.0F,	4500, 5).add(MAGNETIC_PASSIVE, DECOMPOSABLE, ELECTROLYSER)																	.setMcfg( 0, Ad				, 3*U, O				, 4*U)																									.aspects(TC.METALLUM, 5, TC.PRAECANTIO							, 5).setOriginalMod(MD.MET).visDefault(Ad).stealStatsEnergetic(Ad),
	AstralSilver			= alloymachore	( 8676, "Astral Silver"			, SET_SHINY				, 230, 230, 255, 255).setQuality(10.0F,	  64, 2).add(MAGICAL, CENTRIFUGE, MOLTEN, WASHING_MERCURY)																	.setMcfg( 2, Ag				, 2*U, Ma				, 1*U)																									.aspects(TC.METALLUM, 2, TC.LUCRUM			, 1, TC.PRAECANTIO	, 1).setOriginalMod(MD.MET).visDefault().stealStatsEnergetic(Ag),
	Midasium				= alloymachore	( 8677, "Midasium"				, SET_SHINY				, 255, 200,  40, 255).setQuality(12.0F,	  64, 2).add(MAGICAL, CENTRIFUGE, MOLTEN, WASHING_MERCURY)																	.setMcfg( 2, Au				, 2*U, Ma				, 1*U)																									.aspects(TC.METALLUM, 2, TC.LUCRUM			, 2, TC.PRAECANTIO	, 1).setOriginalMod(MD.MET).visDefault().stealStatsEnergetic(Au),
	Mithril					= alloymachore	( 8678, "Mithril"				, SET_SHINY				, 100, 140, 250, 255).setQuality(14.0F,	  64, 3).add(MAGICAL, CENTRIFUGE, MOLTEN, WASHING_MERCURY)																	.setMcfg( 2, Pt				, 2*U, Ma				, 1*U)																									.aspects(TC.METALLUM, 2, TC.LUCRUM			, 3, TC.PRAECANTIO	, 1).setOriginalMod(MD.MET).visDefault().stealStatsEnergetic(Pt),
	ShadowSteel				= alloy			( 8671, "Shadow Steel"			, SET_METALLIC			, 136, 115,  98, 255).setQuality( 6.0F,  768, 2).add(MAGNETIC_PASSIVE)																								.setMcfg( 0, ShadowIron		, 1*U, Lemurite			, 1*U)																									.aspects(TC.METALLUM, 2, TC.TENEBRAE		, 1, TC.MOTUS		, 1).setOriginalMod(MD.MET).visDefault().alloyCentrifuge(),
	Inolashite				= alloy			( 8777, "Inolashite"			, SET_METALLIC			, 148, 216, 187, 255).setQuality( 8.0F, 2304, 3)																													.setMcfg( 0, Alduorite		, 1*U, Ceruclase		, 1*U)																									.aspects(TC.METALLUM, 2, TC.AER				, 1, TC.AQUA		, 1).setOriginalMod(MD.MET).visDefault().alloyCentrifuge(),
	Haderoth				= alloy			( 8778, "Haderoth"				, SET_METALLIC			, 119,  52,  30, 255).setQuality(10.0F, 3200, 3)																													.setMcfg( 0, Mithril		, 1*U, Rubracium		, 1*U)																									.aspects(TC.METALLUM, 2, TC.LUCRUM			, 1, TC.PRAECANTIO	, 1).setOriginalMod(MD.MET).visDefault().alloyCentrifuge(),
	Celenegil				= alloy			( 8779, "Celenegil"				, SET_METALLIC			, 148, 204,  72, 255).setQuality(10.0F, 4096, 3)																													.setMcfg( 0, Pt				, 1*U, Orichalcum		, 1*U)																									.aspects(TC.METALLUM, 2, TC.NEBRISUM		, 1, TC.VITREUS		, 1).setOriginalMod(MD.MET).visDefault(Orichalcum).alloyCentrifuge(),
	Desichalkos				= alloy			( 8781, "Desichalkos"			, SET_METALLIC			, 114,  47, 168, 255).setQuality(11.0F, 4608, 4)																													.setMcfg( 0, Eximite		, 1*U, Meutoite			, 1*U)																									.aspects(TC.METALLUM, 2, TC.ITER			, 1, TC.ALIENIS		, 1).setOriginalMod(MD.MET).visDefault().alloyCentrifuge(),
	Tartarite				= alloy			( 8782, "Tartarite"				, SET_METALLIC			, 255, 118,  60, 255).setQuality(20.0F, 7680, 5).add(MAGNETIC_PASSIVE)																								.setMcfg( 0, Adamantine		, 1*U, Atlarus			, 1*U)																									.aspects(TC.METALLUM, 2, TC.INSTRUMENTUM	, 1, TC.COGNITO		, 1).setOriginalMod(MD.MET).visDefault().alloyCentrifuge(),
	Amordrine				= alloy			( 8783, "Amordrine"				, SET_METALLIC			, 169, 141, 177, 255).setQuality( 7.0F, 3328, 3)																													.setMcfg( 0, Prometheum		, 1*U, Kalendrite		, 1*U)																									.aspects(TC.METALLUM, 2, TC.ORDO			, 1, TC.POTENTIA	, 1).setOriginalMod(MD.MET).visDefault().alloyCentrifuge(),
	
	
	Electrum				= alloymachore	( 8600, "Electrum"				, SET_SHINY				, 255, 255, 100, 255).add(UUM, WIRES, RAILS, WASHING_MERCURY, MORTAR, MOLTEN).setQuality(12.0F,   64, 2)															.setMcfg( 0, Ag				, 1*U, Au				, 1*U)																									.aspects(TC.METALLUM, 2, TC.LUCRUM, 2).setOriginalMod(MD.TE).alloyCentrifuge(),
	SterlingSilver			= alloymachine	( 8601, "Sterling Silver"		, SET_SHINY				, 250, 220, 225, 255).add(UUM, MORTAR).setQuality(13.0F,  128, 2)																									.setMcfg( 0, Cu				, 1*U, Ag				, 4*U)																									.aspects(TC.METALLUM, 2, TC.LUCRUM, 2).setOriginalMod(MD.TFC).alloyCentrifuge(),
	RoseGold				= alloymachine	( 8602, "Rose Gold"				, SET_SHINY				, 255, 230,  30, 255).add(UUM, MORTAR).setQuality(14.0F,  128, 2)																									.setMcfg( 0, Cu				, 1*U, Au				, 4*U)																									.aspects(TC.METALLUM, 2, TC.LUCRUM, 2).setOriginalMod(MD.TFC).alloyCentrifuge(),
	Angmallen				= alloymachine	( 8603, "Angmallen"				, SET_SHINY				, 215, 225, 138, 255).add(UUM, MORTAR, MAGNETIC_PASSIVE, MOLTEN).setQuality(10.0F,  128, 2)																			.setMcfg( 0, Au				, 1*U, WroughtIron		, 1*U)																									.aspects(TC.METALLUM, 2, TC.LUCRUM, 1).setOriginalMod(MD.MET).alloyCentrifuge(),
	InductiveAlloy			= alloymachine	( 8604, "GoldInductive"			, SET_SHINY				, 255, 150,  75, 255).add(UUM)																																		.setMcfg( 0, Au				, 1*U, Redstone			, 1*U)																									.aspects(TC.METALLUM, 2, TC.MAGNETO, 1).setOriginalMod(MD.RoC).alloyCentrifuge().setLocal("Inductive Alloy"),
	Cd_In_Ag_Alloy			= alloymachine	( 8605, "Cd-In-Ag-Alloy"		, SET_METALLIC			, 100, 100, 128, 255).add(UUM)																																		.setMcfg( 0, Cd				, 1*U, In				, 1*U, Ag				, 1*U)																			.aspects(TC.METALLUM, 2, TC.TENEBRAE, 1).setOriginalMod(MD.RoC).alloyCentrifuge(),
	GildedIron				= alloymachine	( 8606, "Gilded Iron"			, SET_SHINY				, 255, 230,  80, 255).add(COATED, CENTRIFUGE, MAGNETIC_PASSIVE).setQuality(12.0F,  256, 2)																			.setMcfg( 9, Fe				, 9*U, Au				, 1*U)																									.aspects(TC.METALLUM, 3, TC.LUCRUM, 1).setSmelting(Fe, U).setForging(Fe, U), // from Flaxbeard, obviously not an Alloy, but coated in Gold. This also gives an example on how to make such things as coated metals.
	
	Brass					= alloymachine	( 8620, "Brass"					, SET_METALLIC			, 255, 180,   0, 255).add(UUM, FURNACE, MORTAR, MOLTEN).setQuality( 7.0F, 96, 1)																					.setMcfg( 0, Cu				, 3*U, Zn				, 1*U)																									.aspects(TC.METALLUM, 2, TC.INSTRUMENTUM, 1).setStatsEnergetic(1160, Cu.mBoilingPoint).setOriginalMod(MD.RP).alloyCentrifuge(),
	CobaltBrass				= alloymachine	( 8621, "Cobalt Brass"			, SET_METALLIC			, 180, 180, 160, 255).add(UUM).setQuality( 8.0F, 256, 2)																											.setMcfg( 0, Brass			, 7*U, Al				, 1*U, Co				, 1*U)																			.aspects(TC.METALLUM, 2, TC.FABRICO, 1).setOriginalMod(MD.GT).alloyCentrifuge(),
	AluminiumAlloy			= alloymachine	( 8622,	"Aluminium Alloy"		, SET_METALLIC			, 200, 200, 180, 255).add(UUM, CENTRIFUGE)																															.setMcfg(45, Al				,45*U, Si				, 1*U)																									.aspects(TC.METALLUM, 2, TC.FABRICO, 1).stealQuality(Al).setOriginalMod(MD.RoC).visDefault(),
	
	Bronze					= alloymachine	( 8610, "Bronze"				, SET_METALLIC			, 210, 130,  60, 255).add(UUM, RAILS, FURNACE, MORTAR, MOLTEN).setQuality( 6.0F,  192, 2)																			.setMcfg( 0, Cu				, 3*U, Sn				, 1*U)																									.aspects(TC.METALLUM, 2, TC.INSTRUMENTUM, 1).setStatsEnergetic(Cu.mMeltingPoint, Cu.mBoilingPoint).setOriginalMod(MD.IC2).alloyCentrifuge(),
	BlackBronze				= alloymachine	( 8611, "Black Bronze"			, SET_DULL				, 100,  50, 125, 255).add(UUM, MORTAR).setQuality(12.0F,	 256, 2)																								.setMcfg( 0, Cu				, 3*U, Electrum			, 2*U)																									.aspects(TC.METALLUM, 2, TC.INSTRUMENTUM, 1).setOriginalMod(MD.TFC).alloyCentrifuge(),
	BismuthBronze			= alloymachine	( 8612, "Bismuth Bronze"		, SET_DULL				, 100, 125, 125, 255).add(UUM, FURNACE, MORTAR).setQuality( 8.0F,  256, 2)																							.setMcfg( 0, Bi				, 1*U, Brass			, 4*U)																									.aspects(TC.METALLUM, 2, TC.INSTRUMENTUM, 1).setOriginalMod(MD.TFC).alloyCentrifuge(),
	Hepatizon				= alloymachine	( 8613, "Hepatizon"				, SET_METALLIC			, 117,  94, 117, 255).add(UUM, MORTAR, MOLTEN).setQuality(12.0F,  128, 2)																							.setMcfg( 0, Au				, 1*U, Bronze			, 1*U)																									.aspects(TC.METALLUM, 2, TC.INSTRUMENTUM, 1).setOriginalMod(MD.MET).alloyCentrifuge(),
	
	Steel					= alloymachore	( 8630, "Steel"					, SET_METALLIC			, 130, 130, 130, 255).add(UUM, WIRES, RAILS, MORTAR, MAGNETIC_PASSIVE, MOLTEN).setQuality( 6.0F,  512, 2)															.setMcfg(50, WroughtIron	,50*U, C 				, 1*U)																									.aspects(TC.METALLUM, 2, TC.ORDO, 1).setRGBaLiquid(220, 80, 40, 255).setOriginalMod(MD.RC).alloyCentrifuge(),
	BlackSteel				= alloymachine	( 8631, "Black Steel"			, SET_METALLIC			,  90,  90,  90, 255).add(UUM).setQuality( 6.5F,  768, 2)																											.setMcfg( 0, Ni				, 1*U, BlackBronze		, 1*U, Steel			, 3*U)																			.aspects(TC.METALLUM, 2, TC.ORDO, 1, TC.INSTRUMENTUM, 1).setOriginalMod(MD.TFC).alloyCentrifuge(),
	RedSteel				= alloymachine	( 8632, "Red Steel"				, SET_METALLIC			, 140, 100, 100, 255).add(UUM).setQuality( 7.0F,  896, 2)																											.setMcfg( 0, SterlingSilver	, 1*U, BismuthBronze	, 1*U, Steel			, 2*U, BlackSteel		, 4*U)													.aspects(TC.METALLUM, 2, TC.ORDO, 1, TC.INSTRUMENTUM, 1).setOriginalMod(MD.TFC).alloyCentrifuge(),
	BlueSteel				= alloymachine	( 8633, "Blue Steel"			, SET_METALLIC			, 100, 100, 140, 255).add(UUM).setQuality( 7.5F, 1024, 2)																											.setMcfg( 0, RoseGold		, 1*U, Brass			, 1*U, Steel			, 2*U, BlackSteel		, 4*U)													.aspects(TC.METALLUM, 2, TC.ORDO, 1, TC.INSTRUMENTUM, 1).setOriginalMod(MD.TFC).alloyCentrifuge(),
	DamascusSteel			= alloymachine	( 8634, "Damascus Steel"		, SET_METALLIC			, 110, 110, 110, 255).add(UUM, CENTRIFUGE, MAGNETIC_PASSIVE).setQuality( 8.0F, 1280, 2)																				.setMcfg(50, Steel			,50*U, V 				, 1*U, W 				, 1*U)																			.aspects(TC.METALLUM, 2, TC.ORDO, 2, TC.INSTRUMENTUM, 1).setOriginalMod(MD.GT),
	VanadiumSteel			= alloymachine	( 8653, "VanadiumSteel"			, SET_METALLIC			, 100, 100, 100, 255).add(UUM, MAGNETIC_PASSIVE, MOLTEN).setQuality(7.0F, 512, 3)																					.setMcfg( 0, Steel			, 4*U, V				, 1*U)																									.aspects(TC.METALLUM, 2, TC.ORDO, 1, TC.INSTRUMENTUM, 1).setOriginalMod(MD.GT).setLocal("Vanadiumsteel").alloyCentrifuge(),
	TungstenSteel			= alloymachine	( 8635, "Tungstensteel"			, SET_METALLIC			, 100, 100, 160, 255).add(UUM, RAILS, MAGNETIC_PASSIVE, MOLTEN).setQuality(10.0F, 5120, 4)																			.setMcfg( 0, Steel			, 1*U, W				, 1*U)																									.aspects(TC.METALLUM, 2, TC.ORDO, 1, TC.TUTAMEN, 1).addIdenticalNames("TungstenSteel", "Wolframsteel", "WolframSteel").setOriginalMod(MD.GT).alloyCentrifuge(),
	TungstenCarbide			= alloymachine	( 8638, "Tungsten Carbide"		, SET_METALLIC			, 123, 123, 123, 255).add(UUM, RAILS).setQuality(10.0F, 5120, 4)																									.setMcfg( 0, W				, 1*U, C				, 1*U)																									.aspects(TC.METALLUM, 2, TC.ORDO, 1, TC.TUTAMEN, 1).setStatsEnergetic(3070, 6270).setDensity(15.6).addIdenticalNames("Carbide", "WolframCarbide").setOriginalMod(MD.ReC).alloyCentrifuge(),
	HSLA					= alloymachine	( 8637,	"HSLA-Steel"			, SET_METALLIC			, 210, 210, 255, 255).add(UUM, WIRES, RAILS, CENTRIFUGE, MORTAR, MAGNETIC_PASSIVE, MOLTEN)																			.setMcfg(50, WroughtIron	,50*U, C 				, 1*U)																									.aspects(TC.METALLUM, 2, TC.ORDO, 1).steal(Steel).setStatsEnergetic(1873, Fe.mBoilingPoint, Fe.mPlasmaPoint).setOriginalMod(MD.RoC).addIdenticalNames("HSLA").visDefault(),
	SpringSteel				= alloymachine	( 8639,	"HSLA-SpringSteel"		, SET_METALLIC			, 220, 100, 100, 255).add(UUM, CENTRIFUGE, MAGNETIC_PASSIVE)																										.setMcfg(45, HSLA			,45*U, Redstone			, 2*U)																									.aspects(TC.METALLUM, 2, TC.ORDO, 1).steal(HSLA).setOriginalMod(MD.RoC).setLocal("Spring Steel").visDefault(),
	AnnealedCopper			= metalnd		( 8640, "Annealed Copper"		, SET_SHINY				, 255, 120,  20, 255).add(UUM, FURNACE, MORTAR, WIRES, MOLTEN)																										.setMcfg( 0, Cu				, 1*U)																															.aspects(TC.METALLUM, 2, TC.PERMUTATIO, 1			).setPulver(Cu		, U)																																						.steal(Cu).setStatsEnergetic(2800, Cu.mBoilingPoint).setOriginalMod(MD.GT).alloyCentrifuge(),
	PigIron					= metalmachore	( 8642, "Pig Iron"				, SET_METALLIC			, 200, 180, 180, 255).add(UUM, MORTAR, MAGNETIC_PASSIVE, MOLTEN)																									.setMcfg( 0, WroughtIron	, 1*U)																															.aspects(TC.METALLUM, 2, TC.INSTRUMENTUM, 1			).setPulver(Fe		, U).setSmelting(WroughtIron	, U)																														.steal(WroughtIron).setQuality( 6.0F,  384,  2),
	IronCompressed			= alloymachnd	( 8644, "IronCompressed"		, SET_METALLIC			, 128, 128, 128, 255).add(UUM, CENTRIFUGE, MORTAR, MAGNETIC_PASSIVE)																								.setMcfg( 0, Fe				, 1*U)																															.aspects(TC.METALLUM, 2, TC.TERRA, 1				).setPulver(Fe		, U).setSmelting(Fe				, U)																														.steal(Fe).setLocal("Compressed Iron"),
	IronMagnetic			= metalmachnd	( 8645, "IronMagnetic"			, SET_MAGNETIC			, 200, 200, 200, 255).add(UUM, LAYERED, MORTAR, MAGNETIC_ACTIVE)																									.setMcfg( 0, Fe				, 1*U)																															.aspects(TC.METALLUM, 2, TC.MAGNETO, 1				).setBending(Fe		, U).setCompressing(Fe		, U).setPulver(Fe		, U).setSmashing(Fe		, U).setSmelting(Fe		, U).setWorking(Fe		, U).setForging(Fe		, U)	.steal(Fe).setLocal("Magnetic Iron").setOriginalMod(MD.GT),
	SteelMagnetic			= metalmachnd	( 8646, "SteelMagnetic"			, SET_MAGNETIC			, 128, 128, 128, 255).add(UUM, LAYERED, MORTAR, MAGNETIC_ACTIVE)																									.setMcfg( 0, Steel			, 1*U)																															.aspects(TC.METALLUM, 1, TC.ORDO, 1, TC.MAGNETO, 1	).setBending(Steel	, U).setCompressing(Steel	, U).setPulver(Steel	, U).setSmashing(Steel	, U).setSmelting(Steel	, U).setWorking(Steel	, U).setForging(Steel	, U)	.steal(Steel).setLocal("Magnetic Steel").setOriginalMod(MD.GT),
	NeodymiumMagnetic		= metalmachnd	( 8647, "NeodymiumMagnetic"		, SET_MAGNETIC			, 100, 100, 100, 255).add(UUM, LAYERED, MORTAR, MAGNETIC_ACTIVE)																									.setMcfg( 0, Nd				, 1*U)																															.aspects(TC.METALLUM, 1, TC.MAGNETO, 3				).setBending(Fe		, U).setCompressing(Nd		, U).setPulver(Nd		, U).setSmashing(Nd		, U).setSmelting(Nd		, U).setWorking(Nd		, U).setForging(Nd		, U)	.steal(Nd).setLocal("Magnetic Neodymium").setOriginalMod(MD.GT),
	DarkIron				= metalmachore	( 8648,	"Dark Iron"				, SET_DULL				,  55,  40,  60, 255).add(MAGNETIC_PASSIVE)																																																																						.aspects(TC.METALLUM, 2, TC.IGNIS, 1).steal(Fe).setQuality( 7.0F,	384,  3).setStatsEnergetic(Steel.mMeltingPoint + 200, Fe.mBoilingPoint).addIdenticalNames("FzDarkIron", "FZDarkIron").setOriginalMod(MD.FZ).visDefault(),
	MeteoricIron			= metalmachore	( 8649,	"Meteoric Iron"			, SET_METALLIC			, 150, 140, 120, 255).add(MAGNETIC_ACTIVE, MOLTEN)																													.setMcfg( 0, Fe				, 1*U)																															.aspects(TC.METALLUM, 2, TC.MAGNETO, 1).steal(Fe).setQuality( 6.0F,	384,  2).setStatsEnergetic(Fe.mMeltingPoint + 200, Fe.mBoilingPoint + 200).visDefault(),
	MeteoricSteel			= metalmachine	( 8650,	"Meteoric Steel"		, SET_METALLIC			, 140, 130, 110, 255).add(MAGNETIC_ACTIVE, MOLTEN)																													.setMcfg( 0, Steel			, 1*U)																															.aspects(TC.METALLUM, 2, TC.MAGNETO, 1, TC.ORDO, 1).steal(Steel).setQuality( 6.0F,	768,  2).setStatsEnergetic(Steel.mMeltingPoint + 200, Steel.mBoilingPoint + 200).visDefault(),
	SteelGalvanized			= alloymachine	( 8651, "SteelGalvanized"		, SET_METALLIC			, 250, 240, 240, 255).add(COATED, CENTRIFUGE).setQuality( 7.0F, 768, 2)																								.setMcfg( 9, Steel			, 9*U, Zn				, 1*U)																									.aspects(TC.METALLUM, 3, TC.SANO, 1).setSmelting(Steel, U).setForging(Steel, U).setLocal("Galvanized Steel"),
	TungstenSintered		= alloymachnd	( 8652, "TungstenSintered"		, SET_METALLIC			,  70,  70,  70, 255).add(UUM, RAILS).setQuality(8.0F, 5120, 3)																										.setMcfg( 0, W				, 1*U)																															.aspects(TC.METALLUM, 3, TC.TUTAMEN, 1).steal(W).setAllToTheOutputOf(W).setForging(null, U).setCutting(null, U).setWorking(null, U).setSmashing(null, U).setLocal("Sintered Tungsten").setOriginalMod(MD.RoC).visDefault(),
	
	ElectrotineAlloy		= alloy			( 8658, "Electrotine Alloy"		, SET_DULL				, 100, 180, 255, 255).add(UUM, WIRES, MOLTEN)																														.setMcfg( 1, WroughtIron	, 1*U, Electrotine		, 8*U)																									.aspects(TC.ELECTRUM, 3).setStatsEnergetic(1400, Fe.mBoilingPoint).setOriginalMod(MD.PR).alloyCentrifuge().visDefault(Electrotine),
	BlueAlloy				= alloy			( 8659, "Blue Alloy"			, SET_DULL				, 100, 180, 255, 255).add(UUM, WIRES, MOLTEN)																														.setMcfg( 1, Ag				, 1*U, Nikolite			, 4*U)																									.aspects(TC.ELECTRUM, 3).setStatsEnergetic(1400, Ag.mBoilingPoint).setOriginalMod(MD.RP).alloyCentrifuge().visDefault(Nikolite),
	RedAlloy				= alloy			( 8660, "Red Alloy"				, SET_DULL				, 200,   0,   0, 255).add(UUM, WIRES, MOLTEN)																														.setMcfg( 1, Cu				, 1*U, Redstone			, 4*U)																									.aspects(TC.MACHINA, 3).setStatsEnergetic(1400, Cu.mBoilingPoint).setOriginalMod(MD.RP).alloyCentrifuge(),
	Invar					= alloymachine	( 8661, "Invar"					, SET_METALLIC			, 220, 220, 150, 255).add(UUM, MORTAR, MAGNETIC_PASSIVE, MOLTEN).setQuality( 6.0F,	256,  2)																		.setMcfg( 0, WroughtIron	, 2*U, Ni				, 1*U)																									.aspects(TC.METALLUM, 2, TC.GELUM, 1).setOriginalMod(MD.TE).alloyCentrifuge(),
	Constantan				= alloymachine	( 8662, "Constantan"			, SET_METALLIC			, 227, 150, 128, 255).add(UUM, MORTAR, MAGNETIC_PASSIVE, MOLTEN).setQuality( 6.0F,	 64,  1)																		.setMcfg( 0, Cu				, 1*U, Ni				, 1*U)																									.alloyCentrifuge().setOriginalMod(MD.IE), Cupronickel = Constantan.addIdenticalNames("Cupronickel"),
	Nichrome				= alloymachine	( 8663, "Nichrome"				, SET_METALLIC			, 205, 206, 246, 255).add(UUM).setQuality( 6.0F,	 64,  2)																										.setMcfg( 0, Ni				, 4*U, Cr				, 1*U)																									.alloyCentrifuge().setOriginalMod(MD.GT),
	Kanthal					= alloymachine	( 8664, "Kanthal"				, SET_METALLIC			, 194, 210, 223, 255).add(UUM).setQuality( 6.0F,	 64,  2)																										.setMcfg( 0, WroughtIron	, 1*U, Al				, 1*U, Cr				, 1*U)																			.alloyCentrifuge().setOriginalMod(MD.GT),
	Magnalium				= alloymachine	( 8665, "Magnalium"				, SET_DULL				, 200, 190, 255, 255).add(UUM, MOLTEN).setQuality( 6.0F,	256,  2)																								.setMcfg( 0, Mg				, 1*U, Al				, 2*U)																									.alloyCentrifuge().setOriginalMod(MD.GT),
	StainlessSteel			= alloymachine	( 8636, "Stainless Steel"		, SET_SHINY				, 200, 200, 220, 255).add(UUM, MOLTEN, RAILS).setQuality( 7.0F, 480,  2)																							.setMcfg( 0, WroughtIron	, 4*U, Invar			, 3*U, Cr				, 1*U, Mn				, 1*U)													.aspects(TC.METALLUM, 2, TC.ORDO, 1, TC.VITREUS, 1).setOriginalMod(MD.GT).alloyCentrifuge(),
	Ultimet					= alloymachine	( 8666, "Ultimet"				, SET_SHINY				, 180, 180, 230, 255).add(UUM).setQuality( 8.0F,1024,  3)																											.setMcfg( 0, Co				, 5*U, Cr				, 2*U, Ni				, 1*U, Mo				, 1*U)													.setOriginalMod(MD.GT).alloyCentrifuge(), // 54% Co, 26% Cr, 9% Ni, 5% Mo, 3% Fe, 2% W, 0.8% Mn, 0.3% Si, 0.08% Ni and 0.06% C
	TinAlloy				= alloymachine	( 8667, "Tin Alloy"				, SET_METALLIC			, 200, 200, 200, 255).add(UUM, MORTAR, MOLTEN).setQuality( 6.5F,	 96,  2)																						.setMcfg( 0, Sn				, 1*U, WroughtIron		, 1*U)																									.aspects(TC.METALLUM, 2, TC.FABRICO, 1).setOriginalMod(MD.RC).alloyCentrifuge(),
	BatteryAlloy			= alloy			( 8668, "Battery Alloy"			, SET_DULL				, 156, 124, 160, 255).add(UUM, MORTAR, MOLTEN)														  																.setMcfg( 0, Pb				, 4*U, Sb				, 1*U)																									.setOriginalMod(MD.GT).alloyCentrifuge(),
	SolderingAlloy			= alloy			( 8669, "Soldering Alloy"		, SET_DULL				, 220, 220, 230, 255).add(UUM, MORTAR, MOLTEN, BRITTLE, SOLDERING_MATERIAL, SOLDERING_MATERIAL_GOOD, WIRES)															.setMcfg( 0, Sn				, 9*U, Sb				, 1*U)																									.setOriginalMod(MD.GT).alloyCentrifuge(),
	IronWood				= alloymachine	( 8672, "IronWood"				, SET_WOOD				, 150, 140, 110, 255).add(MAGICAL, WOOD, FURNACE, MORTAR, MAGNETIC_PASSIVE, MOLTEN).setQuality( 6.0F,  384,  2).setLocal("Ironwood")								.setMcfg(18, WroughtIron	, 8*U, LiveRoot			, 9*U, Angmallen		, 2*U)																			.aspects(TC.METALLUM, 2, TC.ARBOR, 1, TC.PRAECANTIO, 1).setOriginalMod(MD.TF).alloyCentrifuge(),
	Steeleaf				= alloymachine	( 8673, "Steeleaf"				, SET_LEAF				,  50, 127,  50, 255).add(MAGICAL, CENTRIFUGE, MAGNETIC_PASSIVE, WOOD, MORTAR).setQuality( 8.0F,  768,  3).setSmelting(Steel, U / 4)								.setMcfg( 1, Steel			, 1*U, Ma				, 1*U)																									.aspects(TC.METALLUM, 2, TC.HERBA, 1, TC.PRAECANTIO, 1).setOriginalMod(MD.TF),
	Knightmetal				= alloymachine	( 8674, "Knightmetal"			, SET_METALLIC			, 210, 240, 200, 255).add(MAGICAL, CENTRIFUGE, MAGNETIC_PASSIVE, MOLTEN, MORTAR).setQuality( 8.0F, 1024,  3).addIdenticalNames("KnightMetal")						.setMcfg( 2, Steel			, 2*U, Ma				, 1*U)																									.aspects(TC.METALLUM, 2, TC.TUTAMEN, 1, TC.PRAECANTIO, 1).setOriginalMod(MD.TF).setStatsEnergetic(Steel.mMeltingPoint+100, Steel.mBoilingPoint+100).visDefault(),
	FierySteel				= alloymachine	( 8675,	"Fiery Steel"			, SET_FIERY				,  64,   0,   0, 255).add(MAGICAL, CENTRIFUGE, MAGNETIC_PASSIVE, MOLTEN, UNBURNABLE, BURNING, GLOWING).setQuality( 8.0F, 1024,  3)									.setMcfg( 1, Steel			, 1*U, Ma				, 1*U)																									.aspects(TC.METALLUM, 2, TC.IGNIS, 3, TC.PRAECANTIO, 1).setOriginalMod(MD.TF).setStatsEnergetic(Steel.mBoilingPoint-200, Steel.mBoilingPoint+500).visDefault(),
	Thaumium				= alloymachine	( 8679, "Thaumium"				, SET_METALLIC			, 150, 100, 200, 255).add(MAGICAL, CENTRIFUGE, MAGNETIC_PASSIVE, MOLTEN).setQuality(12.0F,	 256,  3)																.setMcfg( 1, Fe				, 1*U, Ma				, 1*U)																									.aspects(TC.METALLUM, 2, TC.PRAECANTIO, 1).setStatsEnergetic(Fe.mMeltingPoint+500, Fe.mBoilingPoint+1000).setOriginalMod(MD.TC).visDefault(),
	DarkThaumium			= alloymachine	( 8680, "Dark Thaumium"			, SET_METALLIC			, 100,  75,  75, 255).add(MAGICAL, CENTRIFUGE, MAGNETIC_PASSIVE, MOLTEN).setQuality(12.0F,	 512,  3)																																																							.visDefault(),
	VoidMetal				= alloymachine	( 8681,	"Void Metal"			, SET_METALLIC			,  30,  10,  30, 100).add(MAGICAL, CENTRIFUGE, MOLTEN).setQuality(12.0F,	2048,  4).addIdenticalNames("Void")																																																					.aspects(TC.METALLUM, 7, TC.TENEBRAE, 8, TC.VACUOS, 8, TC.ALIENIS, 2, TC.HERBA, 1).setOriginalMod(MD.TC).setStatsEnergetic(3000, 5000).visDefault(),
	Osmiridium				= alloymachine	( 8682,	"Osmiridium"			, SET_METALLIC			, 100, 100, 255, 255).add(UUM).setQuality(11.0F, 3840, 4)																											.setMcfg( 0, Os				, 1*U, Ir				, 1*U)																									.aspects(TC.METALLUM, 2, TC.MACHINA, 2, TC.NEBRISUM, 1).alloyCentrifuge(),
	Sunnarium				= alloymachine	( 8683,	"Sunnarium"				, SET_SHINY				, 255, 255,   0, 255)																																																																											,
	ChromiumDioxide 		= alloy			( 8685, "Chromium Dioxide"		, SET_DULL				,  10,  20,  10, 255).add(UUM, ELECTROLYSER, MAGNETIC_PASSIVE)																										.setMcfg( 1, Cr				, 1*U, O 				, 2*U)																									.aspects(TC.METALLUM, 2, TC.FABRICO, 1).setQuality(11.0F,  256,  3).setMeltingPoint(650), CrO2 = ChromiumDioxide,
	VanadiumGallium			= alloy			( 8686, "Vanadium-Gallium"		, SET_SHINY				, 128, 128, 140, 255).add(UUM, BRITTLE)														 																		.setMcfg( 0, V 				, 3*U, Ga				, 1*U)																									.setOriginalMod(MD.GT).alloyCentrifuge(),
	YttriumBariumCuprate	= alloy			( 8687, "Yttrium-Barium-Cuprate", SET_METALLIC			,  80,  64,  70, 255).add(UUM, BRITTLE, ELECTROLYSER, LAYERED)																										.setMcfg( 6, Y 				, 1*U, Ba				, 2*U, Cu				, 3*U, O				, 7*U)													.setOriginalMod(MD.GT).setMeltingPoint(1200),
	NiobiumNitride			= alloy			( 8688, "Niobium Nitride"		, SET_DULL				,  29,  41,  29, 255).add(UUM, BRITTLE, ELECTROLYSER)			 								 																	.setMcfg( 0, Nb				, 1*U, N 				, 1*U)																									.setOriginalMod(MD.GT).setMeltingPoint(2573), // Anti-Reflective Material
	NiobiumTitanium			= alloy			( 8689, "Niobium Titanium"		, SET_DULL				,  29,  29,  41, 255).add(UUM, BRITTLE)														  																		.setMcfg( 0, Nb				, 1*U, Ti				, 1*U)																									.setOriginalMod(MD.GT).alloyCentrifuge(),
	AluminiumBrass			= alloymachine	( 8700,	"Aluminium Brass"		, SET_METALLIC			, 220, 220, 130, 255).add(UUM, MOLTEN).setQuality( 6.0F,  64,  2).addIdenticalNames("AluminumBrass")																.setMcfg( 0, Al				, 3*U, Cu				, 1*U)																									.aspects(TC.STRONTIO, 1).setOriginalMod(MD.TiC),
	Ardite					= metalore		( 8707,	"Ardite"				, SET_METALLIC			, 200, 120,  20, 255).setQuality( 6.0F,  64,  2)																																																																				.aspects(TC.STRONTIO, 1).setOriginalMod(MD.TiC).visDefault(),
	Aredrite				= metalore		( 8701,	"Aredrite"				, SET_METALLIC			, 255, 255,   0, 255).add(MOLTEN).setQuality( 6.0F,  64,  2)																																																																	.aspects(TC.STRONTIO, 1).visDefault(),
	Alumite					= alloymachine	( 8702, "Alumite"				, SET_METALLIC			, 230, 100, 230, 255).add(MOLTEN).setQuality( 1.5F,	 64,  0)																										.setMcfg( 9, Al				, 5*U, WroughtIron		, 2*U, Obsidian			,18*U)																			.aspects(TC.STRONTIO, 2).alloyCentrifuge().setOriginalMod(MD.TiC),
	Manyullyn				= alloymachine	( 8703, "Manyullyn"				, SET_METALLIC			, 175, 100, 175, 255).add(MOLTEN).setQuality( 2.0F,	 96,  1)																										.setMcfg( 0, Co				, 1*U, Ardite			, 1*U)																									.aspects(TC.STRONTIO, 2).alloyCentrifuge().setOriginalMod(MD.TiC).visDefault(Ardite),
	VibraniumSteel			= alloymachine	( 8704, "Vibranium Steel"		, SET_SHINY				,  40,  24,  50, 255).add(CENTRIFUGE, MAGNETIC_PASSIVE).setQuality(  50.0F, 2048, 10)																				.setMcfg( 0, Vb				, 1*U, Steel			, 3*U)																									.aspects(TC.SENSUS, 1, TC.VITREUS, 1, TC.METALLUM, 1).setOriginalMod(MD.GT).visDefault(Vb),
	VibraniumSilver			= alloy			( 8705, "Vibranium Silver"		, SET_SHINY				, 240, 240, 255, 255).add(CENTRIFUGE).setQuality( 100.0F,  512,  9)																									.setMcfg( 0, Vb				, 1*U, Ag				, 3*U)																									.aspects(TC.SENSUS, 1, TC.VITREUS, 1, TC.LUCRUM, 1).setOriginalMod(MD.GT).visDefault(Vb),
	Vibramantium			= alloymachine	( 8706, "Vibramantium"			, SET_SHINY				, 250, 250, 250, 255).add(CENTRIFUGE, MAGNETIC_PASSIVE).setQuality(1000.0F, 5120, 15)																				.setMcfg( 0, Vb				, 1*U, Ad				, 3*U)																									.aspects(TC.SENSUS, 1, TC.VITREUS, 1, TC.PRAECANTIO, 1, TC.METALLUM, 1).setOriginalMod(MD.GT).visDefault(Vb),
	Signalum				= alloymachine	( 8708,	"Signalum"				, SET_METALLIC			, 255,  64,   0, 255).add(UUM, MOLTEN)																																.setMcfg( 8, Cu				, 1*U, Ag				, 2*U, RedAlloy			, 5*U)																			.aspects(TC.METALLUM, 3, TC.POTENTIA, 1).setOriginalMod(MD.TE).alloyCentrifuge(),
	Lumium					= alloymachine	( 8709,	"Lumium"				, SET_SHINY				, 255, 255,  80, 255).add(UUM, MOLTEN, LIGHTING, GLOWING)																											.setMcfg( 4, Sn				, 3*U, Ag				, 1*U, Glowstone		, 4*U)																			.aspects(TC.METALLUM, 3, TC.LUX, 1).setOriginalMod(MD.TE).alloyCentrifuge(),
	EnderiumBase			= alloymachine	( 8729,	"Enderium Base"			, SET_METALLIC			,  53,  85, 108, 255).add(UUM, MOLTEN)																																.setMcfg( 4, Sn				, 2*U, Ag				, 1*U, Pt				, 1*U)																			.aspects(TC.METALLUM, 3).setOriginalMod(MD.EIO).alloyCentrifuge(),
	Enderium				= alloymachine	( 8710,	"Enderium"				, SET_METALLIC			,  60, 125, 115, 255).add(MAGICAL, MOLTEN).setQuality( 8.0F, 256, 3)																								.setMcfg( 1, EnderiumBase	, 1*U, EnderPearl		, 1*U)																									.aspects(TC.METALLUM, 3, TC.ALIENIS, 1).setOriginalMod(MD.TE).alloyCentrifuge(),
	InfusedGold				= alloymachore	( 8712,	"Infused Gold"			, SET_SHINY				, 255, 200,  60, 255).setQuality(12.0F,	64, 3)																																																																					.visDefault(),
	RefinedGlowstone		= alloy			( 8713,	"GlowstoneRefined"		, SET_ROUGH				, 255, 255,   0, 255).setLocal("Refined Glowstone").addIdenticalNames("RefinedGlowstone")																																																										.visDefault(),
	RefinedObsidian			= alloy			( 8714,	"ObsidianRefined"		, SET_ROUGH				,  80,  50, 100, 255).setLocal("Refined Obsidian").addIdenticalNames("RefinedObsidian")																																																											.visDefault(),
	Yellorium				= metalore		( 8715,	"Yellorium"				, SET_SHINY				, 140, 130,  20, 255)																																																																											.aspects(TC.RADIO, 2).setOriginalMod(MD.BR).visDefault(),
	Blutonium				= metalore		( 8716,	"Blutonium"				, SET_SHINY				,  60,  60, 180, 255)																																																																											.aspects(TC.RADIO, 3).setOriginalMod(MD.BR).visDefault(),
	Cyanite					= metalore		( 8717,	"Cyanite"				, SET_SHINY				,  50, 110, 150, 255)																																																																											.aspects(TC.RADIO, 1).setOriginalMod(MD.BR).visDefault(),
	Bedrock_HSLA_Alloy		= alloy			( 8718,	"Bedrock-HSLA-Alloy"	, SET_STONE				,  64,  64,  64, 255).add(CENTRIFUGE, MAGNETIC_PASSIVE)																												.setMcfg( 1, Bedrock		, 4*U, HSLA				, 1*U)																									.aspects(TC.TERRA, 5).setQuality(10.0F, 2560, 5).setMeltingPoint(4000).setOriginalMod(MD.RoC).visDefault(),
	FakeOsmium				= alloy			( 8719,	"Osmium"				, SET_METALLIC			, 160, 160, 255, 255).add(G_INGOT_MACHINE_ORES, MOLTEN).steal(Os).setQuality(7.0F, 256, 2)																																																										.aspects(TC.METALLUM, 3, TC.MACHINA, 1).setLocal("'Osmium'").setOriginalMod(MD.Mek).visDefault(),
	Manasteel				= alloymachine	( 8720,	"Manasteel"				, SET_SHINY				, 110, 200, 250, 255).add(MAGICAL)																																																																								.aspects(TC.PRAECANTIO, 2).visDefault(),
	Terrasteel				= alloymachine	( 8721,	"Terrasteel"			, SET_SHINY				, 110, 200,  50, 255).add(MAGICAL)																																																																								.aspects(TC.PRAECANTIO, 2).visDefault(),
	ElvenElementium			= alloymachine	( 8722,	"Elven Elementium"		, SET_SHINY				, 250, 120, 250, 255).add(MAGICAL)																																																																								.aspects(TC.AURAM, 2).visDefault(),
	
	PulsatingIron			= alloy			( 8725,	"Pulsating Iron"		, SET_METALLIC			, 100, 160, 110, 255).add(MAGNETIC_PASSIVE).stealQuality(WroughtIron)																								.setMcfg( 1, WroughtIron	, 1*U, EnderPearl		, 1*U)																									.aspects(TC.METALLUM, 3, TC.ALIENIS, 1).setOriginalMod(MD.EIO).addIdenticalNames("PhasedIron").alloyCentrifuge(),
	ConductiveIron			= alloy			( 8727,	"Conductive Iron"		, SET_METALLIC			, 170, 140, 140, 255).add(MAGNETIC_PASSIVE, UUM).stealQuality(WroughtIron)																							.setMcfg( 1, WroughtIron	, 1*U, Redstone			, 1*U)																									.aspects(TC.METALLUM, 3, TC.POTENTIA, 1).setOriginalMod(MD.EIO).alloyCentrifuge(),
	EnergeticAlloy			= alloy			( 8728,	"Energetic Alloy"		, SET_DULL				, 200, 120,  50, 255).add(UUM).stealQuality(Au)																														.setMcfg( 1, InductiveAlloy	, 2*U, Glowstone		, 1*U)																									.aspects(TC.METALLUM, 2, TC.LUCRUM, 1, TC.LUX, 1).setOriginalMod(MD.EIO).alloyCentrifuge(),
	VibrantAlloy			= alloy			( 8726,	"Vibrant Alloy"			, SET_SHINY				, 160, 170,  70, 255).stealQuality(EnergeticAlloy)																													.setMcfg( 1, EnergeticAlloy	, 1*U, EnderPearl		, 1*U)																									.aspects(TC.METALLUM, 2, TC.LUCRUM, 1, TC.LUX, 1, TC.ALIENIS, 1).setOriginalMod(MD.EIO).addIdenticalNames("PhasedGold").alloyCentrifuge(),
	ElectrumFlux			= alloy			( 8711,	"Electrum Flux"			, SET_SHINY				, 255, 255, 120, 255).add(UUM).setQuality(14.0F, 64, 2)																												.setMcfg( 1, Electrum		, 1*U, Redstone			, 2*U)																									.aspects(TC.METALLUM, 2, TC.PERMUTATIO, 2).alloyCentrifuge(),
	ElectricalSteel			= alloy			( 8730,	"Electrical Steel"		, SET_METALLIC			, 140, 140, 140, 255).add(MAGNETIC_PASSIVE, UUM).stealQuality(Steel)																								.setMcfg( 1, Steel			, 1*U, Si				, 1*U)																									.aspects(TC.METALLUM, 3, TC.ELECTRUM, 1).setOriginalMod(MD.EIO).alloyCentrifuge(),
	DarkSteel				= alloy			( 8731,	"Dark Steel"			, SET_METALLIC			,  60,  60,  60, 255).add(MAGNETIC_PASSIVE, RAILS).stealQuality(Steel)																								.setMcfg( 1, Steel			, 1*U, Obsidian			, 9*U)																									.aspects(TC.METALLUM, 3, TC.TENEBRAE, 1).setOriginalMod(MD.EIO).alloyCentrifuge(),
	Soularium				= alloy			( 8732,	"Soularium"				, SET_DULL				,  90,  70,  50, 255).stealQuality(Au)																																.setMcfg( 1, SoulSand		, 1*U, Au				, 1*U)																									.aspects(TC.METALLUM, 3, TC.SPIRITUS, 1).setOriginalMod(MD.EIO).alloyCentrifuge(),
	RedstoneAlloy			= alloy			( 8733,	"Redstone Alloy"		, SET_METALLIC			, 140,  50,  50, 255).add(UUM, MOLTEN)																																.setMcfg( 1, Si				, 1*U, Redstone			, 1*U)																									.aspects(TC.METALLUM, 3, TC.MACHINA, 1).setOriginalMod(MD.EIO).alloyCentrifuge(),
	SpectreIron				= alloymachine	( 8734,	"Spectre Iron"			, SET_SHINY				, 150, 200, 200, 200).add(MAGNETIC_PASSIVE, MAGICAL, MOLTEN, GLOWING).setQuality( 8.5F, 768, 2)																		.setMcfg( 1, WroughtIron	, 1*U, Ectoplasm		, 1*U)																									.aspects(TC.METALLUM, 3, TC.SPIRITUS, 3).setOriginalMod(MD.RT).alloyCentrifuge().visDefault().stealStatsEnergetic(Fe),
	
	Desh					= metalmachore	( 8750,	"Desh"					, SET_DULL				,  40,  40,  40, 255).setQuality( 4.0F,   1280,  3)																																																																				.setOriginalMod(MD.GC),
	DuraniumAlloy			= alloymachine	( 8751,	"Duranium"				, SET_METALLIC			,  75, 175, 175, 255).add(UUM).setQuality( 8.0F,   1280,  4)																										.setMcfg( 0, Dn				, 7*U, Mg				, 1*U)																									.aspects(TC.METALLUM, 2, TC.TUTAMEN, 2).setOriginalMod(MD.MO).setLocal("Duranium Alloy").alloyCentrifuge(), // That Info with Magnesium/Magnesite was pretty hard to find, but I found it!
	TritaniumAlloy			= alloymachine	( 8752,	"Tritanium"				, SET_METALLIC			,  55, 155, 155, 255).add(UUM).setQuality(12.0F,   2560,  5)																										.setMcfg( 0, Tn				, 3*U, Dn				, 1*U)																									.aspects(TC.METALLUM, 2, TC.TUTAMEN, 3, TC.VITREUS, 1).setOriginalMod(MD.MO).setLocal("Tritanium Alloy").alloyCentrifuge(),
	Dolamide				= oredust		( 8753,	"Dolamide"				, SET_METALLIC			, 188, 100, 122, 255)																																																																											.aspects(TC.POTENTIA, 3, TC.RADIO, 2).setOriginalMod(MD.GT),
	Oriharukon				= metalmachore	( 8754,	"Oriharukon"			, SET_METALLIC			, 220, 220, 240, 255).setQuality( 8.0F,   2560,  2)																																																																				.aspects(TC.METALLUM, 2, TC.MACHINA, 2).setOriginalMod(MD.GC_GALAXYSPACE),
	Adamantite				= metalmachore	( 8755,	"Adamantite"			, SET_METALLIC			, 255, 255, 190, 255).setQuality( 6.0F,   2560,  3)																																																																				.aspects(TC.METALLUM, 2, TC.TUTAMEN, 2).setOriginalMod(MD.GC_GALAXYSPACE),
	Duralumin				= alloymachine	( 8756,	"Duralumin"				, SET_DULL				, 255, 255, 220, 255).add(UUM).setQuality(10.0F, 512, 2)																											.setMcfg( 0, Al				, 1*U, Cu				, 1*U)																									.aspects(TC.METALLUM, 2, TC.PERMUTATIO, 1, TC.VOLATUS, 1).setOriginalMod(MD.GC_GALAXYSPACE),
	
	Draconium				= metalmachore	( 8791,	"Draconium"				, SET_METALLIC			, 150,  50, 250, 255)																																																																											.aspects(TC.METALLUM, 2, TC.ALIENIS, 1).setOriginalMod(MD.DE).visDefault(),
	DraconiumAwakened		= alloymachine	( 8792,	"DraconiumAwakened"		, SET_METALLIC			, 250, 150,  50, 255)																																				.setMcfg( 1, Draconium		, 1*U)																															.aspects(TC.METALLUM, 2, TC.ALIENIS, 4).setOriginalMod(MD.DE).visDefault().setLocal("Awakened Draconium"),
	
	Trinaquadalloy			= alloymachine	( 8684,	"Trinaquadalloy"		, SET_METALLIC			, 146, 186, 146, 255).add(CENTRIFUGE, UUM).setQuality(14.0F,  20480,  5)																							.setMcfg( 0, Ke				, 6*U, Nq				, 2*U, C				, 1*U)																			.aspects(TC.METALLUM, 3, TC.TUTAMEN, 3, TC.NEBRISUM, 1).setOriginalMod(MD.GT).addIdenticalNames("NaquadahAlloy"),
	Trinitanium				= alloymachine	( 8790,	"Trinitanium"			, SET_METALLIC			, 235, 175, 255, 255).add(UUM).setQuality(16.0F,   5120,  4)																										.setMcfg( 0, Ke				, 2*U, Ti				, 1*U)																									.aspects(TC.METALLUM, 2, TC.TUTAMEN, 2, TC.VITREUS, 2).setOriginalMod(MD.GT).alloyCentrifuge(),
	
	Iritanium				= alloymachine	( 8793,	"Titanium Iridium"		, SET_METALLIC			, 220, 220, 255, 255).add(UUM).setQuality( 8.0F,   7680,  4)																										.setMcfg( 0, Ir				, 1*U, Ti				, 1*U)																									.aspects(TC.METALLUM, 2, TC.TUTAMEN, 1, TC.MACHINA, 1).setOriginalMod(MD.GC_ADV_ROCKETRY).alloyCentrifuge().setLocal("Iritanium").addIdenticalNames("Iritanium"),
	TitaniumAluminide		= alloymachine	( 8794,	"Titanium Aluminide"	, SET_METALLIC			, 200, 200, 255, 255).add(UUM).setQuality(12.0F,   2560,  3)																										.setMcfg( 3, Ti				, 3*U, Al				, 7*U)																									.aspects(TC.METALLUM, 2, TC.TUTAMEN, 1, TC.VOLATUS, 1).setOriginalMod(MD.GC_ADV_ROCKETRY).alloyCentrifuge();
	
	/** Technical Materials, which are only there for Recipes and such. */
	public static class TECH {
		public static final OreDictMaterial
		Brick						= invalid("Brick"						).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT).add(STONE, BRITTLE, MORTAR).setMeltingPoint(2000),
		Organic						= invalid("Organic"						).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT),
		Crystal						= invalid("Crystal"						).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT).add(BRITTLE, CRYSTAL),
		Quartz						= invalid("Quartz"						).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT).add(BRITTLE, CRYSTAL, QUARTZ),
		Unknown						= invalid("Unknown"						).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT),
		Cobblestone					= invalid("Cobblestone"					).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT).add(UNRECYCLABLE),
		
		Voltage8					= invalid("Voltage8"					).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT),
		Voltage32					= invalid("Voltage32"					).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT),
		Voltage128					= invalid("Voltage128"					).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT),
		Voltage512					= invalid("Voltage512"					).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT),
		Voltage2048					= invalid("Voltage2048"					).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT),
		Voltage8192					= invalid("Voltage8192"					).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT),
		Voltage32768				= invalid("Voltage32768"				).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT),
		Voltage131072				= invalid("Voltage131072"				).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT),
		Voltage524288				= invalid("Voltage524288"				).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT),
		Voltage2147483647			= invalid("Voltage2147483647"			).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT),
		
		DiamondIndustrial			= invalid("DiamondIndustrial"			).stealLooks(Diamond		).steal(Diamond			).setLocal("Industrial Diamond"		).setAllToTheOutputOf(Diamond	).add(IGNORE_IN_COLOR_LOG, AUTO_BLACKLIST, CRYSTAL, BRITTLE, DECOMPOSABLE).addReRegistrations(Diamond),
		RefinedIron					= invalid("RefinedIron"					).stealLooks(HSLA			).steal(WroughtIron		).setLocal("Refined Iron"			).setAllToTheOutputOf(Fe		).add(IGNORE_IN_COLOR_LOG, SMITHABLE, MELTING			).addReRegistrationToThis(WroughtIron),
		
		AnyGlowstone				= invalid("Any Glowstone"				).stealLooks(Glowstone		).steal(Glowstone		).setLocal("Glowstone"				).setAllToTheOutputOf(Glowstone	).add(IGNORE_IN_COLOR_LOG								).addReRegistrationToThis(Glowstone, GlowstoneCeres, GlowstoneIo, GlowstoneEnceladus, GlowstoneProteus, GlowstonePluto),
		AnyWax						= invalid("Any Wax"						).stealLooks(Wax			).steal(Wax				).setLocal("Wax"					).setAllToTheOutputOf(Wax		).add(IGNORE_IN_COLOR_LOG								).addReRegistrationToThis(Wax, WaxBee, WaxRefractory, WaxParaffin, WaxPlant, WaxMagic),
		AnyWood						= invalid("Any Wood"					).stealLooks(Wood			).steal(Wood			).setLocal("Wood"					).setAllToTheOutputOf(Wood		).add(IGNORE_IN_COLOR_LOG, WOOD, FLAMMABLE				).addReRegistrationToThis(Wood, WoodSealed, WoodPolished, Peanutwood).setFurnaceBurnTime(TICKS_PER_SMELT/ 2),
		AnyStone					= invalid("Any Stone"					).stealLooks(Stone			).steal(Stone			).setLocal("Stone"					).setAllToTheOutputOf(Stone		).add(IGNORE_IN_COLOR_LOG, STONE, BRITTLE, UNRECYCLABLE	).addReRegistrationToThis(Stone, Concrete),
		AnyIron						= invalid("Any Iron"					).stealLooks(Fe				).steal(Fe				).setLocal("Iron"					).setAllToTheOutputOf(Fe		).add(IGNORE_IN_COLOR_LOG, SMITHABLE, MELTING			).addReRegistrationToThis(Fe, WroughtIron, PigIron, MeteoricIron),
		AnyIronSteel				= invalid("Any Iron-Steel"				).stealLooks(Steel			).steal(Steel			).setLocal("Steel"					).setAllToTheOutputOf(Steel		).add(IGNORE_IN_COLOR_LOG, SMITHABLE, MELTING			).addReRegistrationToThis(Steel, HSLA, Knightmetal, MeteoricSteel),
		AnyCopper					= invalid("Any Copper"					).stealLooks(Cu				).steal(Cu				).setLocal("Copper"					).setAllToTheOutputOf(Cu		).add(IGNORE_IN_COLOR_LOG, SMITHABLE, MELTING			).addReRegistrationToThis(Cu, AnnealedCopper),
		AnySilicon					= invalid("Any Silicon"					).stealLooks(Si				).steal(Si				).setLocal("Silicon"				).setAllToTheOutputOf(Si		).add(IGNORE_IN_COLOR_LOG, SMITHABLE, MELTING			).addReRegistrationToThis(Si),
		AnyTungsten					= invalid("Any Tungsten"				).stealLooks(W				).steal(W				).setLocal("Tungsten"				).setAllToTheOutputOf(W			).add(IGNORE_IN_COLOR_LOG, SMITHABLE, MELTING			).addReRegistrationToThis(W, TungstenSintered),
		AnyThaumicCrystal			= invalid("Any Thaumic Crystal"			).stealLooks(InfusedBalance	).steal(InfusedDull		).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT).addReRegistrationToThis(InfusedDull, InfusedVis, InfusedOrder, InfusedEntropy, InfusedAir, InfusedWater, InfusedEarth, InfusedFire, InfusedBalance),
		AnySalt						= invalid("Any Salt"					).stealLooks(NaCl			).steal(NaCl			).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT).addReRegistrationToThis(NaCl, KCl, ICl),
		
		AnySteel					= invalid("Any Steel"					).stealLooks(Steel			).steal(Steel			).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT, SMITHABLE, MELTING, EXTRUDER).addReRegistrationToThis(/*AnyIronSteel, SteelGalvanized, ShadowSteel, Steeleaf, Knightmetal, FierySteel, BlackSteel, RedSteel, BlueSteel, DamascusSteel, TungstenSteel, TungstenCarbide, StainlessSteel*/),
		AnyBronze					= invalid("Any Bronze"					).stealLooks(Bronze			).steal(Bronze			).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT, SMITHABLE, MELTING, EXTRUDER).addReRegistrationToThis(/*Bronze*/),
		AnyMetal					= invalid("Any Metal"					).stealLooks(Fe				).steal(Fe				).add(IGNORE_IN_COLOR_LOG, DONT_SHOW_THIS_COMPONENT, SMITHABLE, MELTING, EXTRUDER).addReRegistrationToThis(/*AnyBronze, AnyCopper, AnyIron, AnySteel*/);
		
		static {
			Ad						.visDefault(Adamantine);
			
			Amber					.setOreMultiplier( 2);
			InfusedAir				.setOreMultiplier( 2);
			InfusedFire				.setOreMultiplier( 2);
			InfusedEarth			.setOreMultiplier( 2);
			InfusedWater			.setOreMultiplier( 2);
			InfusedEntropy			.setOreMultiplier( 2);
			InfusedOrder			.setOreMultiplier( 2);
			InfusedVis				.setOreMultiplier( 2);
			InfusedDull				.setOreMultiplier( 2);
			InfusedBalance			.setOreMultiplier( 2);
			NaCl					.setOreMultiplier( 2);
			KCl						.setOreMultiplier( 2);
			ICl						.setOreMultiplier( 2);
			Draconium				.setOreMultiplier( 2);
			OREMATS.Cassiterite		.setOreMultiplier( 2);
			OREMATS.CassiteriteSand	.setOreMultiplier( 2);
			NetherQuartz			.setOreMultiplier( 2);
			CertusQuartz			.setOreMultiplier( 2);
			ChargedCertusQuartz		.setOreMultiplier( 2);
			Monazite				.setOreMultiplier( 2);
			Phosphorus				.setOreMultiplier( 3);
			KNO3					.setOreMultiplier( 3);
			Apatite					.setOreMultiplier( 4);
			Electrotine				.setOreMultiplier( 4);
			Nikolite				.setOreMultiplier( 4);
			Teslatite				.setOreMultiplier( 4);
			Redstone				.setOreMultiplier( 4);
			Glowstone				.setOreMultiplier( 4);
			GlowstoneCeres			.setOreMultiplier( 4);
			GlowstoneIo				.setOreMultiplier( 4);
			GlowstoneEnceladus		.setOreMultiplier( 4);
			GlowstoneProteus		.setOreMultiplier( 4);
			GlowstonePluto			.setOreMultiplier( 4);
			Lapis					.setOreMultiplier( 5);
			Sodalite				.setOreMultiplier( 5);
			Lazurite				.setOreMultiplier( 5);
			
			Plastic					.addEnchantmentForTools(Enchantment.knockback, 1);
			Rubber					.addEnchantmentForTools(Enchantment.knockback, 2);
			Kalendrite				.addEnchantmentForTools(Enchantment.knockback, 2);
			InfusedAir				.addEnchantmentForTools(Enchantment.knockback, 2);
			DarkIron				.addEnchantmentForTools(Enchantment.knockback, 3);
			Tartarite				.addEnchantmentForTools(Enchantment.knockback, 3);
			
			IronWood				.addEnchantmentForTools(Enchantment.fortune, 1);
			Steeleaf				.addEnchantmentForTools(Enchantment.fortune, 2);
			Soularium				.addEnchantmentForTools(Enchantment.fortune, 2);
			Midasium				.addEnchantmentForTools(Enchantment.fortune, 2);
			Mithril					.addEnchantmentForTools(Enchantment.fortune, 3);
			Vinteum					.addEnchantmentForTools(Enchantment.fortune, 1);
			Thaumium				.addEnchantmentForTools(Enchantment.fortune, 2);
			DarkThaumium			.addEnchantmentForTools(Enchantment.fortune, 3);
			Magic					.addEnchantmentForTools(Enchantment.fortune, 3);
			InfusedWater			.addEnchantmentForTools(Enchantment.fortune, 3);
			Eximite					.addEnchantmentForTools(Enchantment.fortune, 3);
			Haderoth				.addEnchantmentForTools(Enchantment.fortune, 4);
			VibraniumSteel			.addEnchantmentForTools(Enchantment.fortune, 4);
			Vb						.addEnchantmentForTools(Enchantment.fortune, 5);
			
			Flint					.addEnchantmentForTools(Enchantment.fireAspect, 1);
			Netherrack				.addEnchantmentForTools(Enchantment.fireAspect, 1);
			Obsidian				.addEnchantmentForTools(Enchantment.fireAspect, 1);
			NetherBrick				.addEnchantmentForTools(Enchantment.fireAspect, 2);
			DarkSteel				.addEnchantmentForTools(Enchantment.fireAspect, 2);
			Ignatius				.addEnchantmentForTools(Enchantment.fireAspect, 2);
			Prometheum				.addEnchantmentForTools(Enchantment.fireAspect, 2);
			Firestone				.addEnchantmentForTools(Enchantment.fireAspect, 3);
			FierySteel				.addEnchantmentForTools(Enchantment.fireAspect, 3);
			Pyrotheum				.addEnchantmentForTools(Enchantment.fireAspect, 3);
			Blaze					.addEnchantmentForTools(Enchantment.fireAspect, 3);
			InfusedFire				.addEnchantmentForTools(Enchantment.fireAspect, 3);
			Vulcanite				.addEnchantmentForTools(Enchantment.fireAspect, 3);
			Amordrine				.addEnchantmentForTools(Enchantment.fireAspect, 3);
			
			Ad						.addEnchantmentForTools(Enchantment.silkTouch, 1);
			Adamantine				.addEnchantmentForTools(Enchantment.silkTouch, 1);
			Force					.addEnchantmentForTools(Enchantment.silkTouch, 1);
			Amber					.addEnchantmentForTools(Enchantment.silkTouch, 1);
			EnderPearl				.addEnchantmentForTools(Enchantment.silkTouch, 1);
			Enderium				.addEnchantmentForTools(Enchantment.silkTouch, 1);
			Inolashite				.addEnchantmentForTools(Enchantment.silkTouch, 1);
			Sanguinite				.addEnchantmentForTools(Enchantment.silkTouch, 1);
			SpectreIron				.addEnchantmentForTools(Enchantment.silkTouch, 1);
			NetherStar				.addEnchantmentForTools(Enchantment.silkTouch, 1);
			InfusedOrder			.addEnchantmentForTools(Enchantment.silkTouch, 1);
			InfusedBalance			.addEnchantmentForTools(Enchantment.silkTouch, 1);
			Vibramantium			.addEnchantmentForTools(Enchantment.silkTouch, 1);
			
			BlackBronze				.addEnchantmentForTools(Enchantment.smite, 2);
			Au						.addEnchantmentForTools(Enchantment.smite, 3);
			RoseGold				.addEnchantmentForTools(Enchantment.smite, 4);
			EnergeticAlloy			.addEnchantmentForTools(Enchantment.smite, 4);
			SpectreIron				.addEnchantmentForTools(Enchantment.smite, 5);
			VibrantAlloy			.addEnchantmentForTools(Enchantment.smite, 5);
			Carmot					.addEnchantmentForTools(Enchantment.smite, 5);
			Pt						.addEnchantmentForTools(Enchantment.smite, 5);
			InfusedVis				.addEnchantmentForTools(Enchantment.smite, 5);
			
			Pb						.addEnchantmentForTools(Enchantment.baneOfArthropods, 2);
			Ni						.addEnchantmentForTools(Enchantment.baneOfArthropods, 2);
			Invar					.addEnchantmentForTools(Enchantment.baneOfArthropods, 3);
			Sb						.addEnchantmentForTools(Enchantment.baneOfArthropods, 3);
			BatteryAlloy			.addEnchantmentForTools(Enchantment.baneOfArthropods, 4);
			Bi						.addEnchantmentForTools(Enchantment.baneOfArthropods, 4);
			Orichalcum				.addEnchantmentForTools(Enchantment.baneOfArthropods, 4);
			BismuthBronze			.addEnchantmentForTools(Enchantment.baneOfArthropods, 5);
			InfusedEarth			.addEnchantmentForTools(Enchantment.baneOfArthropods, 5);
			Celenegil				.addEnchantmentForTools(Enchantment.baneOfArthropods, 5);
			
			Fe						.addEnchantmentForTools(Enchantment.sharpness, 1);
			Ice						.addEnchantmentForTools(Enchantment.sharpness, 1);
			Bronze					.addEnchantmentForTools(Enchantment.sharpness, 1);
			PulsatingIron			.addEnchantmentForTools(Enchantment.sharpness, 1);
			ConductiveIron			.addEnchantmentForTools(Enchantment.sharpness, 1);
			RedstoneAlloy			.addEnchantmentForTools(Enchantment.sharpness, 1);
			ElectricalSteel			.addEnchantmentForTools(Enchantment.sharpness, 2);
			Brass					.addEnchantmentForTools(Enchantment.sharpness, 2);
			HSLA					.addEnchantmentForTools(Enchantment.sharpness, 2);
			Steel					.addEnchantmentForTools(Enchantment.sharpness, 2);
			WroughtIron				.addEnchantmentForTools(Enchantment.sharpness, 2);
			PigIron					.addEnchantmentForTools(Enchantment.sharpness, 2);
			DeepIron				.addEnchantmentForTools(Enchantment.sharpness, 2);
			VanadiumSteel			.addEnchantmentForTools(Enchantment.sharpness, 3);
			StainlessSteel			.addEnchantmentForTools(Enchantment.sharpness, 3);
			Knightmetal				.addEnchantmentForTools(Enchantment.sharpness, 3);
			ShadowIron				.addEnchantmentForTools(Enchantment.sharpness, 3);
			ShadowSteel				.addEnchantmentForTools(Enchantment.sharpness, 4);
			BlackSteel				.addEnchantmentForTools(Enchantment.sharpness, 4);
			RedSteel				.addEnchantmentForTools(Enchantment.sharpness, 4);
			BlueSteel				.addEnchantmentForTools(Enchantment.sharpness, 5);
			DamascusSteel			.addEnchantmentForTools(Enchantment.sharpness, 5);
			InfusedEntropy			.addEnchantmentForTools(Enchantment.sharpness, 5);
			Glass					.addEnchantmentForTools(Enchantment.sharpness, 5);
			Ke						.addEnchantmentForTools(Enchantment.sharpness, 6);
			Trinitanium				.addEnchantmentForTools(Enchantment.sharpness, 7);
			Trinaquadalloy			.addEnchantmentForTools(Enchantment.sharpness, 8);
			
			Oureclase				.addEnchantmentForArmors(Enchantment.respiration, 3);
			InfusedAir				.addEnchantmentForArmors(Enchantment.respiration, 3);
			
			Atlarus					.addEnchantmentForArmors(Enchantment.featherFalling, 4);
			InfusedFire				.addEnchantmentForArmors(Enchantment.featherFalling, 4);
			
			Steeleaf				.addEnchantmentForArmors(Enchantment.protection, 2);
			Knightmetal				.addEnchantmentForArmors(Enchantment.protection, 1);
			Celenegil				.addEnchantmentForArmors(Enchantment.protection, 4);
			InfusedEarth			.addEnchantmentForArmors(Enchantment.protection, 4);
			InfusedVis				.addEnchantmentForArmors(Enchantment.protection, 4);
			InfusedBalance			.addEnchantmentForArmors(Enchantment.protection, 4);
			Tartarite				.addEnchantmentForArmors(Enchantment.protection, 5);
			Adamantine				.addEnchantmentForArmors(Enchantment.protection, 5);
			VibraniumSilver			.addEnchantmentForArmors(Enchantment.protection, 6);
			VibraniumSteel			.addEnchantmentForArmors(Enchantment.protection, 8);
			Vibramantium			.addEnchantmentForArmors(Enchantment.protection,10);
			Vb						.addEnchantmentForArmors(Enchantment.protection,10);
			Ad						.addEnchantmentForArmors(Enchantment.protection,10);
			
			Vyroxeres				.addEnchantmentForArmors(Enchantment.thorns, 3);
			InfusedEntropy			.addEnchantmentForArmors(Enchantment.thorns, 3);
			
			InfusedWater			.addEnchantmentForArmors(Enchantment.aquaAffinity, 1);
			IronWood				.addEnchantmentForArmors(Enchantment.aquaAffinity, 1);
			Ceruclase				.addEnchantmentForArmors(Enchantment.aquaAffinity, 1);
			Inolashite				.addEnchantmentForArmors(Enchantment.aquaAffinity, 1);
			
			InfusedOrder			.addEnchantmentForArmors(Enchantment.projectileProtection, 4);
			
			Obsidian				.addEnchantmentForArmors(Enchantment.blastProtection, 3);
			InfusedDull				.addEnchantmentForArmors(Enchantment.blastProtection, 4);
			Amordrine				.addEnchantmentForArmors(Enchantment.blastProtection, 5);
			
			/*
			Stone					.setSpecialEffect(SpecialToolEffect.Crushing, 1);
			GraniteRed				.setSpecialEffect(SpecialToolEffect.Crushing, 2);
			GraniteBlack			.setSpecialEffect(SpecialToolEffect.Crushing, 2);
			Adamantium				.setSpecialEffect(SpecialToolEffect.Crushing, 3);
			
			DeepIron				.setSpecialEffect(SpecialToolEffect.Magnetic, 1);
			MeteoricIron			.setSpecialEffect(SpecialToolEffect.Magnetic, 2);
			MeteoricSteel			.setSpecialEffect(SpecialToolEffect.Magnetic, 3);
			IronMagnetic			.setSpecialEffect(SpecialToolEffect.Magnetic, 1);
			SteelMagnetic			.setSpecialEffect(SpecialToolEffect.Magnetic, 2);
			NeodymiumMagnetic		.setSpecialEffect(SpecialToolEffect.Magnetic, 3);
			
			FryingOilHot			.setHeatDamage(1.0F);
			Lava					.setHeatDamage(3.0F);
			Firestone				.setHeatDamage(5.0F);
			Pyrotheum				.setHeatDamage(5.0F);
			*/
			
			OREMATS.Pitchblende				.addOreByProducts(Pb					, Th						, U_238					, Ra					, He					, RareEarth				);
			OREMATS.Uraninite				.addOreByProducts(Pb					, Th						, U_238					, Ra					, He					, RareEarth				);
			U_238							.addOreByProducts(Pb					, Th						, Pu					, Am					);
			Am								.addOreByProducts(Pb					, U_238						, Pu					);
			Pu								.addOreByProducts(Pb					, U_238						, Am					);
			Th								.addOreByProducts(Pb					, U_238						);
			
			OREMATS.Chalcopyrite			.addOreByProducts(Pyrite				, Co						, Cd					, Au					, OREMATS.Sperrylite	, OREMATS.Stannite		);
			OREMATS.Fluorite				.addOreByProducts(OREMATS.Huebnerite	, Y							, Ce					, Fe					, Na					, Ba					);
			OREMATS.Sperrylite				.addOreByProducts(Sb					, Cu						, Fe					, Rh					, Pt					);
			OREMATS.Pentlandite				.addOreByProducts(Fe					, S							, Co					, OREMATS.Sperrylite	);
			OREMATS.Sphalerite				.addOreByProducts(Cd					, Ga						, Zn					, OREMATS.Kesterite		);
			OREMATS.Tetrahedrite			.addOreByProducts(Cu					, Sb						, Zn					, OREMATS.Kesterite		);
			Pyrite							.addOreByProducts(S						, Phosphorus				, Fe					, OREMATS.Stannite		);
			Sn								.addOreByProducts(OREMATS.Molybdenite	, OREMATS.Wolframite		, OREMATS.Fluorite		, OREMATS.Arsenopyrite	, OREMATS.Stannite		, OREMATS.Sperrylite	, OREMATS.Huebnerite	, Apatite); // Tourmaline
			OREMATS.Cassiterite				.addOreByProducts(OREMATS.Molybdenite	, OREMATS.Wolframite		, OREMATS.Fluorite		, OREMATS.Arsenopyrite	, OREMATS.Stannite		, OREMATS.Sperrylite	, OREMATS.Huebnerite	, Apatite); // Tourmaline
			OREMATS.CassiteriteSand			.addOreByProducts(OREMATS.Molybdenite	, OREMATS.Wolframite		, OREMATS.Fluorite		, OREMATS.Arsenopyrite	, OREMATS.Stannite		, OREMATS.Sperrylite	, OREMATS.Huebnerite	, Apatite); // Tourmaline
			OREMATS.Huebnerite				.addOreByProducts(OREMATS.Molybdenite	, OREMATS.Wolframite		, OREMATS.Fluorite		, OREMATS.Arsenopyrite	, OREMATS.Cassiterite	, Topaz					); // Tourmaline, Rhodochrosite
			OREMATS.Arsenopyrite			.addOreByProducts(Au					, OREMATS.Fluorite			, OREMATS.Cassiterite	, OREMATS.Huebnerite	);
			MeteoricIron					.addOreByProducts(Fe					, Ni						, Ir					, Pt					);
			OREMATS.Bauxite					.addOreByProducts(OREMATS.Kaolinite		, OREMATS.Ilmenite			, OREMATS.Hematite		, Al					);
			OREMATS.Ilmenite				.addOreByProducts(OREMATS.Rutile		, OREMATS.Hematite			, Mg					, Mn					);
			OREMATS.Rutile					.addOreByProducts(Fe					, Nb						, Ta					, Zr					);
			Ti								.addOreByProducts(Fe					, Nb						, Ta					, Zr					);
			OREMATS.Hematite				.addOreByProducts(OREMATS.Ilmenite		, OREMATS.Magnetite			, Fe					, Clay					);
			OREMATS.Zeolite					.addOreByProducts(Na					, K							, Ca					, Mg					);
			OREMATS.YellowLimonite			.addOreByProducts(Ni					, OREMATS.BrownLimonite		, Co					);
			Cu								.addOreByProducts(Co					, Au						, Ni					);
			Ni								.addOreByProducts(Co					, Pt						, Fe					);
			OREMATS.Stannite				.addOreByProducts(Ge					, Pyrite					, OREMATS.Kesterite		);
			OREMATS.Kesterite				.addOreByProducts(Ge					, Pyrite					, OREMATS.Stannite		);
			OREMATS.GlauconiteSand			.addOreByProducts(Na					, Al						, Fe					);
			OREMATS.Glauconite				.addOreByProducts(Na					, Al						, Fe					);
			OREMATS.Vermiculite				.addOreByProducts(Fe					, Al						, Mg					);
			OREMATS.FullersEarth			.addOreByProducts(Al					, SiO2						, Mg					);
			OREMATS.Bentonite				.addOreByProducts(Al					, Ca						, Mg					);
			OREMATS.Galena					.addOreByProducts(S						, Ag						, Pb					);
			Lapis							.addOreByProducts(Lazurite				, Sodalite					, Pyrite				);
			OREMATS.Cooperite				.addOreByProducts(Pd					, Ni						, Ir					);
			OREMATS.Cinnabar				.addOreByProducts(Redstone				, S							, Glowstone				);
			OREMATS.Tantalite				.addOreByProducts(Mn					, Nb						, Ta					);
			OREMATS.Pollucite				.addOreByProducts(Cs					, Al						, Rb					);
			OREMATS.Chrysotile				.addOreByProducts(Asbestos				, SiO2						, Mg					);
			Asbestos						.addOreByProducts(Asbestos				, SiO2						, Mg					);
			Phosphorus						.addOreByProducts(Phosphorite			, Phosphate					, Apatite				);
			Apatite							.addOreByProducts(Phosphorite			, Phosphate					, Phosphorus			);
			Sapphire						.addOreByProducts(OrangeSapphire		, YellowSapphire			, GreenSapphire			);
			BlueSapphire					.addOreByProducts(OrangeSapphire		, YellowSapphire			, GreenSapphire			);
			GreenSapphire					.addOreByProducts(BlueSapphire			, OrangeSapphire			, YellowSapphire		);
			YellowSapphire					.addOreByProducts(BlueSapphire			, OrangeSapphire			, GreenSapphire			);
			OrangeSapphire					.addOreByProducts(BlueSapphire			, YellowSapphire			, GreenSapphire			);
			OREMATS.Kaolinite				.addOreByProducts(OREMATS.Bauxite		, Clay						, Al					);
			OREMATS.QuartzSand				.addOreByProducts(CertusQuartz			, Quartzite					, OREMATS.Barite		);
			Quartzite						.addOreByProducts(CertusQuartz			, OREMATS.Barite			, ChargedCertusQuartz	);
			CertusQuartz					.addOreByProducts(Quartzite				, ChargedCertusQuartz		, OREMATS.Barite		);
			ChargedCertusQuartz				.addOreByProducts(CertusQuartz			, Quartzite					, OREMATS.Barite		);
			Redstone						.addOreByProducts(OREMATS.Cinnabar		, RareEarth					, Glowstone				);
			Os								.addOreByProducts(Ir					, Pt						, Ru					);
			Ir								.addOreByProducts(Pt					, Os						, Rh					);
			Pt								.addOreByProducts(Ni					, Ir						, Pd					);
			Au								.addOreByProducts(Cu					, Ni						, OREMATS.Cinnabar		);
			Ag								.addOreByProducts(Pb					, S							, OREMATS.Bromargyrite	);
			Ke								.addOreByProducts(Sn					, OREMATS.Rutile			, Fe					);
			Monazite						.addOreByProducts(Th					, Nd						, RareEarth				);
			Forcicium						.addOreByProducts(Th					, Nd						, RareEarth				);
			Forcillium						.addOreByProducts(Th					, Nd						, RareEarth				);
			FakeOsmium						.addOreByProducts(Fe					, Sn						, Cr					);
			OREMATS.Molybdenite				.addOreByProducts(Mo					, Re						, Os					);
			OREMATS.Malachite				.addOreByProducts(Cu					, OREMATS.BrownLimonite		, CaCO3					);
			OREMATS.Scheelite				.addOreByProducts(Mn					, Mo						, Ca					);
			OREMATS.Tungstate				.addOreByProducts(Mn					, Ag						, Li					);
			OREMATS.Wolframite				.addOreByProducts(W						, Fe						, OREMATS.Stannite		);
			OREMATS.Stolzite				.addOreByProducts(W						, Pb						);
			OREMATS.Russellite				.addOreByProducts(W						, Bi						);
			OREMATS.Pinalite				.addOreByProducts(W						, Pb						);
			OREMATS.BrownLimonite			.addOreByProducts(OREMATS.Malachite		, OREMATS.YellowLimonite	);
			OREMATS.Garnierite				.addOreByProducts(Ni					, OREMATS.Sperrylite		);
			Nd								.addOreByProducts(Monazite				, RareEarth					);
			OREMATS.Bastnasite				.addOreByProducts(Nd					, RareEarth					);
			NaCl							.addOreByProducts(KCl					, ICl						);
			KCl								.addOreByProducts(ICl					, NaCl						);
			ICl								.addOreByProducts(NaCl					, KCl						);
			Glowstone						.addOreByProducts(Redstone				, Au						);
			GlowstoneCeres					.addOreByProducts(Redstone				, Au						, Glowstone				);
			GlowstoneIo						.addOreByProducts(Redstone				, Au						, Glowstone				);
			GlowstoneEnceladus				.addOreByProducts(Redstone				, Au						, Glowstone				);
			GlowstoneProteus				.addOreByProducts(Redstone				, Au						, Glowstone				);
			GlowstonePluto					.addOreByProducts(Redstone				, Au						, Glowstone				);
			Zn								.addOreByProducts(Sn					, Ga						);
			W								.addOreByProducts(Mn					, Mo						);
			OREMATS.Diatomite				.addOreByProducts(OREMATS.Hematite		, BlueSapphire				);
			Jasper							.addOreByProducts(BalasRuby				, Spinel					);
			BalasRuby						.addOreByProducts(Jasper				, Spinel					);
			Spinel							.addOreByProducts(Jasper				, BalasRuby					);
			Fe								.addOreByProducts(Ni					, Sn						);
			OREMATS.Lepidolite				.addOreByProducts(Li					, Cs						);
			Sb								.addOreByProducts(Zn					, Fe						);
			OREMATS.Smithsonite				.addOreByProducts(Zn					, OREMATS.Bromargyrite		);
			OREMATS.Pyrolusite				.addOreByProducts(Mn					, OREMATS.Bromargyrite		);
			Pb								.addOreByProducts(Ag					, S							);
			Electrum						.addOreByProducts(Au					, Ag						);
			Bronze							.addOreByProducts(Cu					, Sn						);
			Brass							.addOreByProducts(Cu					, Zn						);
			Coal							.addOreByProducts(Lignite				, Th						);
			Lignite							.addOreByProducts(Coal					, Ge						);
			Mn								.addOreByProducts(Cr					, Fe						);
			Emerald							.addOreByProducts(Be					, Al						);
			Cr								.addOreByProducts(Fe					, Mg						);
			OREMATS.Chromite				.addOreByProducts(Fe					, Mg						);
			OREMATS.Ferrovanadium			.addOreByProducts(OREMATS.Magnetite		, VanadiumPentoxide			);
			OREMATS.Magnetite				.addOreByProducts(Fe					, Au						);
			OREMATS.GraniticMineralSand		.addOreByProducts(GraniteBlack			, OREMATS.Magnetite			);
			OREMATS.BasalticMineralSand		.addOreByProducts(Basalt				, OREMATS.Magnetite			);
			Basalt							.addOreByProducts(Olivine				, DarkAsh					);
			OREMATS.Celestine				.addOreByProducts(Sr					, S							);
			Lazurite						.addOreByProducts(Sodalite				, Lapis						);
			Sodalite						.addOreByProducts(Lazurite				, Lapis						);
			OREMATS.Spodumene				.addOreByProducts(Al					, Li						);
			Co_60							.addOreByProducts(OREMATS.Cobaltite		, Co						);
			Zr								.addOreByProducts(OREMATS.Rutile		, Hf						);
			OREMATS.DiduraniumTrioxide		.addOreByProducts(Dn					, Tn						);
			OREMATS.DuraniumHexafluoride	.addOreByProducts(Dn					);
			OREMATS.DuraniumHexachloride	.addOreByProducts(Dn					);
			OREMATS.DuraniumHexabromide		.addOreByProducts(Dn					);
			OREMATS.DuraniumHexaiodide		.addOreByProducts(Dn					, I							);
			OREMATS.DuraniumHexaastatide	.addOreByProducts(Dn					, At						);
			OREMATS.TritaniumDioxide		.addOreByProducts(Tn					, Dn						);
			OREMATS.TritaniumHexafluoride	.addOreByProducts(Tn					);
			OREMATS.TritaniumHexachloride	.addOreByProducts(Tn					);
			OREMATS.TritaniumHexabromide	.addOreByProducts(Tn					);
			OREMATS.TritaniumHexaiodide		.addOreByProducts(Tn					, I							);
			OREMATS.TritaniumHexaastatide	.addOreByProducts(Tn					, At						);
			Nq_528							.addOreByProducts(Nq					, Nq_522					);
			Nq_522							.addOreByProducts(Nq					, Nq_528					);
			Nq								.addOreByProducts(Nq_528				);
			Ad								.addOreByProducts(OREMATS.Magnetite		, Adamantine				);
			Adamantine						.addOreByProducts(OREMATS.Magnetite		, Ad						);
			OREMATS.Stibnite				.addOreByProducts(Sb					);
			Diamond							.addOreByProducts(Graphite				);
			Pyrope							.addOreByProducts(Mg					);
			Almandine						.addOreByProducts(Al					);
			Spessartine						.addOreByProducts(Mn					);
			Andradite						.addOreByProducts(Fe					);
			Grossular						.addOreByProducts(Ca					);
			Uvarovite						.addOreByProducts(Cr					);
			Ruby							.addOreByProducts(Cr					);
			Olivine							.addOreByProducts(Mg					);
			Be								.addOreByProducts(Emerald				);
			Nikolite						.addOreByProducts(Cu					);
			Teslatite						.addOreByProducts(Cu					);
			Electrotine						.addOreByProducts(Cu					);
			OREMATS.Magnesite				.addOreByProducts(Mg					);
			NetherQuartz					.addOreByProducts(Netherrack			);
			PigIron							.addOreByProducts(Fe					);
			DarkIron						.addOreByProducts(Fe					);
			Steel							.addOreByProducts(Fe					);
			Graphite						.addOreByProducts(C						);
			CaCO3							.addOreByProducts(OREMATS.Malachite		);
			Netherrack						.addOreByProducts(S						);
			Flint							.addOreByProducts(Obsidian				);
			KNO3							.addOreByProducts(Niter					);
			Niter							.addOreByProducts(KNO3					);
			Endstone						.addOreByProducts(He_3					);
			Hf								.addOreByProducts(Zr					);
			Mg								.addOreByProducts(Olivine				);
			Al								.addOreByProducts(OREMATS.Bauxite		);
			OREMATS.Cobaltite				.addOreByProducts(Co					);
			Co								.addOreByProducts(OREMATS.Cobaltite		);
			Ardite							.addOreByProducts(Co					);
			Obsidian						.addOreByProducts(Olivine				);
			Ash								.addOreByProducts(C						);
			DarkAsh							.addOreByProducts(C						);
			Redrock							.addOreByProducts(Clay					);
			Limestone						.addOreByProducts(CaCO3					);
			Marble							.addOreByProducts(CaCO3					);
			Clay							.addOreByProducts(Clay					);
			GraniteBlack					.addOreByProducts(Biotite				);
			GraniteRed						.addOreByProducts(PotassiumFeldspar		);
			Eclogite						.addOreByProducts(OREMATS.Rutile		);
			Limestone						.addOreByProducts(Phosphorite			);
			Phosphate						.addOreByProducts(P						);
			P								.addOreByProducts(Phosphate				);
			Tanzanite						.addOreByProducts(Opal					);
			Opal							.addOreByProducts(Tanzanite				);
			Topaz							.addOreByProducts(BlueTopaz				);
			BlueTopaz						.addOreByProducts(Topaz					);
			Si								.addOreByProducts(SiO2					);
			
			Dn								.addOreByProducts(Tn					);
			Tn								.addOreByProducts(Dn					);
			Dolamide						.addOreByProducts(OREMATS.DiduraniumTrioxide, OREMATS.DuraniumHexafluoride, OREMATS.DuraniumHexaastatide, OREMATS.DuraniumHexabromide);
			
			Alduorite						.addOreByProducts(Cd					);
			Infuscolium						.addOreByProducts(Cu					);
			Rubracium						.addOreByProducts(Cr					);
			Meutoite						.addOreByProducts(VanadiumPentoxide		);
			Lemurite						.addOreByProducts(Mg					);
			Ceruclase						.addOreByProducts(Sb					);
			Atlarus							.addOreByProducts(OREMATS.Rutile		);
			Oureclase						.addOreByProducts(Ni					);
			Kalendrite						.addOreByProducts(Os					);
			Orichalcum						.addOreByProducts(Cs					);
			Carmot							.addOreByProducts(Zn					);
			Sanguinite						.addOreByProducts(Hg					);
			Vyroxeres						.addOreByProducts(Ir					);
			Eximite							.addOreByProducts(Pd					);
			Prometheum						.addOreByProducts(Co					);
			Ignatius						.addOreByProducts(In					);
			Vulcanite						.addOreByProducts(W						);
			DeepIron						.addOreByProducts(Fe					);
			ShadowIron						.addOreByProducts(Fe					);
			AstralSilver					.addOreByProducts(Ag					);
			Midasium						.addOreByProducts(Au					);
			Mithril							.addOreByProducts(Pt					);
			
			Fe						.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(OREMATS.YellowLimonite		, 4*U), OM.stack(DarkAsh			, 1*U)));
			Fe						.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(OREMATS.BrownLimonite			, 4*U), OM.stack(DarkAsh			, 1*U)));
// TODO		Fe						.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(OREMATS.Chalcopyrite			, 4*U), OM.stack(DarkAsh			, 1*U)));
			Fe						.addAlloyingRecipe(new OreDictConfigurationComponent( 2, OM.stack(OREMATS.Hematite				, 5*U), OM.stack(DarkAsh			, 2*U), OM.stack(CaCO3, 2*U)));
			Fe						.addAlloyingRecipe(new OreDictConfigurationComponent( 3, OM.stack(OREMATS.Magnetite				, 7*U), OM.stack(DarkAsh			, 3*U)));
			Fe						.addAlloyingRecipe(new OreDictConfigurationComponent( 3, OM.stack(OREMATS.Ferrovanadium			,14*U), OM.stack(DarkAsh			, 3*U)));
			Fe						.addAlloyingRecipe(new OreDictConfigurationComponent( 3, OM.stack(OREMATS.BasalticMineralSand	,14*U), OM.stack(DarkAsh			, 3*U)));
			Fe						.addAlloyingRecipe(new OreDictConfigurationComponent( 3, OM.stack(OREMATS.GraniticMineralSand	,14*U), OM.stack(DarkAsh			, 3*U)));
			
			Fe						.addAlloyingRecipe(new OreDictConfigurationComponent( 2, OM.stack(OREMATS.YellowLimonite		, 8*U), OM.stack(C					, 1*U)));
			Fe						.addAlloyingRecipe(new OreDictConfigurationComponent( 2, OM.stack(OREMATS.BrownLimonite			, 8*U), OM.stack(C					, 1*U)));
// TODO		Fe						.addAlloyingRecipe(new OreDictConfigurationComponent( 2, OM.stack(OREMATS.Chalcopyrite			, 8*U), OM.stack(C					, 1*U)));
			Fe						.addAlloyingRecipe(new OreDictConfigurationComponent( 4, OM.stack(OREMATS.Hematite				,10*U), OM.stack(C					, 2*U), OM.stack(CaCO3, 2*U)));
			Fe						.addAlloyingRecipe(new OreDictConfigurationComponent( 6, OM.stack(OREMATS.Magnetite				,14*U), OM.stack(C					, 3*U)));
			Fe						.addAlloyingRecipe(new OreDictConfigurationComponent( 6, OM.stack(OREMATS.Ferrovanadium			,28*U), OM.stack(C					, 3*U)));
			Fe						.addAlloyingRecipe(new OreDictConfigurationComponent( 6, OM.stack(OREMATS.BasalticMineralSand	,28*U), OM.stack(C					, 3*U)));
			Fe						.addAlloyingRecipe(new OreDictConfigurationComponent( 6, OM.stack(OREMATS.GraniticMineralSand	,28*U), OM.stack(C					, 3*U)));
			
			Si						.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(SiO2							, 3*U), OM.stack(DarkAsh			, 2*U)));
			Si						.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(SiO2							, 3*U), OM.stack(C					, 1*U)));
			
			Al						.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(OREMATS.Bauxite				, 2*U), OM.stack(Na					, 1*U)));
			Al						.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(OREMATS.Bauxite				, 2*U), OM.stack(K					, 1*U)));
			
			Fe						.addAlloyingRecipe(new OreDictConfigurationComponent( 2, OM.stack(ShadowIron					, 1*U), OM.stack(Ignatius			, 1*U)));
			Fe						.addAlloyingRecipe(new OreDictConfigurationComponent( 2, OM.stack(DeepIron						, 1*U), OM.stack(Prometheum			, 1*U)));
			BlackSteel				.addAlloyingRecipe(new OreDictConfigurationComponent( 2, OM.stack(DeepIron						, 1*U), OM.stack(Infuscolium		, 1*U)));
			
			
			EnergeticAlloy			.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(InductiveAlloy				, 2*U), OM.stack(GlowstoneCeres		, 1*U)));
			EnergeticAlloy			.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(InductiveAlloy				, 2*U), OM.stack(GlowstoneIo		, 1*U)));
			EnergeticAlloy			.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(InductiveAlloy				, 2*U), OM.stack(GlowstoneEnceladus	, 1*U)));
			EnergeticAlloy			.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(InductiveAlloy				, 2*U), OM.stack(GlowstoneProteus	, 1*U)));
			EnergeticAlloy			.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(InductiveAlloy				, 2*U), OM.stack(GlowstonePluto		, 1*U)));
			Lumium					.addAlloyingRecipe(new OreDictConfigurationComponent( 4, OM.stack(Sn							, 3*U), OM.stack(Ag					, 1*U), OM.stack(GlowstoneCeres, 4*U)));
			Lumium					.addAlloyingRecipe(new OreDictConfigurationComponent( 4, OM.stack(Sn							, 3*U), OM.stack(Ag					, 1*U), OM.stack(GlowstoneIo, 4*U)));
			Lumium					.addAlloyingRecipe(new OreDictConfigurationComponent( 4, OM.stack(Sn							, 3*U), OM.stack(Ag					, 1*U), OM.stack(GlowstoneEnceladus, 4*U)));
			Lumium					.addAlloyingRecipe(new OreDictConfigurationComponent( 4, OM.stack(Sn							, 3*U), OM.stack(Ag					, 1*U), OM.stack(GlowstoneProteus, 4*U)));
			Lumium					.addAlloyingRecipe(new OreDictConfigurationComponent( 4, OM.stack(Sn							, 3*U), OM.stack(Ag					, 1*U), OM.stack(GlowstonePluto, 4*U)));
			BlackSteel				.addAlloyingRecipe(new OreDictConfigurationComponent( 5, OM.stack(Ni							, 1*U), OM.stack(BlackBronze		, 1*U), OM.stack(HSLA	, 3*U)));
			RedSteel				.addAlloyingRecipe(new OreDictConfigurationComponent( 8, OM.stack(SterlingSilver				, 1*U), OM.stack(BismuthBronze		, 1*U), OM.stack(HSLA	, 2*U), OM.stack(BlackSteel	, 4*U)));
			BlueSteel				.addAlloyingRecipe(new OreDictConfigurationComponent( 8, OM.stack(RoseGold						, 1*U), OM.stack(Brass				, 1*U), OM.stack(HSLA	, 2*U), OM.stack(BlackSteel	, 4*U)));
			TungstenSteel			.addAlloyingRecipe(new OreDictConfigurationComponent( 2, OM.stack(HSLA							, 1*U), OM.stack(W					, 1*U)));
			TungstenSteel			.addAlloyingRecipe(new OreDictConfigurationComponent( 2, OM.stack(HSLA							, 1*U), OM.stack(TungstenSintered	, 1*U)));
			TungstenSteel			.addAlloyingRecipe(new OreDictConfigurationComponent( 2, OM.stack(Steel							, 1*U), OM.stack(TungstenSintered	, 1*U)));
			VanadiumSteel			.addAlloyingRecipe(new OreDictConfigurationComponent( 5, OM.stack(HSLA							, 4*U), OM.stack(V					, 1*U)));
			ElectricalSteel			.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(HSLA							, 1*U), OM.stack(Si					, 1*U)));
			DarkSteel				.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(HSLA							, 1*U), OM.stack(Lava				, 9*U)));
			DarkSteel				.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(Steel							, 1*U), OM.stack(Lava				, 9*U)));
			Alumite					.addAlloyingRecipe(new OreDictConfigurationComponent( 5, OM.stack(Al							, 5*U), OM.stack(WroughtIron		, 2*U), OM.stack(Lava,18*U)));
			SterlingSilver			.addAlloyingRecipe(new OreDictConfigurationComponent( 5, OM.stack(AnnealedCopper				, 1*U), OM.stack(Ag					, 4*U)));
			RoseGold				.addAlloyingRecipe(new OreDictConfigurationComponent( 5, OM.stack(AnnealedCopper				, 1*U), OM.stack(Au					, 4*U)));
			AluminiumBrass			.addAlloyingRecipe(new OreDictConfigurationComponent( 4, OM.stack(AnnealedCopper				, 1*U), OM.stack(Al					, 3*U)));
			Brass					.addAlloyingRecipe(new OreDictConfigurationComponent( 4, OM.stack(AnnealedCopper				, 3*U), OM.stack(Zn					, 1*U)));
			Bronze					.addAlloyingRecipe(new OreDictConfigurationComponent( 4, OM.stack(AnnealedCopper				, 3*U), OM.stack(Sn					, 1*U)));
			BlackBronze				.addAlloyingRecipe(new OreDictConfigurationComponent( 5, OM.stack(AnnealedCopper				, 3*U), OM.stack(Electrum			, 2*U)));
			RedAlloy				.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(AnnealedCopper				, 1*U), OM.stack(Redstone			, 4*U)));
			Signalum				.addAlloyingRecipe(new OreDictConfigurationComponent( 8, OM.stack(AnnealedCopper				, 1*U), OM.stack(Ag					, 2*U), OM.stack(RedAlloy, 5*U)));
			Constantan				.addAlloyingRecipe(new OreDictConfigurationComponent( 2, OM.stack(AnnealedCopper				, 1*U), OM.stack(Ni					, 1*U)));
			YttriumBariumCuprate	.addAlloyingRecipe(new OreDictConfigurationComponent( 6, OM.stack(AnnealedCopper				, 3*U), OM.stack(Ba					, 2*U), OM.stack(Y, U)));
			YttriumBariumCuprate	.addAlloyingRecipe(new OreDictConfigurationComponent( 6, OM.stack(Cu							, 3*U), OM.stack(Ba					, 2*U), OM.stack(Y, U)));
			Ferrite					.addAlloyingRecipe(new OreDictConfigurationComponent( 8, OM.stack(OREMATS.Hematite				, 5*U), OM.stack(Li2O				, 3*U)));
			
			String tMakeSignalum =
			  "The Alloy Signalum can be a little bit hard to produce as you could end up with unwanted Sterling Silver in the Process. To prevent that, insert the Copper last, and always in small amounts at a time.";
			String tMakeSteel =
			  "In order to make Steel you just need to melt Iron or Wrought Iron in a Smelting Crucible and give some Carbon (1 per 50 Iron), Graphite (1 per 25 Iron) or Diamonds (1 per 100 Iron at "+Diamond.mMeltingPoint+"K) to it.";
			String tMakeWroughtIron =
			  "Wrought Iron is created by heating up Iron until " + WroughtIron.mMeltingPoint + " Kelvin to dissolve most unwanted impurities.";
			String tMakeAnnealedCopper =
			  "Annealed Copper is created by heating up Copper until " + AnnealedCopper.mMeltingPoint + " Kelvin to dissolve most unwanted impurities.";
			String tKillWerewolf =
			  "It is also very useful in order to kill Werewolves and alike, since everyone knows how Werewolves are allergic to Silver! It also works on Armor like some kind of Thorns (without the stupid extra armor damage)";
			String tKillSlime =
			  "Somehow this Material dissolves Slimey substances and therefore causes severe damage to Slimes and similar Creatures!";
			
			
			Signalum.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  ""
			, tMakeSignalum
			};
			
			Redstone.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Redstone consists out of many things and has a lot in common with Cinnabar, even though containing much more than just Mercury and Sulfur. Redstone is usually found at places with Rare Earths."
			, tMakeSignalum
			};
			
			Ag.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Silver, the Material Endermen fear the most. Somehow this Material can interfere with the ability of Endermen to teleport, and even damages them dramatically, as if it were poisonous to them."
			, tKillWerewolf
			, tMakeSignalum
			};
			
			Cu.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Copper is a Material, which is needed in almost every electrical Device. But not only that, it is also used to make Bronze, Brass, Cupronickel and some other Alloys."
			, tKillSlime
			, tMakeAnnealedCopper
			, tMakeSignalum
			};
			
			AnnealedCopper.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Annealed Copper is just Copper cleaned from impurities. Therefore some of Coppers natural properties are better than in unpurified Copper."
			, tKillSlime
			, tMakeAnnealedCopper
			, tMakeSignalum
			};
			
			Fe.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Iron, the most needed Material in Minecraft, unless you have an Iron Titan or something. Nearly every Mod needs it in order to proceed with its Machines. It exists in many Shapes and Forms, such as"
			, "Compressed Iron, Wrought Iron, Steel, HSLA Steel, Dark Iron, Deep Iron, Meteoric Iron, Conductive Iron, Electrical Steel, Pulsating Iron, Mutated Iron, Shadow Iron, Ironwood and many many more things."
			, "It is a very useful Material as it is required to make Steel, the Stuff almost everything Technological is made of. You can also make Wrought Iron, to craft Tools slightly better than Iron Tools."
			, tMakeWroughtIron
			, tMakeSteel
			};
			
			WroughtIron.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Wrought Iron, a Material that is worth as much as Iron. It is just a step between Iron itself and Steel, and is just a cleaner Version of Iron. It is also a slightly better Tool Material than Iron."
			, tMakeWroughtIron
			, tMakeSteel
			};
			
			Steel.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Steel, the Material Standard Machinery is made of. Cheap, abundant and common, as it only requires Iron and some source of Carbon. There are many different Ways of making Steel supplied by many Mods."
			, tMakeSteel
			};
			
			C.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Carbon. Not much to say about it. It is used to make Steel. Uhhm, maybe it has some other uses explained in a later Edition of this Book."
			, tMakeSteel
			};
			
			Diamond.mDescription = DiamondIndustrial.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Diamonds, the shiny Stuff every Minecrafter looks for. Some people may even create living Horses made of Diamonds."
			, tMakeSteel
			};
			
			Graphite.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Graphite is a Source of Carbon. It can also be used to make Steel directly."
			, tMakeSteel
			};
			
			DamascusSteel.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Damascus Steel is a very rare kind of Steel. Rare meaning actually nobody knows how to produce it anymore as they are all dead. Maybe ask a Zombie or a Skeleton on how to make it?"
			, "Because of that, it can only be found in old Structures such as Dungeons. Sometimes Villagers stumble upon this rare Steel or things made of it and take it home to give it to their Blacksmith."
			, "It is a very good Tool Material and even surpasses Blue Steel in its Quality. It has Vanadium and Tungsten impurities which improve the stability of the Carbides when Forging at higher Temperatures."
			};
			
			Craponite.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Craponite is a very exquisite Material used only in the fanciest Jewelry. It looks best in combination with Peridot. That is why Teleshopping Channels often sell Peridot Craponite Earrings and alike."
			};
			
			Ir.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Iridium, a very rare Metal used in only the most advanced Technology. Its properties are very versatile and used in many advanced Devices."
			, "Iridium is a Metal that can stabilise a Tesseract, so that it can transfer much larger Amounts of Matter and Energy."
			, "As Weapon it is very useful in order to kill Shapeshifters, since it is highly Toxic to them. Now, if there were Shapeshifters in our World this Information would be more useful."
			};
			
			Tc.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Technetium, named after Gregorius Techneticies, is the first Element of the Periodic System, which has no stable Isotope. Its usage is mostly limited to Medical Applications."
			, "Fun Fact:\nEvery Element, which has no Elemental Stats assigned to, will automatically default to the Elemental Stats of Technetium."
			};
			
			Mcg.mDescription = new String[] {
			// ======================================================================================================================================================================================================== 
			  "Mc-Guffium, the most useful Material of them all. It can be used for everything, as long as its underlying Science is too obscure to be understood."
			, "Possible Applications are: Time Travel, Infinite Impossibility Drives, World Peace, World Domination, World War, World War II, World War IV, Curing all Diseases, Stopping Global Warming,"
			, "Controlling the Weather, Turning everything into Gold, Turning you into a God, Killing a God, Making Dark Matter an Energy Source, Tesseracts, Wormholes, Black Holes, Bending Space and Time, Tardis,"
			, "Ascension to a higher Level of existence, Invisibility, Killing Chuck Norris, Cloning, Making Profit of collecting Underpants, Making the Impossible possible, Love Rays, Hate Rays, Perpetuum Mobiles,"
			, "Making you able to hold and use Thors Hammer, and ofcourse baking delicious Cookies."
			};
			
			Nq.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "This extremly rare Material is called Naquadah. Its properties make it very useful in highly advanced Technology."
			, "It is a Superconductor even at very high Temperatures, what makes it seeming Ideal for electric Wiring as long as the amount of transferred Electricity is not too large."
			, "Another property of it is that it emits Gamma Radiation when supplied with enough Electricity, due to creating Positrons out of said Energy, which then collide with Electrons very quickly."
			, "However that property makes it very unstable and can result in a Nuclear Explosion when not regulated properly, especially with molten Naquadah."
			};
			
			Nq_528.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "This is a heavy Isotope of Naquadah mainly used to generate Energy, or to create Bombs."
			, "Due to being less stable than regular Naquadah, supplying this Isotope with Electricity results in a very strong Nuclear Reaction, which when regulated properly can result in a constant Energy Source."
			, "The instability increases when this Isotope is molten, resulting in a larger Energy Output when used in Reactors."
			, "Its instability is also often used to make Naquadah Bombs, which are much more destructive than the regular Plutonium Nukes. It doesn't make any difference to use the molten or the solid Isotope"
			, "for Bombs, since its explosion heats up the whole Bomb far above the melting Point anyways."
			};
			
			Nq_522.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Naquadria, a light and extremely unstable Isotope of Naquadah. It can be used for Reactors as well as Bombs and is much stronger than heavy Naquadah."
			, "It is created when Naquadah is receiving too large amounts of Gamma Radiation. During its creation it emits more Gamma Radiation turning adjacent Naquadah into Naquadria too."
			, "The Natural origin of this Isotope is a Gamma Ray Burst, which hits a Naquadah Vein directly. Planets to which that happens are then turning into a Desert due to the enormous Heat created by the Ray."
			, "Gamma Bombs which are exploding far enough away from a Naquadah Vein, or just a strong enough Gamma Ray, can also trigger a Chain Reaction turing Naquadah into Naquadria."
			, "However most of the time a Gamma Ray hits a Naquadah Vein, it just explodes, what is the main Factor of Naquadria being very rare. Also Naquadria turns back into Naquadah after a few thousand years."
			};
			
			for (OreDictMaterial tMaterial : ALL_MATERIALS_REGISTERED_HERE) {
				tMaterial.mHandleMaterial = (tMaterial.contains(BURNING)?Blaze:tMaterial.getMass()>Tc.getMass()*2?TungstenSteel:tMaterial.getMass()>Tc.getMass()?Steel:Wood);
				if (tMaterial.mReRegistrations.contains(AnyIron) || tMaterial.mReRegistrations.contains(AnyIronSteel)) {
					tMaterial.mAchievementsForCreation.add(AchievementList.openInventory);
					tMaterial.mAchievementsForCreation.add(AchievementList.mineWood);
					tMaterial.mAchievementsForCreation.add(AchievementList.buildWorkBench);
					tMaterial.mAchievementsForCreation.add(AchievementList.buildPickaxe);
					tMaterial.mAchievementsForCreation.add(AchievementList.buildFurnace);
					tMaterial.mAchievementsForCreation.add(AchievementList.acquireIron);
				}
				if (tMaterial.mOriginalMod == null && !tMaterial.contains(INVALID_MATERIAL)) tMaterial.setOriginalMod(MD.GAPI);
			}
			
			Diamond.mAchievementsForCreation.add(AchievementList.openInventory);
			Diamond.mAchievementsForCreation.add(AchievementList.mineWood);
			Diamond.mAchievementsForCreation.add(AchievementList.buildWorkBench);
			Diamond.mAchievementsForCreation.add(AchievementList.buildPickaxe);
			Diamond.mAchievementsForCreation.add(AchievementList.buildFurnace);
			Diamond.mAchievementsForCreation.add(AchievementList.acquireIron);
			Diamond.mAchievementsForCreation.add(AchievementList.diamonds);
			
			SpectreIron.mHandleMaterial = Obsidian;
			EnderAmethyst.mHandleMaterial = Steel;
			Wood.mHandleMaterial = Stone.mHandleMaterial = Fe.mHandleMaterial = BismuthBronze.mHandleMaterial = Bronze.mHandleMaterial = Brass.mHandleMaterial = Bi.mHandleMaterial = Pb.mHandleMaterial = Cu.mHandleMaterial = Zn.mHandleMaterial = Sn.mHandleMaterial = Au.mHandleMaterial = Diamond.mHandleMaterial = Thaumium.mHandleMaterial = Wood;
			Desh.mHandleMaterial = Desh;
			Vb.mHandleMaterial = VibraniumSteel.mHandleMaterial = VibraniumSilver.mHandleMaterial = VibraniumSteel;
			Vibramantium.mHandleMaterial = Vibramantium;
			VoidMetal.mHandleMaterial = InfusedAir.mHandleMaterial = InfusedBalance.mHandleMaterial = InfusedDull.mHandleMaterial = InfusedEarth.mHandleMaterial = InfusedEntropy.mHandleMaterial = InfusedFire.mHandleMaterial = InfusedOrder.mHandleMaterial = InfusedVis.mHandleMaterial = InfusedVis.mHandleMaterial = MD.TC.mLoaded?Thaumium:DamascusSteel;
		}
	}
	
	public static class DATA {
		public static final OreDictItemData[]
		WIRES_01 = {
			OP.wireGt01.dat(Pb),
			OP.wireGt01.dat(Sn),
			OP.wireGt01.dat(TECH.AnyCopper),
			OP.wireGt01.dat(Au),
			OP.wireGt01.dat(Al),
			OP.wireGt01.dat(Pt),
			OP.wireGt01.dat(Graphene),
			OP.wireGt01.dat(Superconductor),
			OP.wireGt01.dat(Superconductor),
			OP.wireGt01.dat(Superconductor),
			OP.wireGt01.dat(Superconductor),
			OP.wireGt01.dat(Superconductor),
			OP.wireGt01.dat(Superconductor),
			OP.wireGt01.dat(Superconductor),
			OP.wireGt01.dat(Superconductor),
			OP.wireGt01.dat(Superconductor)
		},
		WIRES_04 = {
			OP.wireGt04.dat(Pb),
			OP.wireGt04.dat(Sn),
			OP.wireGt04.dat(TECH.AnyCopper),
			OP.wireGt04.dat(Au),
			OP.wireGt04.dat(Al),
			OP.wireGt04.dat(Pt),
			OP.wireGt04.dat(Graphene),
			OP.wireGt04.dat(Superconductor),
			OP.wireGt04.dat(Superconductor),
			OP.wireGt04.dat(Superconductor),
			OP.wireGt04.dat(Superconductor),
			OP.wireGt04.dat(Superconductor),
			OP.wireGt04.dat(Superconductor),
			OP.wireGt04.dat(Superconductor),
			OP.wireGt04.dat(Superconductor),
			OP.wireGt04.dat(Superconductor)
		},
		CABLES_01 = {
			OP.cableGt01.dat(Pb),
			OP.cableGt01.dat(Sn),
			OP.cableGt01.dat(TECH.AnyCopper),
			OP.cableGt01.dat(Au),
			OP.cableGt01.dat(Al),
			OP.cableGt01.dat(Pt),
			OP.wireGt01.dat(Graphene),
			OP.wireGt01.dat(Superconductor),
			OP.wireGt01.dat(Superconductor),
			OP.wireGt01.dat(Superconductor),
			OP.wireGt01.dat(Superconductor),
			OP.wireGt01.dat(Superconductor),
			OP.wireGt01.dat(Superconductor),
			OP.wireGt01.dat(Superconductor),
			OP.wireGt01.dat(Superconductor),
			OP.wireGt01.dat(Superconductor)
		},
		CABLES_04 = {
			OP.cableGt04.dat(Pb),
			OP.cableGt04.dat(Sn),
			OP.cableGt04.dat(TECH.AnyCopper),
			OP.cableGt04.dat(Au),
			OP.cableGt04.dat(Al),
			OP.cableGt04.dat(Pt),
			OP.wireGt04.dat(Graphene),
			OP.wireGt04.dat(Superconductor),
			OP.wireGt04.dat(Superconductor),
			OP.wireGt04.dat(Superconductor),
			OP.wireGt04.dat(Superconductor),
			OP.wireGt04.dat(Superconductor),
			OP.wireGt04.dat(Superconductor),
			OP.wireGt04.dat(Superconductor),
			OP.wireGt04.dat(Superconductor),
			OP.wireGt04.dat(Superconductor)
		},
		CIRCUITS = {
			OP.circuit.dat(Primitive),
			OP.circuit.dat(Basic),
			OP.circuit.dat(Good),
			OP.circuit.dat(Advanced),
			OP.circuit.dat(Elite),
			OP.circuit.dat(Master),
			OP.circuit.dat(Ultimate),
			OP.circuit.dat(Quantum),
			OP.circuit.dat(Quantum),
			OP.circuit.dat(Quantum),
			OP.circuit.dat(Quantum),
			OP.circuit.dat(Quantum),
			OP.circuit.dat(Quantum),
			OP.circuit.dat(Quantum),
			OP.circuit.dat(Quantum),
			OP.circuit.dat(Quantum)
		};
		
		
		
		public static final OreDictMaterial
		Dye_Materials[]				= {Black, Red, Green, Brown, Blue, Purple, Cyan, LightGray, Gray, Pink, Lime, Yellow, LightBlue, Magenta, Orange, White},
		
		Heat_T[]					= {TECH.AnyStone	, TECH.AnyIronSteel	, Invar				, Ti				, TungstenCarbide	, TECH.AnyTungsten	, TECH.AnyTungsten	, TECH.AnyTungsten	, TECH.AnyTungsten	, TECH.AnyTungsten	, TECH.AnyTungsten	, TECH.AnyTungsten	, TECH.AnyTungsten	, TECH.AnyTungsten	, TECH.AnyTungsten	, TECH.AnyTungsten	},
		Kinetic_T[]					= {TECH.AnyWood		, Bronze			, TECH.AnyIronSteel	, Ti				, TungstenSteel		, Ir				, Os				, Os				, Os				, Os				, Os				, Os				, Os				, Os				, Os				, Os				},
		Electric_T[]				= {TinAlloy			, SteelGalvanized	, Al				, StainlessSteel	, Cr				, Ti				, Ir				, Os				, Trinitanium		, Trinaquadalloy	, Neutronium		, Neutronium		, Neutronium		, Neutronium		, Neutronium		, Neutronium		},
		Flux_T[]					= {Sn				, Pb				, Invar				, Electrum			, EnderiumBase		, Enderium			, TungstenCarbide	, TungstenCarbide	, TungstenCarbide	, TungstenCarbide	, TungstenCarbide	, TungstenCarbide	, TungstenCarbide	, TungstenCarbide	, TungstenCarbide	, TungstenCarbide	};
	}
	
	/** The Section where I place regular Ores that are only used in advanced processing anyways due to complexity. */
	public static class OREMATS {
		public static final OreDictMaterial
		Hematite				= oredustelec	( 9104, "Hematite"					, SET_DULL				, 145,  90,  90, 255).add(MELTING, MOLTEN, MAGNETIC_PASSIVE											)							.setMcfg( 0, Fe				, 2*U, O 				, 3*U)																									.setMeltingPoint(2 * Fe.mMeltingPoint / 3).addIdenticalNames("BandedIron", "IronOxide"),
		Magnetite				= oredustelec	( 9122, "Magnetite"					, SET_METALLIC			,  30,  30,  30, 255).add(BLACKLISTED_SMELTER, MAGNETIC_PASSIVE										)							.setMcfg( 0, Fe				, 3*U, O 				, 4*U)																									.setMeltingPoint(Fe.mMeltingPoint).aspects(TC.METALLUM, 2, TC.MAGNETO, 1),
		Cassiterite				= oredustelec	( 9108, "Cassiterite"				, SET_METALLIC			, 220, 220, 220, 255).add(FURNACE																	).setSmelting(Sn, 3*U/4)	.setMcfg( 1, Sn				, 1*U, O 				, 2*U)																									.setMeltingPoint(3 * Sn.mMeltingPoint / 2),
		CassiteriteSand			= oredustelec	( 9109, "Cassiterite Sand"			, SET_SAND				, 220, 220, 220, 255).add(FURNACE																	).setSmelting(Sn, 3*U/4)	.setMcfg( 1, Sn				, 1*U, O 				, 2*U)																									.setMeltingPoint(3 * Sn.mMeltingPoint / 2),
		Garnierite				= oredustelec	( 9118, "Garnierite"				, SET_METALLIC			,  50, 200,  70, 255).add(BLACKLISTED_SMELTER, MAGNETIC_PASSIVE										).setSmelting(Ni, 3*U/4)	.setMcfg( 1, Ni				, 1*U, O 				, 1*U)																									, 
		Pyrolusite				= oredustelec	( 9126, "Pyrolusite"				, SET_DULL				, 150, 150, 170, 255).add(BLACKLISTED_SMELTER, MAGNETIC_PASSIVE										).setSmelting(Mn, 3*U/4)	.setMcfg( 1, Mn				, 1*U, O 				, 2*U)																									,
		Uraninite				= oredustelec	( 9134, "Uraninite"					, SET_METALLIC			,  35,  35,  35, 255).add(BLACKLISTED_SMELTER														).setSmelting(U_238,U/3)	.setMcfg( 1, U_238			, 1*U, O 				, 2*U)																									,
		Rutile					= oredustdcmp	( 9192, "Rutile"					, SET_METALLIC			, 110,  80, 120, 255).add(MELTING, MOLTEN															)							.setMcfg( 1, Ti				, 1*U, O 				, 2*U)																									.setStatsEnergetic(Ti.mMeltingPoint + 100, Ti.mBoilingPoint),
		Ilmenite				= oredustdcmp	( 9120, "Ilmenite"					, SET_METALLIC			,  70,  55,  50, 255).add(MELTING, MOLTEN, MAGNETIC_PASSIVE											)							.setMcfg( 0, Fe				, 1*U, Ti				, 1*U, O 				, 3*U)																			,
		Bauxite					= oredustelec	( 9105, "Bauxite"					, SET_DULL				, 200, 100,   0, 255).add(BLACKLISTED_SMELTER														).setSmelting(Al,   U/2)	.setMcfg( 0, Rutile			, 1*U, Al				,16*U, H2O				,15*U, O				, 7*U)													.setMeltingPoint(2800),
		BrownLimonite			= oredustelec	( 9106, "Brown Limonite"			, SET_METALLIC			, 200, 100,   0, 255).add(BLACKLISTED_SMELTER, MAGNETIC_PASSIVE										)							.setMcfg( 0, Fe				, 1*U, H 				, 1*U, O				, 2*U)																			.setMeltingPoint(2 * Fe.mMeltingPoint / 3), // FeO(OH)
		Celestine				= oredustelec	( 9110, "Celestine"					, SET_DULL				, 200, 205, 240, 255).add(BLACKLISTED_SMELTER														).setSmelting(Sr,   U/9)	.setMcfg( 0, Sr				, 1*U, S 				, 1*U, O				, 4*U)																			,
		Chalcopyrite			= oredustelec	( 9111, "Chalcopyrite"				, SET_DULL				, 160, 120,  40, 255).add(FURNACE																	).setSmelting(Cu, 2*U/9)	.setMcfg( 0, Cu				, 1*U, Fe				, 1*U, S				, 2*U)																			,
		Chromite				= oredustelec	( 9113, "Chromite"					, SET_METALLIC			,  35,  20,  15, 255).add(BLACKLISTED_SMELTER														).setSmelting(Cr, 2*U/9)	.setMcfg( 0, Fe				, 1*U, Cr				, 2*U, O				, 4*U)																			,
		Cinnabar				= oredustcent	( 9114, "Cinnabar"					, SET_ROUGH				, 150,   0,   0, 255).add(FURNACE																	).setSmelting(Hg,   U/3)	.setMcfg( 0, Hg				, 1*U, S 				, 1*U)																									,
		Cobaltite				= oredustelec	( 9115, "Cobaltite"					, SET_METALLIC			,  80,  80, 250, 255).add(BLACKLISTED_SMELTER, MAGNETIC_PASSIVE										).setSmelting(Co,   U/4)	.setMcfg( 0, Co				, 1*U, As				, 1*U, S				, 1*U)																			,
		Cooperite				= oredustcent	( 9116, "Cooperite"					, SET_METALLIC			, 130, 160, 230, 255).add(BLACKLISTED_SMELTER, WASHING_MERCURY										).setSmelting(Pt,   U/3)	.setMcfg( 0, Pt				, 3*U, Ni				, 1*U, S				, 1*U, Pd				, 1*U)													.setLocal("Sheldonite").addIdenticalNames("Sheldonite"),
		Galena					= oredustelec	( 9117, "Galena"					, SET_DULL				, 100,  60, 100, 255).add(FURNACE																	).setSmelting(Pb,   U/3)	.setMcfg( 0, Pb				, 3*U, Ag				, 3*U, S				, 2*U)																			,
		Magnesite				= oredustelec	( 9121, "Magnesite"					, SET_METALLIC			, 250, 250, 180, 255).add(BLACKLISTED_SMELTER														).setSmelting(Mg,   U/5)	.setMcfg( 0, Mg				, 1*U, C 				, 1*U, O 				, 3*U)																			,
		Molybdenite				= oredustelec	( 9123, "Molybdenite"				, SET_METALLIC			,  25,  25,  25, 255).add(BLACKLISTED_SMELTER														).setSmelting(Mo,   U/4)	.setMcfg( 0, Mo				, 1*U, S 				, 2*U)																									,
		Powellite				= oredustcent	( 9124, "Powellite"					, SET_DULL				, 255, 255,   0, 255).add(BLACKLISTED_SMELTER														).setSmelting(Mo,   U/9)	.setMcfg( 0, Ca				, 1*U, Mo				, 1*U, O				, 4*U)																			,
		Sphalerite				= oredustelec	( 9130, "Sphalerite"				, SET_DULL				, 222, 222,   0, 255).add(BLACKLISTED_SMELTER														).setSmelting(Zn,   U/3)	.setMcfg( 0, Zn				, 1*U, S 				, 1*U)																									,
		Stibnite				= oredustcent	( 9131, "Stibnite"					, SET_METALLIC			,  70,  70,  70, 255).add(BLACKLISTED_SMELTER														).setSmelting(Sb,   U/4)	.setMcfg( 0, Sb				, 2*U, S 				, 3*U)																									,
		Tetrahedrite			= oredustcent	( 9132, "Tetrahedrite"				, SET_DULL				, 200,  32,   0, 255).add(FURNACE																	).setSmelting(Cu,   U/4)	.setMcfg( 0, Cu				, 3*U, Sb				, 1*U, S				, 3*U, Fe				, 1*U)													, // Cu3SbS3 + x(Fe,Zn)6Sb2S9
		Scheelite				= oredustelec	( 9128, "Scheelite"					, SET_DULL				, 200, 140,  20, 255).add(BLACKLISTED_SMELTER														).setSmelting(W ,   U/9)	.setMcfg( 0, W 				, 1*U, Ca				, 1*U, O 				, 4*U)																			,
		Tungstate				= oredustelec	( 9133, "Tungstate"					, SET_DULL				,  55,  50,  35, 255).add(BLACKLISTED_SMELTER														).setSmelting(W ,   U/9)	.setMcfg( 0, W 				, 1*U, Li				, 1*U, O				, 4*U)																			,
		Stolzite				= oredustelec	( 9193, "Stolzite"					, SET_DULL				,  55,  50,  35, 255).add(BLACKLISTED_SMELTER														).setSmelting(W ,   U/9)	.setMcfg( 0, W 				, 1*U, Pb				, 1*U, O				, 4*U)																			.addIdenticalNames("Raspite"),
		Wolframite				= oredustelec	( 9194, "Wolframite"				, SET_DULL				,  55,  50,  35, 255).add(BLACKLISTED_SMELTER														).setSmelting(W ,   U/9)	.setMcfg( 0, W 				, 1*U, Fe				, 1*U, O				, 4*U)																			.addIdenticalNames("Ferberite"),
		Huebnerite				= oredustelec	( 9195, "Huebnerite"				, SET_DULL				,  55,  50,  35, 255).add(BLACKLISTED_SMELTER														).setSmelting(W ,   U/9)	.setMcfg( 0, W 				, 1*U, Mn				, 1*U, O				, 4*U)																			.addIdenticalNames("Gyubnera"),
		Russellite				= oredustelec	( 9196, "Russellite"				, SET_DULL				,  55,  50,  35, 255).add(BLACKLISTED_SMELTER														).setSmelting(Bi,   U/9)	.setMcfg( 0, W 				, 1*U, Bi				, 2*U, O				, 6*U)																			,
		Pinalite				= oredustelec	( 9197, "Pinalite"					, SET_DULL				,  55,  50,  35, 255).add(BLACKLISTED_SMELTER														).setSmelting(Pb,   U/9)	.setMcfg( 0, W 				, 1*U, Pb				, 3*U, Cl				, 2*U, O				, 5*U)													,
		Wulfenite				= oredustcent	( 9136, "Wulfenite"					, SET_DULL				, 255, 128,   0, 255).add(BLACKLISTED_SMELTER														).setSmelting(Mo,   U/9)	.setMcfg( 0, Pb				, 1*U, Mo				, 1*U, O 				, 4*U)																			,
		YellowLimonite			= oredustelec	( 9137, "Yellow Limonite"			, SET_METALLIC			, 200, 200,   0, 255).add(BLACKLISTED_SMELTER, MAGNETIC_PASSIVE										)							.setMcfg( 0, Fe				, 1*U, H 				, 1*U, O 				, 2*U)																			.setMeltingPoint(2 * Fe.mMeltingPoint / 3), // FeO(OH) + a bit Ni and Co
		Perlite					= oredustcent	( 9138, "Perlite"					, SET_DULL				,  30,  20,  30, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Obsidian		, 2*U, H2O				, 1*U)																									,
		Borax					= oredustdcmp	( 9139, "Borax"						, SET_FINE				, 250, 250, 250, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Na				, 2*U, B				, 4*U, H2O				,30*U, O				, 7*U)													,
		Chrysotile				= oredustcent	( 9142, "Chrysotile"				, SET_DULL				, 110, 140, 110, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Asbestos		, 1*U)																															,
		Ferrovanadium			= oredustcent	( 9143, "Ferrovanadium"				, SET_METALLIC			,  35,  35,  60, 255).add(BLACKLISTED_SMELTER, MAGNETIC_PASSIVE										)							.setMcfg( 0, Magnetite		, 1*U, V2O5				, 1*U)																									.setLocal("Ferrovanadium").aspects(TC.METALLUM, 2, TC.MAGNETO, 1), VanadiumMagnetite = Ferrovanadium.addIdenticalNames("Vanadium Magnetite"), // Mixture of Fe3O4 and V2O5
		Bastnasite				= oredustelec	( 9144, "Bastnasite"				, SET_FINE				, 200, 110,  45, 255).add(BLACKLISTED_SMELTER														).setSmelting(Ce,   U/9)	.setMcfg( 0, Ce				, 1*U, C				, 1*U, F				, 1*U, O				, 3*U)													, // (Ce, La, Y)CO3F
		Pentlandite				= oredustelec	( 9145, "Pentlandite"				, SET_DULL				, 165, 150,   5, 255).add(BLACKLISTED_SMELTER, MAGNETIC_PASSIVE										).setSmelting(Ni,   U/3)	.setMcfg( 0, Ni				, 9*U, S				, 8*U)																									, // (Fe,Ni)9S8
		Spodumene				= oredustelec	( 9146, "Spodumene"					, SET_DULL				, 190, 170, 170, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Li				, 1*U, Al				, 1*U, Si				, 2*U, O				, 6*U)													, // LiAl(SiO3)2
		Pollucite				= oredustelec	( 9147, "Pollucite"					, SET_DULL				, 240, 210, 210, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Cs				, 2*U, Al				, 2*U, Si				, 4*U, H2O				, 2*U, O				,12*U)							, // (Cs,Na)2Al2Si4O12 2H2O (also a source of Rb)
		Tantalite				= oredustelec	( 9148, "Tantalite"					, SET_METALLIC			, 145,  80,  40, 255).add(BLACKLISTED_SMELTER														).setSmelting(Ta,   U/9)	.setMcfg( 0, Mn				, 1*U, Ta				, 2*U, O				, 6*U)																			, // (Fe, Mn)Ta2O6 (also source of Nb)
		Lepidolite				= oredustelec	( 9149, "Lepidolite"				, SET_FINE				, 240,  50, 140, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, K				, 1*U, Li				, 3*U, Al				, 4*U, F				, 2*U, O				,10*U)							, // K(Li,Al,Rb)3(Al,Si)4O10(F,OH)2
		Glauconite				= oredustelec	( 9150, "Glauconite"				, SET_DULL				, 130, 180,  60, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, K				, 1*U, Mg				, 2*U, Al				, 4*U, H2O				, 3*U, O				,11*U)							, // (K,Na)(Fe3+,Al,Mg)2(Si,Al)4O10(OH)2
		GlauconiteSand			= oredustelec	( 9151, "Glauconite Sand"			, SET_DULL				, 130, 180,  60, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, K				, 1*U, Mg				, 2*U, Al				, 4*U, H2O				, 3*U, O				,11*U)							, // (K,Na)(Fe3+,Al,Mg)2(Si,Al)4O10(OH)2
		Vermiculite				= oredustelec	( 9152, "Vermiculite"				, SET_METALLIC			, 200, 180,  15, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Fe				, 3*U, Al				, 4*U, Si				, 4*U, H2O				, 7*U, O				,11*U)							, // (Mg+2, Fe+2, Fe+3)3 [(AlSi)4O10] (OH)2 4H2O)
		Bentonite				= oredustelec	( 9153, "Bentonite"					, SET_ROUGH				, 245, 215, 210, 255).add(BLACKLISTED_SMELTER														)							.setMcfg(33, Na				, 1*U, Mg				, 6*U, Si				,12*U, H2O				,14*U, O				,33*U)							, // (Na,Ca)0.33(Al,Mg)2(Si4O10)(OH)2 nH2O
		FullersEarth			= oredustelec	( 9154, "Fullers Earth"				, SET_FINE				, 160, 160, 120, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Mg				, 1*U, Si				, 4*U, H				, 1*U, H2O				, 4*U, O				,11*U)							, // (Mg,Al)2Si4O10(OH) 4(H2O)
		Pitchblende				= oredustcent	( 9155, "Pitchblende"				, SET_DULL				, 200, 210,   0, 255).add(BLACKLISTED_SMELTER														).setSmelting(U_238,U/5)	.setMcfg( 0, Uraninite		, 3*U, Th				, 1*U, Pb				, 1*U)																			,
		Malachite				= create		( 9156, "Malachite"					, SET_LAPIS				,   5,  95,   5, 255).add(G_GEM_ORES, ELECTROLYSER, DECOMPOSABLE, FURNACE							).setSmelting(Cu,   U/6)	.setMcfg( 0, Cu				, 2*U, C				, 1*U, H2O				, 3*U, O				, 4*U)													, // Cu2CO3(OH)2
		Mirabilite				= oredustelec	( 9157, "Mirabilite"				, SET_DULL				, 240, 250, 210, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Na				, 2*U, S				, 1*U, H2O				,10*U, O				, 4*U)													, // Na2SO4 10H2O
		Mica					= oredustelec	( 9158, "Mica"						, SET_FINE				, 195, 195, 205, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, K				, 1*U, Al				, 3*U, Si				, 3*U, F				, 2*U, O				,10*U)							, // KAl2(AlSi3O10)(F,OH)2
		Trona					= oredustelec	( 9159, "Trona"						, SET_METALLIC			, 135, 135,  95, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Na				, 3*U, C				, 2*U, H				, 1*U, H2O				, 2*U, O				, 6*U)							, // Na3(CO3)(HCO3) 2H2O
		Barite					= oredustelec	( 9160, "Barite"					, SET_DULL				, 230, 235, 255, 255).add(BLACKLISTED_SMELTER														).setSmelting(Ba,   U/9)	.setMcfg( 0, Ba				, 1*U, S				, 1*U, O				, 4*U)																			,
		Gypsum					= oredustelec	( 9161, "Gypsum"					, SET_DULL				, 230, 230, 250, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Ca				, 1*U, S				, 1*U, H2O				, 2*U, O				, 4*U)													, // CaSO4 2H2O
		Alunite					= oredustelec	( 9162, "Alunite"					, SET_METALLIC			, 225, 180,  65, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, K				, 1*U, Al				, 3*U, Si				, 2*U, H2O				, 9*U, O				,11*U)							, // KAl3(SO4)2(OH)6
		Wollastonite			= oredustelec	( 9164, "Wollastonite"				, SET_DULL				, 240, 240, 240, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Ca				, 1*U, Si				, 1*U, O				, 3*U)																			, // CaSiO3
		Zeolite					= oredustelec	( 9165, "Zeolite"					, SET_DULL				, 240, 230, 230, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Na				, 2*U, Al				, 2*U, Si				, 3*U, H2O				, 6*U, O				,10*U)							,
		Kyanite					= oredustelec	( 9166, "Kyanite"					, SET_FLINT				, 110, 110, 250, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Al				, 2*U, Si				, 1*U, O				, 5*U)																			, // Al2SiO5
		Kaolinite				= oredustelec	( 9167, "Kaolinite"					, SET_DULL				, 245, 235, 235, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Al				, 2*U, Si				, 2*U, H2O				, 6*U, O				, 7*U)													, // Al2Si2O5(OH)4
		DiduraniumTrioxide		= oredustelec	( 9198, "Diduranium Trioxide"		, SET_DULL				,  45, 145, 145, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Dn				, 2*U, O				, 3*U)																									,
		DuraniumHexafluoride	= oredustelec	( 9199, "Duranium Hexafluoride"		, SET_DULL				,  25, 175, 125, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Dn				, 1*U, F				, 6*U)																									,
		DuraniumHexachloride	= oredustelec	( 9200, "Duranium Hexachloride"		, SET_DULL				,  75, 175, 145, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Dn				, 1*U, Cl				, 6*U)																									,
		DuraniumHexabromide		= oredustelec	( 9201, "Duranium Hexabromide"		, SET_DULL				,  45, 125, 175, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Dn				, 1*U, Br				, 6*U)																									,
		DuraniumHexaiodide		= oredustelec	( 9202, "Duranium Hexaiodide"		, SET_DULL				,  75, 125, 175, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Dn				, 1*U, I				, 6*U)																									,
		DuraniumHexaastatide	= oredustelec	( 9203, "Duranium Hexaastatide"		, SET_DULL				,  25, 145, 175, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Dn				, 1*U, At				, 6*U)																									,
		TritaniumDioxide		= oredustelec	( 9204, "Tritanium Dioxide"			, SET_DULL				,  25, 185, 125, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Tn				, 1*U, O				, 2*U)																									,
		TritaniumHexafluoride	= oredustelec	( 9205, "Tritanium Hexafluoride"	, SET_DULL				,  85, 125, 125, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Tn				, 1*U, F				, 6*U)																									,
		TritaniumHexachloride	= oredustelec	( 9206, "Tritanium Hexachloride"	, SET_DULL				,  55, 185, 155, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Tn				, 1*U, Cl				, 6*U)																									,
		TritaniumHexabromide	= oredustelec	( 9207, "Tritanium Hexabromide"		, SET_DULL				,  55, 125, 155, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Tn				, 1*U, Br				, 6*U)																									,
		TritaniumHexaiodide		= oredustelec	( 9208, "Tritanium Hexaiodide"		, SET_DULL				,  85, 185, 185, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Tn				, 1*U, I				, 6*U)																									,
		TritaniumHexaastatide	= oredustelec	( 9209, "Tritanium Hexaastatide"	, SET_DULL				,  25, 125, 185, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Tn				, 1*U, At				, 6*U)																									,
		Bromargyrite			= oredustelec	( 9210, "Bromargyrite"				, SET_DULL				,  90,  45,  10, 255).add(BLACKLISTED_SMELTER														).setSmelting(Ag,   U/3)	.setMcfg( 0, Ag				, 1*U, Br				, 1*U)																									,
		Smithsonite				= oredustelec	( 9211, "Smithsonite"				, SET_DULL				, 110, 223, 210, 255).add(BLACKLISTED_SMELTER														).setSmelting(Zn,   U/6)	.setMcfg( 0, Zn				, 1*U, C				, 1*U, O				, 3*U)																			,
		Sperrylite				= oredustelec	( 9212, "Sperrylite"				, SET_SHINY				, 105, 105, 105, 255).add(BLACKLISTED_SMELTER														).setSmelting(Pt,   U/4)	.setMcfg( 0, Pt				, 1*U, As				, 2*U)																									,
		Kesterite				= oredustelec	( 9213, "Kesterite"					, SET_DULL				, 105, 155, 105, 255).add(BLACKLISTED_SMELTER														).setSmelting(Cu,   U/9)	.setMcfg( 0, Cu				, 2*U, Zn				, 1*U, Sn				, 1*U, S				, 4*U)													,
		Stannite				= oredustelec	( 9214, "Stannite"					, SET_METALLIC			, 155, 145,  55, 255).add(BLACKLISTED_SMELTER														).setSmelting(Cu,   U/9)	.setMcfg( 0, Cu				, 2*U, Fe				, 1*U, Sn				, 1*U, S				, 4*U)													,
		Fluorite				= oredustdcmp	( 9215, "Fluorite"					, SET_RUBY				,  50, 255,  50, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Ca				, 1*U, F				, 2*U)																									,
		Arsenopyrite			= oredustelec	( 9216, "Arsenopyrite"				, SET_CUBE_SHINY		, 250, 240,  30, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Fe				, 1*U, As				, 1*U, S				, 1*U)																			,
		Pumice					= oredustcent	( 9000, "Pumice"					, SET_DULL				, 230, 185, 185, 255).add(BLACKLISTED_SMELTER														)			  				.setMcfg( 0, Stone			, 1*U)																															,
		Diatomite				= oredustcent	( 9001, "Diatomite"					, SET_DULL				, 225, 225, 225, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Flint			, 8*U, Hematite			, 1*U, BlueSapphire		, 1*U)																			,
		BasalticMineralSand		= oredustcent	( 9003, "Basaltic Mineral Sand"		, SET_SAND				,  40,  50,  40, 255).add(BLACKLISTED_SMELTER, MAGNETIC_PASSIVE										)							.setMcfg( 0, Magnetite		, 1*U, Basalt			, 1*U)																									.aspects(TC.METALLUM, 2, TC.MAGNETO, 1),
		GraniticMineralSand		= oredustcent	( 9004, "Granitic Mineral Sand"		, SET_SAND				,  40,  60,  60, 255).add(BLACKLISTED_SMELTER, MAGNETIC_PASSIVE										)							.setMcfg( 0, Magnetite		, 1*U, GraniteBlack		, 1*U)																									.aspects(TC.METALLUM, 2, TC.MAGNETO, 1),
		GarnetSand				= oredustcent	( 9005, "Garnet Sand"				, SET_SAND				, 200, 100,   0, 255).add(BLACKLISTED_SMELTER														)							.setMcfg( 0, Almandine		, 2*U, Andradite		, 2*U, Grossular		, 2*U, Pyrope			, 2*U, Spessartine		, 2*U, Uvarovite		, 1*U)	,
		QuartzSand				= oredustcent	( 9006, "Quartz Sand"				, SET_SAND				, 200, 200, 200, 255).add(BLACKLISTED_SMELTER, QUARTZ												).setSmelting(SiO2, U/3)	.setMcfg( 0, CertusQuartz	, 1*U, Quartzite	, 1*U)																										;
	}
	
	/** The "I don't care" Section, everything I don't want to do anything with right now, is right here. Just to make the Material Finder shut up about them. But I do see potential uses in some of these Materials. */
	public static class UNUSED {
		public static final OreDictMaterial
		OsmiumTetroxide				= unused		("Osmium Tetroxide"				).setMcfg( 0, Os, 1*U, O, 4*U),
		SodiumPeroxide				= unused		("Sodium Peroxide"				),
		IridiumSodiumOxide			= unused		("Iridium Sodium Oxide"			),
		SolutionBlueVitriol			= unused		("SolutionBlueVitriol"			).setLocal("Blue Vitriol Solution"),
		SolutionNickelSulfate		= unused		("SolutionNickelSulfate"		).setLocal("Nickel Sulfate Solution").addIdenticalNames("SolutionNickelSulphate"),
		Iridiron					= unused		("IridiumIron"					).add(G_INGOT).setMcfg( 0, Ir, 1*U, Fe, 1*U).setLocal("Iridiron"),
		IridironReinforced			= unused		("IridiumIronReinforced"		).add(G_INGOT).setMcfg( 0, Ir, 1*U, Fe, 1*U).setLocal("Reinforced Iridiron"),
		Quicklime					= unused		("Quicklime"					).add(G_DUST).setMcfg( 0, Ca, 1*U, O, 1*U),
		LimePure					= unused		("LimePure"				   		).setLocal("Pure Lime"),
		Prismarine					= unused		("Prismarine"					).setOriginalMod(MD.MC),
		TNT							= unused		("TNT"					   		).add(EXPLOSIVE, FLAMMABLE).aspects(TC.PERDITIO, 3, TC.IGNIS, 1).setOriginalMod(MD.MC),
		TerrasteelAlloyRaw			= unused		("TerrasteelAlloyRaw"			).add(G_INGOT, MAGICAL).setLocal("Raw Terrasteel Alloy").addIdenticalNames("RawTerrasteelAlloy"),
		TerrasteelAlloyStrengthened	= unused		("TerrasteelAlloyStrengthened"	).add(G_INGOT, MAGICAL).setLocal("Strengthened Terrasteel Alloy").addIdenticalNames("StrengthenedTerrasteelAlloy"),
		Vis							= unused		("Vis"							).add(DECOMPOSABLE).setMcfg( 0, Ma, 1*U).aspects(TC.AURAM, 2, TC.PRAECANTIO, 1),
		Fluix						= unused		("Fluix"						).add(CRYSTAL, BRITTLE, QUARTZ, CRYSTALLISABLE), // AE
		Unstable					= unused		("Unstable" 					).add(AUTO_BLACKLIST).aspects(TC.PERDITIO, 4).addIdenticalNames("Unstableingot"),
		Moonstone					= unused		("Moonstone"					).add(G_GEM_ORES).aspects(TC.VITREUS, 1, TC.ALIENIS, 1).setOriginalMod(MD.ARS).visDefault(),
		Sunstone					= unused		("Sunstone"						).add(G_GEM_ORES).aspects(TC.VITREUS, 1, TC.ALIENIS, 1).setOriginalMod(MD.ARS).visDefault(),
		Chimerite					= unused		("Chimerite"					).add(G_GEM_ORES).setOriginalMod(MD.ARS).visDefault(),
		Voidstone					= unused		("Voidstone"					).aspects(TC.VITREUS, 1, TC.VACUOS, 1),
		Mercassium					= unused		("Mercassium"					).setQuality( 6.0F,	 64,  1).add(G_INGOT_ORES),
		Osmonium					= unused		("Osmonium"						).setQuality( 6.0F,	 64,  1).add(G_INGOT_ORES),
		Phoenixite					= unused		("Phoenixite"					).setQuality( 6.0F,	 64,  1).add(G_INGOT_ORES),
		Antimatter					= unused		("Antimatter"					).add(ANTIMATTER),
		Starconium					= unused		("Starconium"					).add(G_INGOT_ORES),
		Thyrium						= unused		("Thyrium"						).add(G_INGOT_ORES),
		Zectium						= unused		("Zectium"						).add(G_INGOT_ORES),
		Draconic					= depricated	("Draconic"						).add(G_DUST),
		IrridantUranium				= unused		("Irridant Uranium"				).add(G_INGOT),
		IrridantReinforced			= unused		("IrridantReinforced"			).add(G_INGOT),
		IronSharp					= unused		("IronSharp"					).add(G_INGOT).setLocal("Sharp Iron"),
		ObsidianFlux				= unused		("Obsidian Flux"				).add(G_INGOT),
		CrystalFlux					= unused		("Crystal Flux"					).add(G_GEM, CRYSTAL, BRITTLE),
		Mimichite					= unused		("Mimichite"					).add(G_GEM_ORES, CRYSTAL, BRITTLE),
		Infernal					= unused		("Infernal"						),
		Invisium					= unused		("Invisium"						).add(G_DUST),
		Lodestone					= unused		("Lodestone"					).add(G_DUST_ORES),
		Luminite					= unused		("Luminite"						).add(G_DUST_ORES),
		Magma						= unused		("Magma"						),
		Mawsitsit					= unused		("Mawsitsit"					).add(G_DUST),
		Meteorite					= unused		("Meteorite"					).add(G_DUST_ORES),
		Nether						= unused		("Nether"						),
		Onyx						= unused		("Onyx"							).add(G_DUST),
		Painite						= unused		("Painite"						),
		Petroleum					= unused		("Petroleum"					).add(G_DUST_ORES),
		Pewter						= unused		("Pewter"						),
		Potash						= unused		("Potash"						),
		Randomite					= unused		("Randomite"					).add(G_DUST_ORES),
		RyuDragonRyder				= unused		("RyuDragonRyder"				),
		Sugilite					= unused		("Sugilite"						).add(G_DUST),
		Tar							= unused		("Tar"							),
		Tapazite					= unused		("Tapazite"						).add(G_DUST),
		Tourmaline					= unused		("Tourmaline"					).add(G_DUST),
		Turquoise					= unused		("Turquoise"					).add(G_DUST),
		Wimalite					= unused		("Wimalite"						).add(G_DUST_ORES),
		Yellorite					= unused		("Yellorite"					).add(G_DUST_ORES),
		Adamite						= unused		("Adamite"						).add(G_DUST_ORES),
		Adluorite					= unused		("Adluorite"					).add(G_DUST_ORES),
		Agate						= unused		("Agate"						).add(G_DUST),
		Ammonium					= unused		("Ammonium"						).add(G_DUST),
		Bitumen						= unused		("Bitumen"						).add(G_DUST_ORES),
		Bloodstone					= unused		("Bloodstone"					).add(G_DUST),
		Citrine						= unused		("Citrine"						).add(G_DUST),
		Coral						= unused		("Coral"						).add(G_DUST),
		Chrysocolla					= unused		("Chrysocolla"					).add(G_DUST),
		DarkStone					= unused		("Dark Stone"					).add(G_DUST),
		Demonite					= unused		("Demonite"						).add(G_DUST),
		Energized					= unused		("Energized"					),
		Reinforced					= unused		("Reinforced"					),
		Mud							= unused		("Mud"					   		),
		Cluster						= unused		("Cluster"				   		),
		Sweet						= unused		("Sweet"						),
		Gelatine					= unused		("Gelatine"						),
		TarPitch					= unused		("Tar Pitch"					),
		Satinspar					= unused		("Satinspar"					),
		Selenite					= unused		("Selenite"						),
		PetrifiedWood				= unused		("Petrified Wood"				),
		Jet							= unused		("Jet"							),
		Microcline					= unused		("Microcline"					),
		Serpentine					= unused		("Serpentine"					),
		Sylvite						= unused		("Sylvite"						),
		Flux						= unused		("Flux"							),
		Goshen						= unused		("Goshen"						),
		Joshen						= unused		("Joshen"						),
		Itarius						= unused		("Itarius"						),
		Legendary					= unused		("Legendary"					),
		MutatedIron					= unused		("Mutated Iron"					),
		Witheria					= unused		("Witheria"						),
		RubberTreeSap				= unused		("Rubber Tree Sap"				),
		Endium						= unused		("Endium"						),
		GraveyardDirt				= unused		("Graveyard Dirt"				),
		Cocaine						= unused		("Cocaine"						),
		Vile						= unused		("Vile"							),
		Dull						= unused		("Dull"							),
		Dark						= unused		("Dark"							),
		Soulium						= unused		("Soulium"						),
		Tennantite					= unused		("Tennantite"					),
		Alfium						= unused		("Alfium"						),
		Ryu							= unused		("Ryu"							),
		Mutation					= unused		("Mutation"						),
		EnrichedCopper				= unused		("Enriched Copper"				),
		DiamondCopper				= unused		("Diamond Copper"				),
		Fairy						= unused		("Fairy"						),
		Ludicrite					= unused		("Ludicrite"					),
		Pokefennium					= unused		("Pokefennium"					);
	}
}